//
//  DietData.m
//  MoveCoachMobile
//
//  Created by Alex Morgan on 7/18/13.
//

#import "DietData.h"
//#import "LocalNotification.h"
#import "UserData.h"
#import "Utilities.h"

@implementation DietData

@synthesize foodName, dateCreated, referenceDate,caloriesEaten,foodCode,portionFactor, portionDesc,meal,servings;

- (id)initWithFoodName:(NSString *)foodname date:(NSString *)date calories:(float)cal foodCode:(NSString *)foodcode portionFactor:(float)portionfactor portionDesc:(NSString *)portiondesc servings:(float)serv meal:(NSString *)mymeal
{
    return [self initWithFoodName:foodname date:date referenceDate:nil calories:cal foodCode:foodcode portionFactor:portionFactor portionDesc:portiondesc servings:serv meal:mymeal];
}

- (id)initWithFoodName:(NSString *)foodname date:(NSString *)date referenceDate:(NSDate *)refDate calories:(float)cal foodCode:(NSString *)foodcode portionFactor:(float)portionfactor portionDesc:(NSString *)portiondesc servings:(float)serv meal:(NSString *)mymeal
{
    self = [super init];
    
    if (self){
        self.foodName = foodname;
        self.dateCreated = date;
        if (refDate) {
            self.referenceDate = refDate;
        } else {
            NSDateFormatter *df = [[Utilities defaultUtilities]getDateFormatter];
            [df setDateStyle:NSDateFormatterShortStyle];
            [df setTimeStyle:NSDateFormatterNoStyle];
            NSDate *indt = [df dateFromString:date];
            NSDate *now = [NSDate date];
            if ([[Utilities defaultUtilities]isSameDay:indt as: now]) {
                self.referenceDate = now;
            } else {
                self.referenceDate = indt;
            }
        }
        self.caloriesEaten = cal;
        self.foodCode = foodcode;
        self.portionFactor = portionfactor;
        self.portionDesc = portiondesc;
        self.meal = mymeal;
        self.servings = serv;
        
        [self addItemToCalorieDictionary];
    }
    //[[LocalNotification notificationManager] updateEntries:@"diet"];
    return self;
}

//this method adds the calorie data for a specific date and meal to a hash stored in userData model.  
- (void) addItemToCalorieDictionary {
    
    if (!self.referenceDate) {
        self.referenceDate = [[NSDate alloc]init];
    }
    
    NSDate *keyDate = [[Utilities defaultUtilities] getDateConfiguredAsKey:self.referenceDate];
    
    //initialize the array that will hold the total calorie values for each meal.  [0]=breakfastCals, [1]=lunchCals, [2]=dinnerCals, [3]=snackCals, [4...inf]=self (the dietData object itself is also stored in the array so we can quickly access what foods were eaten on each day.  
    NSNumber *zeroNumber = [NSNumber numberWithInt:0];
    NSMutableArray *calorieValues;
    
    //figure out which meal we will be working with
    int index = [self getIndexForMeal];
    
    //create or update the hash data
    if ([[[UserData dataStore] caloriesByDateDictionary] objectForKey:keyDate] == nil) {
        calorieValues = [NSMutableArray arrayWithObjects:zeroNumber, zeroNumber, zeroNumber, zeroNumber, self, nil];
        [calorieValues setObject:[NSNumber numberWithFloat:self.caloriesEaten] atIndexedSubscript:index];
        [[[UserData dataStore] caloriesByDateDictionary] setObject:calorieValues forKey:keyDate];
        
    } else {
        calorieValues = [[[UserData dataStore] caloriesByDateDictionary] objectForKey:keyDate];
        float currentCalorieValue = ((NSNumber *)[calorieValues objectAtIndex:index]).floatValue;
        currentCalorieValue += self.caloriesEaten;
        [calorieValues setObject:[NSNumber numberWithFloat:currentCalorieValue] atIndexedSubscript:index];
        
        [calorieValues addObject:self];
        
        [[[UserData dataStore] caloriesByDateDictionary] setObject:calorieValues forKey:keyDate];
    }
}

- (void) removeFromCalorieDictionary {
    //get the array from the hash
    NSDate *keyDate = [[Utilities defaultUtilities] getDateConfiguredAsKey:self.referenceDate];
    
    NSMutableArray *calorieValues = [[[UserData dataStore] caloriesByDateDictionary] objectForKey:keyDate];
    
    int index = [self getIndexForMeal];
    if (calorieValues) {
        float currentCalorieValue = ((NSNumber *)[calorieValues objectAtIndex:index]).floatValue;
        currentCalorieValue -= self.caloriesEaten;
        [calorieValues setObject:[NSNumber numberWithFloat:currentCalorieValue] atIndexedSubscript:index];
        
        [calorieValues removeObject:self];
        
        [[[UserData dataStore] caloriesByDateDictionary] setObject:calorieValues forKey:keyDate];
    }
    
}


- (int) getIndexForMeal {
    int index = 0;
    if ([self.meal isEqualToString:@"Breakfast"]) {
        index=0;
    } else if ([self.meal isEqualToString:@"Lunch"]) {
        index=1;
    } else if ([self.meal isEqualToString:@"Dinner"]) {
        index=2;
    } else if ([self.meal isEqualToString:@"Snack"]) {
        index=3;
    }
    return index;
}

- (void) encodeWithCoder:(NSCoder *)aCoder
{
    [aCoder encodeObject:foodName forKey:@"foodName"];
    [aCoder encodeObject:dateCreated forKey:@"dateCreated"];
    [aCoder encodeObject:referenceDate forKey:@"referenceDate"];
    [aCoder encodeFloat:caloriesEaten forKey:@"calories"];
    [aCoder encodeObject:foodCode forKey:@"foodCode"];
    [aCoder encodeFloat:portionFactor forKey:@"portionFactor"];
    [aCoder encodeObject:portionDesc forKey:@"portionDesc"];
    [aCoder encodeObject:meal forKey:@"meal"];
    [aCoder encodeFloat:servings forKey:@"servings"];
}

- (id) initWithCoder:(NSCoder *)aDecoder
{
    self = [super init];
    
    if (self) {
        [self setDateCreated:[aDecoder decodeObjectForKey:@"dateCreated"]];
        [self setReferenceDate:[aDecoder decodeObjectForKey:@"referenceDate"]];
        [self setFoodName:[aDecoder decodeObjectForKey:@"foodName"]];
        [self setFoodCode:[aDecoder decodeObjectForKey:@"foodCode"]];
        [self setCaloriesEaten:[aDecoder decodeFloatForKey:@"calories"]];
        [self setPortionFactor:[aDecoder decodeFloatForKey:@"portionFactor"]];
        [self setPortionDesc:[aDecoder decodeObjectForKey:@"portionDesc"]];
        [self setMeal:[aDecoder decodeObjectForKey:@"meal"]];
        [self setServings:[aDecoder decodeFloatForKey:@"servings"]];
    }
    return self;
}

@end
