//
//  Goal.h
//  MoveCoachMobile
//
//  Created by Joel Russell on 9/18/13.
//

#import <Foundation/Foundation.h>

#define ARCHIVE_THRESHOLD 48 * 60 * 60

/**
 *  This Class is part of the Data Model and represents the base class for a single instance of a goal.  The UserData Class maintains arrays of WeightGoal, Activity Goal and DietGoal objects to provide a history of each.
 */
@interface Goal : NSObject <NSCoding>

/**
 *  This property represents the date associated with the goal (usually the date the goal was entered into the system)
 */
@property (nonatomic, strong)NSDate *date;

/**
 * This property is the User's statement of their first Goal of a particular type.
 */
@property (nonatomic, strong)NSString *goal1;

/**
 * This property is the User's statement of their second Goal of a particular type.
 */
@property (nonatomic, strong)NSString *goal2;

/**
 * This Boolean property is set to YES when any of the other properties of the Goal is modified during a session.
 */
@property (nonatomic,getter=isModified)BOOL modified;

/**
 * This is the estimated completion date for the current goal, as inferred from the weightLossRate and the totalWeightLossGoal.
 */
@property (nonatomic)NSDate *expectedGoalCompletionDate;

/**
 * This is the identifier that is used to find the event related to this goal in the native device calendar
 */
@property (nonatomic, strong)NSString *calendarEventIdentifier;



/**
 *  This is the primary method to instantiate a goal object with the statement of the first goal of the pair. The date will be set to the current date.
 *
 *  @param goal1Text - The text for goal1
 */
- (id)initWithGoal1Text:(NSString *)goal1Text;

/**
 *  This method allows a goal object to be created with a specific pair of goal statements and date. This is currently only used in the test harness to auto populate weight goal data into the past.
 *
 *  @param goal1Text - The text for goal1
 *
 *  @param goal2Text - The text for goal2
 *
 *  @param aDate - a specific date to set date to.
 */
- (id)initWithGoal1Text:(NSString *)goal1Text andGoal2Text:(NSString *)goal2Text andDate:(NSDate *)aDate;

/**
 * This initializer allows instantiation of a new Goal based on a prior Goal.
 *
 * @param aGoal - prior Goal
 */
- (id)initWithGoal:(Goal *)aGoal;

/**
 * This Boolean method is used to determine whether a weight goal is ready for archiving when a change occurs. If the modified goal is older than the ARCHIVE_THRESHOLD, isArchivable will return YES.
 */
- (BOOL)isArchivable;

@end
