//
//  GuideTimer.h
//  MoveCoachMobile
//
//  Created by Alex Morgan on 8/29/12.
//

#import <Foundation/Foundation.h>
#import "Guide.h"

@interface GuideTimer : NSObject <NSCoding>
{
    NSMutableDictionary *guideTracker;
    NSMutableDictionary *chapterTracker;
    NSString *currentActiveChapter;
    NSArray *guideTitles;
}

/**
 *  This NSDictionary keeps track of the guides.  Key=NSNumber guideNumber, value=Guide Object
 *
 */
@property (nonatomic,strong) NSMutableDictionary *guideTracker;

/**
 *  This NSDictionary keeps track of which chapters have been completed.  Key=NSString uniqueChapterCode,   Value=NSString status
 *
 *  chapterCode is computed as @"<guide#><<tableSection#><tableRow#>"  Example: Guide 1 section 2 row 3 would be @"123"
 */
@property (nonatomic,strong) NSMutableDictionary *chapterTracker;

/**
 *  This NSString keeps track of the currently active chapter using a unique string identifier
 *
 */
@property (nonatomic,strong) NSString *currentActiveChapter;


+ (GuideTimer *)defaultStore;

//+ (void) reset;

- (BOOL) areGuidesStarted;

- (BOOL)isGuideActive:(int)guideIndex;

//- (BOOL)isGuideComplete:(int)guideIndex;

//- (BOOL)canGuideBeMadeActive:(int)guideIndex;

//- (void)showAlert:(int)guideIndex;

//these methods will set and query a data structure to determine if the specific chapter has been completed in the specific guide.  This is based on Guide number, table section number and table row within that section: these are encoded as a unique string @"<guide#><<tableSection#><tableRow#>", so Guide 1 section 2 row 3 would be @"123"
//- (void) setChapterAsCompleteForGuide:(NSString *)guideChapterKey andNextChapter:(NSString *)nextChapterKey;
- (BOOL) isChapterCompleteInGuide:(NSString *)guideChapterKey;
- (BOOL) isChapterActiveInGuide:(NSString *)guideChapterKey;

- (int) getCurrentActiveGuide;
//- (void) createNotificationForCurrentGuide;

//write guide status info to disk
- (NSString *) guideArchivePath;
//- (BOOL) saveChanges;

//make guide titles available publicly for reuse
- (NSString *) getGuideTitleForIndex:(int)index;


//TEST DATA POPULATIONG
//- (void) setGuideDurations;
@property (nonatomic) int oneWeek;

@property (nonatomic)BOOL aboutSMGDone;

@property (nonatomic) NSUInteger currentGuideBeingViewed;

/**
 * This Boolean method restarts all Guides for the user. It can only be executed after all Guides have been completed for a given pass through the Self-Management Guides.
 * @return YES if successful
 */
//- (BOOL)restartGuides;

@end
