//
//  ImageStore.m
//  MoveCoachMobile
//
//  Created by Alex Morgan on 4/1/13.
//

#import "ImageStore.h"

@implementation ImageStore

@synthesize dictionary,imageArray,userArchiveFilePath;

+ (id)allocWithZone:(NSZone *)zone
{
    return [self defaultImageStore];
}

+ (ImageStore *)defaultImageStore
{
    static ImageStore *defaultImageStore = nil;
    if (!defaultImageStore) {
        defaultImageStore = [[super allocWithZone:NULL]init];
        
        NSArray *documentDirectories = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
        NSString *documentDirectory = [documentDirectories objectAtIndex:0];  //need to check and make sure that the userdata files are not overwritten.
        
        NSString *archivePath = [documentDirectory stringByAppendingPathComponent:@"images.archive"];
        Boolean archiveExists = [[NSFileManager defaultManager] fileExistsAtPath:archivePath];
        
        if (archiveExists) {
            defaultImageStore = [[NSKeyedUnarchiver unarchiveObjectWithFile:archivePath] init];
        } else {
            defaultImageStore = [[super allocWithZone:nil] init];
        }
        //grab the archive file path so we can use it when writing to disk
        [defaultImageStore setUserArchiveFilePath:archivePath];
    }
    
    return defaultImageStore;
}

- (id)init
{
    self = [super init];
    if (self) {
        if (!self.dictionary){
            self.dictionary = [[NSMutableDictionary alloc]init];
        }
        
        if(!self.imageArray){
            self.imageArray = [[NSMutableArray alloc]init];
        }
    }
    
    return self;
}

-(void)setImage:(UIImage *)i forKey:(NSString *)s
{
    [self.dictionary setObject:i forKey:s];
    
}

-(UIImage *)imageForKey:(NSString *)s
{
    return [self.dictionary objectForKey:s];
}

-(void)deleteImageForKey:(NSString *)s
{
    if (!s) {
        return;
    }else {
        [self.dictionary removeObjectForKey:s];
    }
    
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
    [aCoder encodeObject:imageArray forKey:@"imageArray"];
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
    self = [super init];
    
    if (self) {
        [self setImageArray:[aDecoder decodeObjectForKey:@"imageArray"]];
    }
    return self;
}

- (BOOL) saveState {
    
    return [NSKeyedArchiver archiveRootObject:self toFile:[self userArchiveFilePath]];
}

@end
