//
//  PAData.m
//  MoveCoachMobile
//
//  Created by Alex Morgan on 8/15/13.
//

#import "PAData.h"
//#import "LocalNotification.h"
#import "UserData.h"

@implementation PAData

@synthesize paName, paCode, timeMin, caloriesBurned, dateCreated, referenceDate, paCategory, METValue,intensityLevel;

- (id)initWithPAName:(NSString *)name paCode:(NSString *)code timeMin:(NSString *)time caloriesBurned:(float)calories dateCreated:(NSString *)dateStr category:(NSString *)category
{
    return [self initWithPAName:name paCode:code timeMin:time caloriesBurned:calories dateCreated:dateStr referenceDate:nil category:category];
}

- (id)initWithPAName:(NSString *)name paCode:(NSString *)code timeMin:(NSString *)time caloriesBurned:(float)calories dateCreated:(NSString *)dateStr referenceDate:(NSDate*)refDate category:(NSString *)category
{
    self = [super init];
    
    if (self){
        self.paName = name;
        self.paCode = code;
        self.timeMin = time;
        self.paCategory = category;
        self.dateCreated = dateStr;
        if (refDate) {
            self.referenceDate = refDate;
        } else {
            NSDateFormatter *df = [[Utilities defaultUtilities]getDateFormatter];
            [df setDateStyle:NSDateFormatterShortStyle];
            [df setTimeStyle:NSDateFormatterNoStyle];
            NSDate *indt = [df dateFromString:dateStr];
            NSDate *now = [NSDate date];
            if ([[Utilities defaultUtilities]isSameDay:indt as: now]) {
                self.referenceDate = now;
            } else {
                self.referenceDate = indt;
            }
        }
        self.caloriesBurned = calories;
        
        self.METValue = (1000 * self.caloriesBurned) / (7.937867 * [[UserData dataStore]getCurrentWeight] * self.timeMin.floatValue);
        
        if (self.METValue < 3){
            self.intensityLevel = @"Lite";
        }else if (self.METValue < 6){
            self.intensityLevel = @"Moderate";
        }else{
            self.intensityLevel = @"Vigorous";
        }
    }

   // [[LocalNotification notificationManager] updateEntries:@"physicalActivity"];

    return self;
}

- (float)caloriesPerHour
{
    return ((self.caloriesBurned / [self.timeMin floatValue]) * 60);
}

- (void) encodeWithCoder:(NSCoder *)aCoder
{
    [aCoder encodeObject:paName forKey:@"paName"];
    [aCoder encodeObject:paCode forKey:@"paCode"];
    [aCoder encodeObject:timeMin forKey:@"timeMin"];
    [aCoder encodeObject:paCategory forKey:@"paCategory"];
    [aCoder encodeObject:dateCreated forKey:@"dateCreated"];
    [aCoder encodeObject:referenceDate forKey:@"referenceDate"];
    [aCoder encodeFloat:caloriesBurned forKey:@"caloriesBurned"];
    [aCoder encodeObject:intensityLevel forKey:@"intensityLevel"];
    [aCoder encodeFloat:METValue forKey:@"METValue"];
}

- (id) initWithCoder:(NSCoder *)aDecoder
{
    self = [super init];
    
    if (self) {
        [self setPaName:[aDecoder decodeObjectForKey:@"paName"]];
        [self setPaCode:[aDecoder decodeObjectForKey:@"paCode"]];
        [self setTimeMin:[aDecoder decodeObjectForKey:@"timeMin"]];
        [self setPaCategory:[aDecoder decodeObjectForKey:@"paCategory"]];
        [self setDateCreated:[aDecoder decodeObjectForKey:@"dateCreated"]];
        [self setReferenceDate:[aDecoder decodeObjectForKey:@"referenceDate"]];
        [self setCaloriesBurned:[aDecoder decodeFloatForKey:@"caloriesBurned"]];
        
        [self setIntensityLevel:[aDecoder decodeObjectForKey:@"intensityLevel"]];
        [self setMETValue:[aDecoder decodeFloatForKey:@"METValue"]];
    }
    return self;
}


@end
