//
//  SurveyData.m
//  MoveCoachMobile
//
//  Created by Alex Morgan on 11/28/12.
//

#import "SurveyData.h"

@implementation SurveyData

@synthesize familyWeight,supportExpected,weightControlImportance, changeConfidence, mealsAndSnacks, consumeSugarDrinks, consumeAlcohol,activityTime,activityDays;

@synthesize surveyIssuesArray;

- (id)init
{
    self = [super init];
    if (self){
        surveyIssuesArray = [[NSMutableArray alloc] init];
    }    
    return self;
}

/*
 This method takes an 'issue' string from the survey list and checks to see if it is already in the surveyIssuesArray.  If it is, then it is removed.  Otherwise the issue String is added to the array
 */
- (void)toggleIssue:(NSString *)str
{
    //Using a boolean to check if this issue is found in the array.  We cannot add/remove elements in the array while it is being fast enumerated
    BOOL isFound = false;
    
    //account for initial empty array
    if ([surveyIssuesArray count] == 0) {
        [surveyIssuesArray addObject:str];

    } else {
        
        for (NSString *s in surveyIssuesArray) {
            if ([s isEqualToString:str]) {
                isFound = YES;
            }
        }
        if (isFound) {
            [surveyIssuesArray removeObject:str];
        } else {
            [surveyIssuesArray addObject:str];
        }
    }    
}

- (void) removeAllIssues {
    
    [surveyIssuesArray removeAllObjects];
    
}


// NEEDS TO BE REFACTORED: change question to NSString and pull section header text.
- (void)selectedAnswer:(NSString *)str question:(int)question
{
    if (question == 1) {
        [self setFamilyWeight:str];
    }else if (question == 2) {
        [self setSupportExpected:str];
    }
}

- (void)guide1GetReadySelectedAnswer:(NSString *)str question:(int)question
{
    if (question == 2) {
        [self setMealsAndSnacks:str];
    }else if (question == 3) {
        [self setConsumeSugarDrinks:str];
    }else if (question == 4) {
        [self setConsumeAlcohol:str];
    }
}

- (void)sliderAnswer:(float)num question:(NSString *)question
{
    if ([question isEqual: @"How important is controlling your weight to you personally?"]) {
        [self setWeightControlImportance:num];
    }else if ([question isEqual: @"How confident are you that you can successfully change your eating and physical activity to control your weight?"]){
        [self setChangeConfidence:num];
    }
}

//Add support for encoding Survey data for archiving to disk
- (void) encodeWithCoder:(NSCoder *)aCoder {
        
    [aCoder encodeObject:surveyIssuesArray forKey:@"surveyIssuesArray"];
    [aCoder encodeObject:familyWeight forKey:@"familyWeight"];
    [aCoder encodeObject:supportExpected forKey:@"supportExpected"];
    [aCoder encodeObject:activityTime forKey:@"activityTime"];
    
    [aCoder encodeFloat:weightControlImportance forKey:@"weightControlImportance"];
    [aCoder encodeFloat:activityDays forKey:@"activityDays"];
    
    [aCoder encodeFloat:changeConfidence forKey:@"changeConfidence"];
    
    [aCoder encodeObject:mealsAndSnacks forKey:@"mealsAndSnacks"];
    [aCoder encodeObject:consumeSugarDrinks forKey:@"consumeSugarDrinks"];
    [aCoder encodeObject:consumeAlcohol forKey:@"consumeAlcohol"];
    
}

//Add support for decoding Survey data from disk into an object
- (id) initWithCoder:(NSCoder *)aDecoder {
    
    self = [super init];
    
    if (self) {
        [self setSurveyIssuesArray:[aDecoder decodeObjectForKey:@"surveyIssuesArray"]];
        
        [self setFamilyWeight:[aDecoder decodeObjectForKey:@"familyWeight"]];
        [self setSupportExpected:[aDecoder decodeObjectForKey:@"supportExpected"]];
        
        [self setWeightControlImportance:[aDecoder decodeFloatForKey:@"weightControlImportance"]];
        
        [self setActivityTime:[aDecoder decodeObjectForKey:@"activityTime"]];
        
        [self setActivityDays:[aDecoder decodeFloatForKey:@"activityDays"]];
     
        [self setChangeConfidence:[aDecoder decodeFloatForKey:@"changeConfidence"]];
        
        [self setMealsAndSnacks:[aDecoder decodeObjectForKey:@"mealsAndSnacks"]];
        [self setConsumeSugarDrinks:[aDecoder decodeObjectForKey:@"consumeSugarDrinks"]];
        [self setConsumeAlcohol:[aDecoder decodeObjectForKey:@"consumeAlcohol"]];
    }
    return self;
}


@end
