//
//  UserData.m
//  MoveCoachMobile
//
//  Created by Alex Morgan on 8/30/12.
//

#import "UserData.h"
#import "WeightData.h"
#import "WeightGoal.h"
#import "ActivityGoal.h"
#import "DietGoal.h"
#import "SurveyData.h"
#import "PARecallData.h"
#import "PAData.h"
#import "StepData.h"
#import "SlipEvent.h"
#import "TriggerData.h"
#import "DietData.h"
#import "ProgressCheckIn.h"

#import "Guide.h"
#import "GuideTimer.h"

//#import "MealHungerMood.h"
#import "ImageData.h"

static UserData *dataStore = nil;

@implementation UserData {
    
    WeightData *latestEnteredWeight;
    
}

@synthesize height, weights,weightsHashByDate, weightGoals, activityGoals, dietGoals, dietSMARTGoal1, dietSMARTGoal2, activitySMARTGoal1, activitySMARTGoal2, caloriesBurned, caloriesConsumed, gender, activityFactor, surveyData, recallActivity, envTriggers, envBadFoods, triggerFixData, fixTriggers, fixes, age, foodEntries, customNutrition, customPortion, mealHungerEntries, mealMoodEntries, mealMPCHEntries, favoriteNutrition, favoritePortion, paEntries, favoritePA, stepsTaken, customWorkout, paEntryScreen, entryDateString, previousPAEntryScreen, dietEntryScreen, previousDietEntryScreen,eulaComplete, initialWarningComplete, caloriesByDateDictionary,askedForNotifications;

@synthesize /*targetWeight,dateOfBirth, */currentMeal,bmiFirst,lastBMIStr,belowMinCalGoal;

//@synthesize /*weightLossRate, totalWeightLossGoal, */totalWeightLossGoalPercent;

@synthesize goalsViewToPush;

@synthesize joinedMOVE1, joinedMOVE2, changeWillHelp1, changeWillHelp2;

@synthesize userArchiveFilePath;

@synthesize fattyFoods, fattyFoodReductionMethods, fattyFoodEnteredTexts;

@synthesize planningArray, planningRewardArray;

@synthesize slipEvents, slipEventDescription;

@synthesize weightCalorieCategoryHasChanged;

@synthesize sleep, sedentary, leisure, strength, flexibility, anrTime, anrDays, everyday, imageStore;

@synthesize isVeteran, hasSeenVideoHelp;

@synthesize calorieGoalOffset;

static NSDateFormatter *dateFormatter;

+ (UserData *)dataStore
{
    if (dataStore == nil) {
        
        if (dateFormatter == nil) {
            dateFormatter = [[NSDateFormatter alloc] init];
        }

        //check to see is an Archive file exists to rebuild the singelton object from
        NSArray *documentDirectories = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
        NSString *documentDirectory = [documentDirectories objectAtIndex:0];
        
        NSString *archivePath = [documentDirectory stringByAppendingPathComponent:@"user.archive"];
        Boolean archiveExists = [[NSFileManager defaultManager] fileExistsAtPath:archivePath];
        
        if (archiveExists) {
            dataStore = [[NSKeyedUnarchiver unarchiveObjectWithFile:archivePath] init];
        } else {
            dataStore = [[super allocWithZone:nil] init];
        }
        //grab the archive file path so we can use it when writing to disk
        [dataStore setUserArchiveFilePath:archivePath];
    }
    return dataStore;
}

- (id)init
{
    self = [super init];
     
    if (self) {
        
        //this is where we will try to get all the data from the archive, and if not, then instantiate new empty objects
        if (!self.weights) {
            self.weights = [[NSMutableArray alloc] init];
        }
        if (!self.weightsHashByDate) {
            self.weightsHashByDate = [[NSMutableDictionary alloc] init];
        }
        
        if (!self.stepsTaken) {
            self.stepsTaken = [[NSMutableArray alloc] init];
        }
        if (!self.customWorkout) {
            self.customWorkout = [[NSMutableArray alloc]init];
        }
        if (!self.foodEntries) {
            self.foodEntries = [[NSMutableArray alloc] init];
        }
        if (!self.mealHungerEntries){
            self.mealHungerEntries = [[NSMutableArray alloc]init];
        }
        if (!self.mealMoodEntries){
            self.mealMoodEntries = [[NSMutableArray alloc]init];
        }
        if (!self.mealMPCHEntries){
            self.mealMPCHEntries = [[NSMutableArray alloc]init];
        }
        if (!self.customNutrition) {
            self.customNutrition = [[NSMutableArray alloc] init];
        }
        if (!self.customPortion) {
            self.customPortion = [[NSMutableArray alloc] init];
        }
        if (!self.favoriteNutrition) {
            self.favoriteNutrition = [[NSMutableArray alloc] init];
        }
        if (!self.favoritePortion) {
            self.favoritePortion = [[NSMutableArray alloc] init];
        }
        if (!self.paEntries) {
            self.paEntries = [[NSMutableArray alloc] init];
        }
        if (!self.favoritePA) {
            self.favoritePA = [[NSMutableArray alloc] init];
        }
        if (!self.surveyData) {
            self.surveyData = [[SurveyData alloc]init];
        }
        if (!self.recallActivity) {
            self.recallActivity = [[NSMutableArray alloc]init];
        }
        if(!self.envTriggers){
            self.envTriggers = [[NSMutableArray alloc]init];
        }
        if(!self.envBadFoods){
            self.envBadFoods = [[NSMutableArray alloc]init];
        }
        if (!self.triggerFixData) {
            self.triggerFixData = [[NSMutableArray alloc]init];
        }
        if (!self.fixes) {
            self.fixes = [[NSMutableArray alloc]init];
        }
        if (!self.fixTriggers) {
            self.fixTriggers = [[NSMutableArray alloc]init];
        }
        if (!self.fattyFoods) {
            self.fattyFoods = [[NSMutableArray alloc]init];
        }
        if (!self.fattyFoodReductionMethods) {
            self.fattyFoodReductionMethods = [[NSMutableArray alloc]init];
        }
        if (!self.fattyFoodEnteredTexts) {
            self.fattyFoodEnteredTexts = [[NSMutableArray alloc]init];
        }
        if (!self.imageStore) {
            self.imageStore = [[NSMutableDictionary alloc] init];
        }
        if (!self.planningArray) {
            self.planningArray = [[NSMutableArray alloc]init];
        }
        if (!self.planningRewardArray) {
            self.planningRewardArray = [[NSMutableArray alloc]init];
        }
        if (!self.weightGoals) {
            self.weightGoals = [[NSMutableArray alloc] init];
        }
        if (!self.activityGoals) {
            self.activityGoals = [[NSMutableArray alloc] init];
        }
        if (!self.dietGoals) {
            self.dietGoals = [[NSMutableArray alloc] init];
        }
        if (!self.caloriesByDateDictionary) {
            self.caloriesByDateDictionary = [[NSMutableDictionary alloc] init];
        }
    }
    return self;
}

+ (BOOL) remove {
    
    BOOL success = YES;
    dataStore = nil;
    
    return success;
}

#pragma mark -
#pragma mark Weight Methods

//- (void)addWeight:(float)weight
//{
//    int currentWeightCalorieCategory = [self getWeightCalorieCategoryForWeight:[self getCurrentWeight]];
//    int newWeightCalorieCategory = [self getWeightCalorieCategoryForWeight:weight];
//    
//    //If the user's new weight has caused them to change calorie categories, this boolean will be true.  It is checked in the Summary section of each guide and the chart is displayed if true.  The only place this is set back to NO is in Summary.
//    if (currentWeightCalorieCategory != newWeightCalorieCategory) {
//        [self setWeightCalorieCategoryHasChanged:YES];
//    }
//
//    
//    WeightData *weightData = [[WeightData alloc]initWithWeight:weight];
//    [[self weights] addObject:weightData];
//
//}

- (void)addWeight:(float)weight andDate:(NSDate *)date
{
    int currentWeightCalorieCategory = [self getWeightCalorieCategoryForWeight:[self getCurrentWeight]];
    int newWeightCalorieCategory = [self getWeightCalorieCategoryForWeight:weight];
    
    //If the user's new weight has caused them to change calorie categories, this boolean will be true.  It is checked in the Summary section of each guide and the chart is displayed if true.  The only place this is set back to NO is in Summary.
    if (currentWeightCalorieCategory != newWeightCalorieCategory) {
        [self setWeightCalorieCategoryHasChanged:YES];
    }
        
    WeightData *weightData = [[WeightData alloc]initWithWeight:weight andDate:date];
    [[self weights] addObject:weightData];
    [self addWeightToWeightsDictionary:weightData];
    
    if (!latestEnteredWeight) {
        latestEnteredWeight = weightData;
    } else {        
        NSDate *currentLatestDate = latestEnteredWeight.dateCreated;
        NSDate *newEntryDate = weightData.dateCreated;
        
        if (([newEntryDate compare:currentLatestDate] == NSOrderedDescending) || ([newEntryDate compare:currentLatestDate] == NSOrderedSame ) ) {
            latestEnteredWeight = weightData;
            
        }
    }
    
}

- (void)addWeightWithWeightData:(WeightData *)wd {
    [[self weights] addObject:wd];
}

/*
 Weight Hash data methods.  These really should be moved to a dedicated weightManager singleton object
 */

//this method adds the weight data for a specific date and weightData object to a hash stored in userData model. Used for fast retrieval of weights based on date
- (void) addWeightToWeightsDictionary:(WeightData *)weightData {
    
    NSDate *keyDate = [[Utilities defaultUtilities] getDateConfiguredAsKey:weightData.dateCreated];
    NSMutableArray *weightDataObjects;
    
    //create or update the hash data
    if ([[self weightsHashByDate] objectForKey:keyDate] == nil) {
        //create array and store current self into it, and then add that to the hash for this date
        weightDataObjects = [NSMutableArray arrayWithObject:weightData];
        [[self weightsHashByDate] setObject:weightDataObjects forKey:keyDate];
        
    } else {
        
        weightDataObjects = [[self weightsHashByDate] objectForKey:keyDate];
        if (weightDataObjects) {
            [weightDataObjects addObject:weightData];
            [[self weightsHashByDate] setObject:weightDataObjects forKey:keyDate];
        }
    }
}



- (NSArray *) getWeightsForDate:(NSDate *)keyDate {
    //return the array of weightData objects entered on keyDate
    
    NSDate *normalizedKeyDate = [[Utilities defaultUtilities] getDateConfiguredAsKey:keyDate];
    
    NSMutableArray *dailyWeightsSortedByDate = [self.weightsHashByDate objectForKey:normalizedKeyDate];
    
    NSSortDescriptor *sortDescriptor;
    sortDescriptor = [[NSSortDescriptor alloc]initWithKey:@"dateCreated" ascending:YES];
    NSArray *sortDescriptors = [NSArray arrayWithObject:sortDescriptor];
    [dailyWeightsSortedByDate sortUsingDescriptors:sortDescriptors];
    
    
    return dailyWeightsSortedByDate;
}

- (WeightData *) getLatestWeightData {
    
    
    //get the latest date that a weight was entered
    //get the latest weight entered on that date
    
    NSArray *sortedKeys = [[self.weightsHashByDate allKeys] sortedArrayUsingSelector: @selector(compare:)];
    
    return [self getLatestWeightDataForDate:[sortedKeys lastObject]];

}

- (WeightData *) getLatestWeightDataForDate:(NSDate *)keyDate {
    
    //get the array of weightData objects entered on keyDate
    //sort by date and return the latest
    WeightData *latestWeightData;
    
    NSDate *normalizedKeyDate = [[Utilities defaultUtilities] getDateConfiguredAsKey:keyDate];
    
    NSMutableArray *weightDataObjects = [self.weightsHashByDate objectForKey:normalizedKeyDate];
    if ([weightDataObjects count] > 0) {
        NSSortDescriptor *sortDescriptor;
        sortDescriptor = [[NSSortDescriptor alloc]initWithKey:@"dateCreated" ascending:YES];
        NSArray *sortDescriptors = [NSArray arrayWithObject:sortDescriptor];
        [weightDataObjects sortUsingDescriptors:sortDescriptors];
        
        latestWeightData = [weightDataObjects objectAtIndex:weightDataObjects.count -1];
    } else {
        latestWeightData = nil;
    }
    return latestWeightData;
}

- (NSDate *) getOldestWeightDate {
    
    NSArray *sortedKeys = [[self.weightsHashByDate allKeys] sortedArrayUsingSelector: @selector(compare:)];
    
    return [sortedKeys firstObject];

}


-(void)removeWeight:(WeightData *)wd
{
    BOOL recalculateLatestDate = NO;
    if (wd == latestEnteredWeight) {
        recalculateLatestDate = YES;
    }
    
    [self removeWeightFromWeightsDictionary:wd];
    [weights removeObject:wd];
    
    if (recalculateLatestDate) {
        [self recalculateLatestEnteredWeightFromExistingEntries];
    }
}

- (void) removeWeightFromWeightsDictionary:(WeightData *)weightData {
    
    NSDate *keyDate = [[Utilities defaultUtilities] getDateConfiguredAsKey:weightData.dateCreated];
    NSMutableArray *weightsForToday = [[self weightsHashByDate] objectForKey:keyDate];
    
    if (weightsForToday) {
        [weightsForToday removeObject:weightData];
    
        if ([weightsForToday count] > 0) {
            [[self weightsHashByDate] setObject:weightsForToday forKey:keyDate];

        } else {
            [[self weightsHashByDate] removeObjectForKey:keyDate];
        }
    }
}



-(float)getCurrentWeight
{
    float latestWeight = 0.0;
    if (latestEnteredWeight) {
        latestWeight = latestEnteredWeight.userWeight;
    }
    return latestWeight;
}


-(float)getStartingWeight
{
    if (self.weights.count > 0) {
        //WeightData *wd = [[self weights] objectAtIndex:0];
        //float initialWeight = [wd userWeight];
        //return initialWeight;
        return [[self.weights objectAtIndex:0] userWeight];
        //return [[[self weights] objectAtIndex:0] userWeight];
    } else {
        return 0.0;
    }
}

-(NSDate *)getStartingWeightDate
{
    if (self.weights.count > 0) {
        WeightData *wd = [[self weights] objectAtIndex:0];
        NSDate *initialWeightDate = [wd dateCreated];
        return initialWeightDate;
        //        return [[self.weights objectAtIndex:0] dateCreated];
        //return [[[self weights] objectAtIndex:0] userWeight];
    } else {
        return nil;
    }
}

-(float)getWeightLostLBS
{
    return ([self getStartingWeight] - [self getCurrentWeight]);
    
}

-(float)getWeightLostPercent
{
    float percent = (([self getWeightLostLBS] / [self getStartingWeight])*100.0);
    
    if (isnan(percent)) {
        return 0.0;
    }
    
    return percent;
}

- (void) recalculateLatestEnteredWeightFromExistingEntries {
    
    NSMutableArray *dateKeys = [NSMutableArray arrayWithArray:[self.weightsHashByDate allKeys]];
    
    NSArray *sortedKeys = [dateKeys sortedArrayUsingSelector: @selector(compare:)];

    latestEnteredWeight = [self getLatestWeightDataForDate:[sortedKeys lastObject]];
    
}

- (void)clearWeights {
    [self.weights removeAllObjects];
    [self.weightsHashByDate removeAllObjects];
    latestEnteredWeight = nil;
}


#pragma mark -
#pragma mark Diet Methods

- (void)addDietEntryWithFoodName:(NSString *)foodname date:(NSString *)date calories:(float)calories foodCode:(NSString *)foodcode portionFactor:(float)portionfactor portionDesc:(NSString *)portiondesc servings:(float)servings meal:(NSString *)meal
{
    
    DietData *dietData = [[DietData alloc]initWithFoodName:foodname date:date calories:calories foodCode:foodcode portionFactor:portionfactor portionDesc:portiondesc servings:servings meal:meal];
    [[self foodEntries]addObject:dietData];
}

- (void)addPARecallActivity:(NSString *)act andDuration:(NSString *)dur
{
    PARecallData *rd = [[PARecallData alloc]initWithActivity:act andDuration:dur];
    
    [[self recallActivity] addObject:rd];

}

- (void)addTrigger:(NSString *)t andFix:(NSString *)f
{
    TriggerData *td = [[TriggerData alloc]initWithTrigger:t andFix:f];
    
    [[self triggerFixData] addObject:td];
}


#pragma mark -
#pragma mark - imageStore methods
//if date is nil, use current date
- (void) addImageToImageStore:(UIImage *)image withDate:(NSDate *)date {
    if (!date) {
        date = [NSDate date];
    }
    ImageData *imageData = [[ImageData alloc] initWithImage:image andDate:date];
    
    if ([[self getSelectedImages] count] == 0) {
        [imageData setCompare:YES];
    } else if ([[self getSelectedImages] count] == 1) {
        //if there is only one image selected to compare, set the latest one as the one to compare which causes it to be shown in the fullScreenImageView
        NSArray *keys = [imageStore allKeys];
        for (NSDate *key in keys) {
            //set all imageData objects as unselected
            ImageData *imageD = [imageStore objectForKey:key];
            [self setImageSelectionBool:imageD withFlag:NO];
        }
        [self setImageSelectionBool:imageData withFlag:YES];
    }
    
    [imageStore setObject:imageData forKey:date];
}

- (void) removeImageFromImageStoreForKey:(NSDate *)key {
    [imageStore removeObjectForKey:key];
}

- (ImageData *) getImageDataForKey:(NSDate *)key {
    
    ImageData *imageData = [imageStore objectForKey:key];
    
    return imageData;
}

- (NSArray *)getSelectedImages {
    NSMutableArray *imageObjectsForCompare = [[NSMutableArray alloc]init];
    
    for (NSDate *key in imageStore) {
        ImageData *imageData = [imageStore objectForKey:key];
        if ([imageData compare]) {
            [imageObjectsForCompare addObject:imageData];
        }
    }
    
    return [NSArray arrayWithArray:imageObjectsForCompare];
}

- (void) setImageSelectionBool:(ImageData *)imageData withFlag:(BOOL)flag {
    
    [imageData setCompare:flag];
    
    NSDate *key = [imageData date];
    [imageStore removeObjectForKey:key];
    
    [imageStore setObject:imageData forKey:key];
    
}



#pragma mark - Calculation methods
-(float)getBMI
{
    float bmi = (([self getCurrentWeight] / ([self height] * [self height]))*703.0);
    if (isnan(bmi)) {
        return 0.0;
    }
    return bmi;
}



-(float)getAverageCalorieBalanceForGuide:(int)guideId
{
    Guide *guide = [[[GuideTimer defaultStore] guideTracker] objectForKey:[NSNumber numberWithInt:guideId]];
    
    NSDate *start = [guide startDate];
    NSDate *end = [guide endDate];
    
    if (!start) {
        start = [NSDate date];
    }
    if (!end) {
        //this should not happen, but just in case we make sure we have an end date
        end = [NSDate date];
    }
    
    int duration = [[Utilities defaultUtilities] daysBetween:start and:end]-1;
    
    int daysInSeconds = 86400;
    
    int averageCounter = 0;
    float calorieDifferenceSum = 0.0;
    
    for (int i=duration; i>=0; i--) {
        
        NSDate *date = [NSDate dateWithTimeInterval:-i * daysInSeconds sinceDate:end];

        float calsConsumed = [self getCaloriesConsumedOnDate:date];
        float calsBurnt = [self getCaloriesBurnedOnDate:date];
        
        float calorieDifference =  calsConsumed - [self getCalorieGoal] - calsBurnt;
        
        
        
        //at least one of these must have valid data for this date or we do not use this date
        if ((calsConsumed != 0) || (calsBurnt != 0)) {

            averageCounter++;
            calorieDifferenceSum += calorieDifference;
        
        
        }
    }

    float calorieAverage;
    if (averageCounter == 0) {
        calorieAverage = 0;
    } else {
        calorieAverage = calorieDifferenceSum / averageCounter;
    }
    
    return calorieAverage;
    
}

- (float)getDailyCalorieBalance
{
    return [self getDailyCalorieGoal] + [self caloriesBurned] - [self caloriesConsumed];
}


-(float)getDailyCalorieGoal
{
    /*float BMR;
    if ([[self gender] isEqualToString:@"Male"]) {
        BMR = (10.0 * ([self getCurrentWeight]*0.45359237)) + (6.25 * ([self height]*2.54)) - (5 * [self age]) + 5;
    }else {
        BMR = (10.0 * ([self getCurrentWeight]*0.45359237)) + (6.25 * ([self height]*2.54)) - (5 * [self age]) - 161;
    }
    
    if (BMR < 0.0) {
        return 0.0;
    }
    
    return (BMR * [self activityFactor]);*/
    
    float currentWeight = [[UserData dataStore]getCurrentWeight];
    WeightGoal *currentWeightGoal = [self getCurrentWeightGoal];
    
    float goalCal;
    float minGoal, maxGoal;
    
    float lossRate = [currentWeightGoal weightLossRate];
    
    if (currentWeight < 200) {
        minGoal = 1200;
        maxGoal = 1500;
        goalCal = minGoal + ((maxGoal - minGoal)*((lossRate-2)/(-2)));
    } else if (currentWeight < 226) {
        minGoal = 1500;
        maxGoal = 1800;
        goalCal = minGoal + ((maxGoal - minGoal)*((lossRate-2)/(-2)));
    } else if (currentWeight < 251) {
        minGoal = 1800;
        maxGoal = 2000;
        goalCal = minGoal + ((maxGoal - minGoal)*((lossRate-2)/(-2)));
    } else if (currentWeight < 301) {
        minGoal = 2000;
        maxGoal = 2500;
        goalCal = minGoal + ((maxGoal - minGoal)*((lossRate-2)/(-2)));
    } else if (currentWeight < 351) {
        minGoal = 2500;
        maxGoal = 3000;
        goalCal = minGoal + ((maxGoal - minGoal)*((lossRate-2)/(-2)));
    } else {
        minGoal = 3000;
        maxGoal = 3500;
        goalCal = minGoal + ((maxGoal - minGoal)*(lossRate/2));
    }
    
    if ([self calorieGoalOffset]) {
        return (goalCal + self.calorieGoalOffset);
    }else{
        return goalCal;
    }
}

-(int) getWeightCalorieCategoryForWeight:(float)weight {
    
    if (weight == 0) {
        return 0;
    }
    
    if (weight < 200) {
        return 1;
    } else if (weight <= 225 ) {
        return 2;
    } else if (weight <= 250 ) {
        return 3;
    } else if (weight <= 300 ) {
        return 4;
    } else if (weight <= 350 ) {
        return 5;
    } else if (weight > 350 ) {
        return 6;
    }
    
    return 0;
}

//passing 0 as category will cause this method to use the current user weight to get the current weight category
-(NSString *) getCaloriesPerDayForWeightCalorieCategory:(int)category formattedForAccessibility:(BOOL)flag{
    
    if (category == 0) {
        category = [self getWeightCalorieCategoryForWeight:[self getCurrentWeight]];
    }
    
    NSString *caloriesPerDay;
    
    switch (category) {
        case 1:
            if (flag) {
                caloriesPerDay = @"twelve hundred to fifteen hundred calories per day";
            }else {
                caloriesPerDay = @"1200-1500 calories/day";
            }
            break;
        case 2:
            if (flag) {
                caloriesPerDay = @"fifteen hundred to eighteen hundred calories per day";
            }else {
                caloriesPerDay =  @"1500-1800 calories/day";
            }
            break;
        case 3:
            if (flag) {
                caloriesPerDay = @"eighteen hundred to two thousand calories per day";
            }else {
                caloriesPerDay =  @"1800-2000 calories/day";
            }
            break;
        case 4:
            if (flag) {
                caloriesPerDay = @"two thousand to twenty five hundred calories per day";
            }else {
                caloriesPerDay =  @"2000-2500 calories/day";
            }
            break;
        case 5:
            if (flag) {
                caloriesPerDay = @"twenty five hundred to three thousand calories per day";
            }else {
                caloriesPerDay =  @"2500-3000 calories/day";
            }
            break;
        case 6:
            caloriesPerDay =  @"See a Move! Dietitian";
            break;
            
            
        default:
            caloriesPerDay = @"Weight is not set";
    }
    return caloriesPerDay;
}

- (float)getCalorieGoal
{
    
    
    float minGoal, maxGoal, goalCal;

    float currentWeight = [self getCurrentWeight];
    float lossRate = [[self getCurrentWeightGoal] weightLossRate];
    
    if (currentWeight < 200) {
        minGoal = 1200;
        maxGoal = 1500;
        goalCal = minGoal + ((maxGoal - minGoal)*((lossRate-2)/(-2)));
    } else if (currentWeight < 226) {
        minGoal = 1500;
        maxGoal = 1800;
        goalCal = minGoal + ((maxGoal - minGoal)*((lossRate-2)/(-2)));
    } else if (currentWeight < 251) {
        minGoal = 1800;
        maxGoal = 2000;
        goalCal = minGoal + ((maxGoal - minGoal)*((lossRate-2)/(-2)));
    } else if (currentWeight < 301) {
        minGoal = 2000;
        maxGoal = 2500;
        goalCal = minGoal + ((maxGoal - minGoal)*((lossRate-2)/(-2)));
    } else if (currentWeight < 351) {
        minGoal = 2500;
        maxGoal = 3000;
        goalCal = minGoal + ((maxGoal - minGoal)*((lossRate-2)/(-2)));
    } else {
        minGoal = 3000;
        maxGoal = 3500;
        goalCal = minGoal + ((maxGoal - minGoal)*(lossRate/2));
    }
    float goal = goalCal;
    
    if ([self calorieGoalOffset]) {
        
        goalCal = (goalCal + self.calorieGoalOffset);
        
        if (goalCal < 1200.0) {
            self.belowMinCalGoal = YES;
            self.calorieGoalOffset = 1200.0 - goal;
            goalCal = 1200.0;
        }
    }
    
    return goalCal;
}



- (NSMutableArray *)getFoodEntriesByMeal:(NSString *)meal andDate:(NSDate *)date
{
    return [self foodEntries];
}



- (NSInteger)age
{
    if (age) {
        return age;
    } else {
        return 35;
    }
}


#pragma mark -
#pragma mark Weight Goals
/**
 * Weight Goal Methods
 **/

-(void)addWeightGoal:(float)referenceWeight
{
    WeightGoal *weightGoal = [[WeightGoal alloc] initWithReferenceWeight:referenceWeight andCurrentWeight:[self getCurrentWeight] andDate:[NSDate date]];
    [[self weightGoals] addObject:weightGoal];
    
}

- (void)addWeightGoalToWeightGoals:(WeightGoal *)weightGoal
{    
    [[self weightGoals] addObject:weightGoal];
}

-(void)removeWeightGoal:(WeightGoal *)weightGoal
{
    [weightGoals removeObject:weightGoal];
}

-(WeightGoal *)getStartingWeightGoal
{
    if (self.weightGoals.count > 0) {
        return [self.weightGoals objectAtIndex:0];
    } else {
        [self addWeightGoal:[self getStartingWeight]];
        return [self.weightGoals objectAtIndex:0];
    }
}

- (WeightGoal *)getCurrentWeightGoal
{
    WeightGoal *currentWeightGoal;
    
    if (self.weightGoals.count > 0) {
        currentWeightGoal = [self.weightGoals objectAtIndex:([weightGoals count]-1)];
        if ([currentWeightGoal isArchivable]) {
            if (!self.draftWeightGoal) {
                [self createDraftWeightGoal: currentWeightGoal];
            }
            currentWeightGoal = self.draftWeightGoal;
        }
    } else {
        [self addWeightGoal:[self getStartingWeight]];
        currentWeightGoal = [self.weightGoals objectAtIndex:0];
    }
    return currentWeightGoal;
}

- (WeightGoal *)getWeightGoalForDate: (NSDate *)date
{
    WeightGoal *goal = nil;

    NSPredicate *predicate = [NSPredicate predicateWithFormat:@"date <= %@", date];
    NSArray *filteredGoals = [weightGoals filteredArrayUsingPredicate:predicate];
    if (filteredGoals.count > 0) {
//        goal = filteredGoals[([filteredGoals count]-1)];
        goal = [filteredGoals lastObject];
    }
    return goal;
}

- (void)createDraftWeightGoal: (WeightGoal *)currentWeightGoal
{
    [self setDraftWeightGoal: [[WeightGoal alloc] initWithWeightGoal:currentWeightGoal]];
}

#pragma mark -
#pragma mark Activity Goals
/**
 * Activity Goal Methods
 **/

-(void)addActivityGoal: (NSString *)goal1Text
{
    ActivityGoal *activityGoal = [[ActivityGoal alloc] initWithGoal1Text:goal1Text];
    [[self activityGoals] addObject:activityGoal];
}

- (void)addActivityGoalToActivityGoals:(ActivityGoal *)activityGoal
{
    [[self activityGoals] addObject:activityGoal];
}

-(void)removeActivityGoal: (ActivityGoal *)activityGoal
{
    [[self activityGoals] removeObject:activityGoal];
}

-(ActivityGoal *)getStartingActivityGoal
{
    if (self.activityGoals.count > 0) {
        return [self.activityGoals objectAtIndex:0];
    } else {
        [self addActivityGoal:@""];
        return [self.activityGoals objectAtIndex:0];
    }
}

-(ActivityGoal *)getCurrentActivityGoal
{
    ActivityGoal *currentActivityGoal;
    
    if (self.activityGoals.count > 0) {
        currentActivityGoal = [self.activityGoals objectAtIndex:([activityGoals count]-1)];
        if ([currentActivityGoal isArchivable]) {
            if (!self.draftActivityGoal) {
                [self createDraftActivityGoal: currentActivityGoal];
            }
            currentActivityGoal = self.draftActivityGoal;
        }
    } else {
        [self addActivityGoal:@""];
        currentActivityGoal = [self.activityGoals objectAtIndex:0];
    }
    return currentActivityGoal;
}

- (void)createDraftActivityGoal: (ActivityGoal *)currentActivityGoal
{
    [self setDraftActivityGoal: [[ActivityGoal alloc] initWithGoal:currentActivityGoal]];
}

#pragma mark -
#pragma mark Diet Goals
/**
 * Diet Goal Methods
 **/

-(void)addDietGoal: (NSString *)goal1Text
{
    DietGoal *dietGoal = [[DietGoal alloc] initWithGoal1Text:goal1Text];
    [[self dietGoals] addObject:dietGoal];
}

- (void)addDietGoalToDietGoals:(DietGoal *)dietGoal
{
    [[self dietGoals] addObject:dietGoal];
}

-(void)removeDietGoal: (DietGoal *)dietGoal
{
    [[self dietGoals] removeObject:dietGoal];
}

-(DietGoal *)getStartingDietGoal
{
    if (self.dietGoals.count > 0) {
        return [self.dietGoals objectAtIndex:0];
    } else {
        [self addDietGoal:@""];
        return [self.dietGoals objectAtIndex:0];
    }
}

-(DietGoal *)getCurrentDietGoal
{
    DietGoal *currentDietGoal;
    
    if (self.dietGoals.count > 0) {
        currentDietGoal = [self.dietGoals objectAtIndex:([dietGoals count]-1)];
        if ([currentDietGoal isArchivable]) {
            if (!self.draftDietGoal) {
                [self createDraftDietGoal: currentDietGoal];
            }
            currentDietGoal = self.draftDietGoal;
        }
    } else {
        [self addDietGoal:@""];
        currentDietGoal = [self.dietGoals objectAtIndex:0];
    }
    return currentDietGoal;
}

- (void)createDraftDietGoal: (DietGoal *)currentDietGoal
{
    [self setDraftDietGoal: [[DietGoal alloc] initWithGoal:currentDietGoal]];
}

#pragma mark -
#pragma mark Calorie Utility Methods
//This is a utility method that returns the calorie range based on the users current weight
- (NSString *) getCalorieRange {
    int currentWeight = [self getCurrentWeight];
    //check for null?
    
    if (currentWeight < 200) {
        return @"1200-1500 calories/day";
    } else if (currentWeight < 226) {
        return @"1500-1800 calories/day";
    } else if (currentWeight < 251) {
        return @"1800-2000 calories/day";
    } else if (currentWeight < 301) {
        return @"2000-2500 calories/day";
    } else if (currentWeight < 351) {
        return @"2500-3000 calories/day";
    } else {
        return @"See a Move! Dietitian";
    }

}

- (float) getCaloriesBurnedOnDate:(NSDate *)date
{
    float totalCalories = 0.0;

    [dateFormatter setDateStyle:NSDateFormatterShortStyle];
    [dateFormatter setTimeStyle:NSDateFormatterNoStyle];
    
    NSString *dateStr = [dateFormatter stringFromDate:date];
    
    for (PAData *activity in paEntries) {
        if ([activity.dateCreated isEqualToString:dateStr]) {
            totalCalories = totalCalories + activity.caloriesBurned;
        }
    }
    return totalCalories;
}

- (float) getCaloriesConsumedOnDate:(NSDate *)date {
    
    float totalCalories = 0.0;
    
    [dateFormatter setDateStyle:NSDateFormatterShortStyle];
    [dateFormatter setTimeStyle:NSDateFormatterNoStyle];
    
    NSString *dateStr = [dateFormatter stringFromDate:date];
    
    for (DietData *dietData in foodEntries) {
        if ([dietData.dateCreated isEqualToString:dateStr]) {
            totalCalories = totalCalories + dietData.caloriesEaten;
        }
    }
    
/*    for (PAData *activity in paEntries) {
        if ([activity.dateCreated isEqualToString:dateStr]) {
            totalCalories = totalCalories + activity.caloriesBurned;
        }
    }
 */ 
 return totalCalories;
    
}

- (int) getStepsForDate:(NSDate *)date
{
    int totalSteps = 0;

    [dateFormatter setDateStyle:NSDateFormatterShortStyle];
    [dateFormatter setTimeStyle:NSDateFormatterNoStyle];
    
    NSString *dateStr = [dateFormatter stringFromDate:date];

    for (StepData *steps in stepsTaken) {
        if ([steps.dateStepsCreated isEqualToString:dateStr]) {
            totalSteps = totalSteps + steps.numberOfSteps;
        }
    }
    
    return totalSteps;
}


- (NSDate *) getOldestDietEntry {
    
    NSDate *retDate;
    
    NSArray *sortedArray;
    sortedArray = [foodEntries sortedArrayUsingComparator:^NSComparisonResult(id a, id b) {
        NSDate *first = [(DietData*)a referenceDate];
        NSDate *second = [(DietData*)b referenceDate];
        return [first compare:second];
    }];
    
    DietData *firstDietData = [sortedArray firstObject];
    
    retDate = firstDietData.referenceDate;
    
    return retDate;
}

- (NSDate *) getOldestActivityEntry {

    NSDate *retDate;
    
    NSArray *sortedArray;
    sortedArray = [paEntries sortedArrayUsingComparator:^NSComparisonResult(id a, id b) {
        NSDate *first = [(PAData*)a referenceDate];
        NSDate *second = [(PAData*)b referenceDate];
        return [first compare:second];
    }];
    
    PAData *firstPAData = [sortedArray firstObject];
    
    retDate = firstPAData.referenceDate;
    
    return retDate;

}



//encode/decode methods
- (void)encodeWithCoder:(NSCoder *)aCoder
{
    [aCoder encodeBool:eulaComplete forKey:@"eulaComplete"];
    
    [aCoder encodeBool:initialWarningComplete forKey:@"initialWarningComplete"];
    
    [aCoder encodeBool:askedForNotifications forKey:@"askedForNotifications"];
    
    [aCoder encodeBool:bmiFirst forKey:@"bmiFirst"];
    
    [aCoder encodeObject:lastBMIStr forKey:@"lastBMIStr"];
    
    [aCoder encodeObject:surveyData forKey:@"surveyData"];
    
    [aCoder encodeObject:weights forKey:@"weights"];

    [aCoder encodeObject:weightGoals forKey:@"weightGoals"];
    
    [aCoder encodeObject:activityGoals forKey:@"activityGoals"];
    
    [aCoder encodeObject:dietGoals forKey:@"dietGoals"];
    
    [aCoder encodeObject:stepsTaken forKey:@"stepsTaken"];
    
    [aCoder encodeObject:customWorkout forKey:@"customWorkout"];
    
    [aCoder encodeObject:imageStore forKey:@"imageStore"];
    
    [aCoder encodeObject:foodEntries forKey:@"foodEntries"];
    
    [aCoder encodeObject:paEntries forKey:@"paEntries"];
    
    [aCoder encodeObject:favoritePA forKey:@"favoritePA"];
    
    [aCoder encodeObject:recallActivity forKey:@"recallActivity"];
    
    [aCoder encodeObject:mealHungerEntries forKey:@"mealHungerEntries"];
    
    [aCoder encodeObject:mealMoodEntries forKey:@"mealMoodEntries"];
    
    [aCoder encodeObject:favoritePortion forKey:@"favoritePortion"];
    
    [aCoder encodeObject:favoriteNutrition forKey:@"favoriteNutrition"];
    
    [aCoder encodeObject:customPortion forKey:@"customPortion"];
    
    [aCoder encodeObject:customNutrition forKey:@"customNutrition"];
    
    [aCoder encodeObject:mealMPCHEntries forKey:@"mealMPCHEntries"];
    
    [aCoder encodeInteger:age forKey:@"age"];

    [aCoder encodeFloat:height forKey:@"height"];
    
    [aCoder encodeObject:gender forKey:@"gender"];
    
    [aCoder encodeFloat:activityFactor forKey:@"activityFactor"];
    
    [aCoder encodeObject:joinedMOVE1 forKey:@"joinedMOVE1"];
    
    [aCoder encodeObject:joinedMOVE2 forKey:@"joinedMOVE2"];

    [aCoder encodeObject:changeWillHelp1 forKey:@"changeWillHelp1"];

    [aCoder encodeObject:changeWillHelp2 forKey:@"changeWillHelp2"];
    
    [aCoder encodeFloat:caloriesConsumed forKey:@"caloriesConsumed"];
    
    [aCoder encodeFloat:caloriesBurned forKey:@"caloriesBurned"];
        
    [aCoder encodeObject:fattyFoods forKey:@"fattyFoods"];
    
    [aCoder encodeObject:fattyFoodReductionMethods forKey:@"fattyFoodReductionMethods"];
    
    [aCoder encodeObject:fattyFoodEnteredTexts forKey:@"fattyFoodEnteredTexts"];
    
    // guide 8 Chapter 1 planning and reward arrays
    [aCoder encodeObject:planningArray forKey:@"planningArray"];
    
    [aCoder encodeObject:planningRewardArray forKey:@"planningRewardArray"];
    
    //guide 9 ch 2 environmental triggers 
    [aCoder encodeObject:envTriggers forKey:@"envTriggers"];
    
    [aCoder encodeObject:envBadFoods forKey:@"envBadFoods"];
    
    [aCoder encodeObject:fixes forKey:@"fixes"];
    
    [aCoder encodeObject:fixTriggers forKey:@"fixTriggers"];
    
    //guide 10 ch 2 chain of events data
    [aCoder encodeObject:slipEvents forKey:@"slipEvents"];
    
    [aCoder encodeObject:[self slipEventDescription] forKey:@"slipEventDescription"];
    
    [aCoder encodeFloat:sleep forKey:@"sleep"];
    
    [aCoder encodeFloat:sedentary forKey:@"sedentary"];
    
    [aCoder encodeFloat:leisure forKey:@"leisure"];
   
    [aCoder encodeFloat:strength forKey:@"strength"];
    
    [aCoder encodeFloat:flexibility forKey:@"flexibility"];
    
    [aCoder encodeFloat:anrTime forKey:@"anrTime"];
    
    [aCoder encodeFloat:anrDays forKey:@"anrDays"];
    
    [aCoder encodeFloat:everyday forKey:@"everyday"];
    
    [aCoder encodeBool:isVeteran forKey:@"isVeteran"];
    
    [aCoder encodeObject:caloriesByDateDictionary forKey:@"caloriesByDateDictionary"];
    
    [aCoder encodeObject:weightsHashByDate forKey:@"weightsHashByDate"];
    
    [aCoder encodeObject:latestEnteredWeight forKey:@"latestEnteredWeight"];
    
    [aCoder encodeBool:hasSeenVideoHelp forKey:@"hasSeenVideoHelp"];
    
    [aCoder encodeFloat:calorieGoalOffset forKey:@"calorieGoalOffset"];
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
    self = [super init];
        
    if (self) {
        [self setEulaComplete:[aDecoder decodeBoolForKey:@"eulaComplete"]];
        
        [self setInitialWarningComplete:[aDecoder decodeBoolForKey:@"initialWarningComplete"]];
        
        [self setAskedForNotifications:[aDecoder decodeBoolForKey:@"askedForNotifications"]];
        
        [self setBmiFirst:[aDecoder decodeBoolForKey:@"bmiFirst"]];
        
        [self setLastBMIStr:[aDecoder decodeObjectForKey:@"lastBMIStr"]];
        
        [self setSurveyData:[aDecoder decodeObjectForKey:@"surveyData"]];
        
        [self setWeights:[aDecoder decodeObjectForKey:@"weights"]];
        
        [self setWeightGoals:[aDecoder decodeObjectForKey:@"weightGoals"]];

        [self setActivityGoals:[aDecoder decodeObjectForKey:@"activityGoals"]];
        
        [self setDietGoals:[aDecoder decodeObjectForKey:@"dietGoals"]];
        
        [self setStepsTaken:[aDecoder decodeObjectForKey:@"stepsTaken"]];
        
        [self setCustomWorkout:[aDecoder decodeObjectForKey:@"customWorkout"]];
        
        [self setImageStore:[aDecoder decodeObjectForKey:@"imageStore"]];
        
        [self setFoodEntries:[aDecoder decodeObjectForKey:@"foodEntries"]];
        
        [self setPaEntries:[aDecoder decodeObjectForKey:@"paEntries"]];
        
        [self setFavoritePA:[aDecoder decodeObjectForKey:@"favoritePA"]];
        
        [self setRecallActivity:[aDecoder decodeObjectForKey:@"recallActivity"]];
        
        [self setMealHungerEntries:[aDecoder decodeObjectForKey:@"mealHungerEntries"]];
        
        [self setMealMoodEntries:[aDecoder decodeObjectForKey:@"mealMoodEntries"]];
        
//        [self setFavoritePortion:[aDecoder decodeObjectForKey:@"favoritePortion"]];
//        
//        [self setFavoriteNutrition:[aDecoder decodeObjectForKey:@"favoriteNutrition"]];
//        
//        [self setCustomPortion:[aDecoder decodeObjectForKey:@"customPortion"]];
//        
//        [self setCustomNutrition:[aDecoder decodeObjectForKey:@"customNutrition"]];
        
        [self setMealMPCHEntries:[aDecoder decodeObjectForKey:@"mealMPCHEntries"]];

        [self setAge:[aDecoder decodeIntegerForKey:@"age"]];

        [self setHeight:[aDecoder decodeFloatForKey:@"height"]];
        
        [self setGender:[aDecoder decodeObjectForKey:@"gender"]];
        
        [self setActivityFactor:[aDecoder decodeFloatForKey:@"activityFactor"]];
        
        [self setJoinedMOVE1:[aDecoder decodeObjectForKey:@"joinedMOVE1"]];
        
        [self setJoinedMOVE2:[aDecoder decodeObjectForKey:@"joinedMOVE2"]];

        [self setChangeWillHelp1:[aDecoder decodeObjectForKey:@"changeWillHelp1"]];
        
        [self setChangeWillHelp2:[aDecoder decodeObjectForKey:@"changeWillHelp2"]];
        
        [self setCaloriesConsumed:[aDecoder decodeFloatForKey:@"caloriesConsumed"]];
        
        [self setCaloriesBurned:[aDecoder decodeFloatForKey:@"caloriesBurned"]];
        
        [self setFattyFoods:[aDecoder decodeObjectForKey:@"fattyFoods"]];
        
        [self setFattyFoodReductionMethods:[aDecoder decodeObjectForKey:@"fattyFoodReductionMethods"]];
        
        [self setFattyFoodEnteredTexts:[aDecoder decodeObjectForKey:@"fattyFoodEnteredTexts"]];
        
        // Guide 8 Chapter 1 planning and reward arrays
        [self setPlanningArray:[aDecoder decodeObjectForKey:@"planningArray"]];
        
        [self setPlanningRewardArray:[aDecoder decodeObjectForKey:@"planningRewardArray"]];
        
        //Guide 9 Chapter 2 environmental triggers
        [self setEnvTriggers:[aDecoder decodeObjectForKey:@"envTriggers"]];
        
        [self setEnvBadFoods:[aDecoder decodeObjectForKey:@"envBadFoods"]];
        
        [self setFixes:[aDecoder decodeObjectForKey:@"fixes"]];
        
        [self setFixTriggers:[aDecoder decodeObjectForKey:@"fixTriggers"]];
        
        //Guide 10, chapter 2 Chain of Events     
        [self setSlipEvents:[aDecoder decodeObjectForKey:@"slipEvents"]];
        
        [self setSlipEventDescription:[aDecoder decodeObjectForKey:@"slipEventDescription"]];
        
        [self setSleep:[aDecoder decodeFloatForKey:@"sleep"]];
        
        [self setLeisure:[aDecoder decodeFloatForKey:@"leisure"]];
        
        [self setStrength:[aDecoder decodeFloatForKey:@"strength"]];
        
        [self setFlexibility:[aDecoder decodeFloatForKey:@"flexibility"]];
        
        [self setAnrTime:[aDecoder decodeFloatForKey:@"anrTime"]];
        
        [self setAnrDays:[aDecoder decodeFloatForKey:@"anrDays"]];
        
        [self setEveryday:[aDecoder decodeFloatForKey:@"everyday"]];

        [self setIsVeteran:[aDecoder decodeBoolForKey:@"isVeteran"]];
        
        [self setCaloriesByDateDictionary:[aDecoder decodeObjectForKey:@"caloriesByDateDictionary"]];
        
        [self setWeightsHashByDate:[aDecoder decodeObjectForKey:@"weightsHashByDate"]];
        
        latestEnteredWeight = [aDecoder decodeObjectForKey:@"latestEnteredWeight"];
        
        [self setHasSeenVideoHelp:[aDecoder decodeBoolForKey:@"hasSeenVideoHelp"]];
        
        [self setCalorieGoalOffset:[aDecoder decodeFloatForKey:@"calorieGoalOffset"]];
    }
    
    return self;
}

- (BOOL) saveState {
    if (self.draftWeightGoal && [self.draftWeightGoal isModified]) {
        
        [self.weightGoals addObject:self.draftWeightGoal];
        self.draftWeightGoal = nil;
    }
    if (self.draftActivityGoal && [self.draftActivityGoal isModified]) {
        [self.activityGoals addObject:self.draftActivityGoal];
        self.draftActivityGoal = nil;
    }
    if (self.draftDietGoal && [self.draftDietGoal isModified]) {
        [self.dietGoals addObject:self.draftDietGoal];
        self.draftDietGoal = nil;
    }
    
    return [NSKeyedArchiver archiveRootObject:self toFile:[self userArchiveFilePath]];
}

#pragma mark - Data

- (void) convertAllDataToJson {
    
    //if the EULA is not accepted, there cannot be any data.  If the EULA has been accepted, there may be data.
    if ([[UserData dataStore] eulaComplete]) {
        
        //TODO: create an indicator so JS code can quickly see if any V1 data needs to be parsed
        
        //we will repeat this call to different methods to convert the specific data for domain
        [self writeDictionary:[self convertInitialDataToJson] toDiskWithName:@"initialData.json"];
        
        [self writeDictionary:[self convertWeightDataToJson] toDiskWithName:@"weightData.json"];
        [self writeDictionary:[self convertDietDataToJson] toDiskWithName:@"dietData.json"];    //TODO
        [self writeDictionary:[self convertActivityDataToJson] toDiskWithName:@"activityData.json"]; //TODO
        
        [self writeDictionary:[self convertWeightGoalDataToJson] toDiskWithName:@"weightGoalData.json"];
        [self writeDictionary:[self convertDietGoalDataToJson] toDiskWithName:@"dietGoalData.json"];
        [self writeDictionary:[self convertActivityGoalDataToJson] toDiskWithName:@"activityGoalData.json"];
        
        [self writeDictionary:[self convertGuideDataToJson] toDiskWithName:@"guideData.json"];

        
    } else {
        NSLog(@"There is no data from V1 to convert to JSON");
    }
    
    
    
}

- (NSDictionary *) convertInitialDataToJson {
    
    NSMutableDictionary *initialData = [[NSMutableDictionary alloc] init];
    
    [initialData setObject:@"YES" forKey:@"eulaComplete"];
    
    NSString *initialWarningCompleteString = @"NO";
    if ([[UserData dataStore] initialWarningComplete]) {
        initialWarningCompleteString = @"YES";
    }
    [initialData setObject:initialWarningCompleteString forKey:@"initialWarningComplete"];
    
    if ([[UserData dataStore] joinedMOVE1] && ![[[UserData dataStore] joinedMOVE1] isEqualToString:@"Tap here to enter your first reason for joining MOVE!"] && ![[[UserData dataStore] joinedMOVE1] isEqualToString:@""]) {
        NSLog(@"joined move 1: %@", [[UserData dataStore] joinedMOVE1]);
        [initialData setObject:[[UserData dataStore] joinedMOVE1] forKey:@"joinedMove1"];
    }
    if ([[UserData dataStore] joinedMOVE2] && ![[[UserData dataStore] joinedMOVE2] isEqualToString:@"Tap here to enter your second reason for joining MOVE!"] && ![[[UserData dataStore] joinedMOVE2] isEqualToString:@""]) {
        NSLog(@"joined move 2: %@", [[UserData dataStore] joinedMOVE2]);
        [initialData setObject:[[UserData dataStore] joinedMOVE2] forKey:@"joinedMove2"];
    }
    if ([[UserData dataStore] changeWillHelp1] && ![[[UserData dataStore] changeWillHelp1] isEqualToString:@"Tap here to enter the first thing this change will impact"] && ![[[UserData dataStore] changeWillHelp1] isEqualToString:@""]) {
        NSLog(@"change will help 1: %@", [[UserData dataStore] changeWillHelp1]);
        [initialData setObject:[[UserData dataStore] changeWillHelp1] forKey:@"changeWillHelp1"];
    }
    if ([[UserData dataStore] changeWillHelp2] && ![[[UserData dataStore] changeWillHelp2] isEqualToString:@"Tap here to enter the second thing this change will impact"] && ![[[UserData dataStore] changeWillHelp2] isEqualToString:@""]) {
        NSLog(@"change will help 2: %@", [[UserData dataStore] changeWillHelp2]);
        [initialData setObject:[[UserData dataStore] changeWillHelp2] forKey:@"changeWillHelp2"];
    }
    [initialData setObject:[NSNumber numberWithFloat:[[UserData dataStore] height]] forKey:@"height"];
    [initialData setObject:[NSNumber numberWithInteger:[[UserData dataStore] age]] forKey:@"age"];
    
    return initialData;
}


- (NSDictionary *) convertWeightDataToJson {
    
    NSMutableDictionary *weightData = [[NSMutableDictionary alloc] init];

    for (WeightData *wd in weights) {
        
        //TODO: Format the date key to be usable by JS.  We have to make it a string here, so just convert it to date/time that JS wants to make import easy.
        [weightData setObject:[NSNumber numberWithFloat:wd.userWeight] forKey:[NSString stringWithFormat:@"%@", wd.dateCreated]];
    }
    return weightData;
}

//TODO:convert diet data
- (NSDictionary *) convertDietDataToJson {
    
    NSMutableDictionary *dietData = [[NSMutableDictionary alloc] init];
    
    return dietData;
    
}

//TODO: convert acttivity data
- (NSDictionary *) convertActivityDataToJson {
    
    NSMutableDictionary *activityData = [[NSMutableDictionary alloc] init];
    
    return activityData;
    
}

- (NSDictionary *) convertWeightGoalDataToJson {
    
    NSMutableDictionary *goalData = [[NSMutableDictionary alloc] init];
    for (WeightGoal *wg in weightGoals) {
        
        NSArray *keys = [NSArray arrayWithObjects:@"referenceWeight", @"currentWeight", @"totalWeightLossGoal", @"totalWeightLossGoalPercent", @"targetWeight", @"weightLossRate", nil];
        NSArray *values = [NSArray arrayWithObjects:
                           [NSNumber numberWithFloat:wg.referenceWeight],
                           [NSNumber numberWithFloat:wg.currentWeight],
                           [NSNumber numberWithFloat:wg.totalWeightLossGoal],
                           [NSNumber numberWithFloat:wg.totalWeightLossGoalPercent],
                           [NSNumber numberWithFloat:wg.targetWeight],
                           [NSNumber numberWithFloat:wg.weightLossRate], nil];
        
        [goalData setObject:[NSDictionary dictionaryWithObjects:values forKeys:keys] forKey:[NSString stringWithFormat:@"%@", wg.date]];
        
    }
    return goalData;
}

- (NSDictionary *) convertDietGoalDataToJson {
    
    NSMutableDictionary *goalData = [[NSMutableDictionary alloc] init];
    for (DietGoal *dg in dietGoals) {
        
        NSArray *keys = [NSArray arrayWithObjects:@"goal1", @"goal2", nil];
        NSArray *values = [NSArray arrayWithObjects:dg.goal1, dg.goal2, nil];
        
        [goalData setObject:[NSDictionary dictionaryWithObjects:values forKeys:keys] forKey:[NSString stringWithFormat:@"%@", dg.date]];
        
    }
    return goalData;
}

- (NSDictionary *) convertActivityGoalDataToJson {
    
    NSMutableDictionary *goalData = [[NSMutableDictionary alloc] init];
    for (ActivityGoal *ag in activityGoals) {
        
        NSArray *keys = [NSArray arrayWithObjects:@"goal1", @"goal2", nil];
        NSArray *values = [NSArray arrayWithObjects:ag.goal1, ag.goal2, nil];

        [goalData setObject:[NSDictionary dictionaryWithObjects:values forKeys:keys] forKey:[NSString stringWithFormat:@"%@", ag.date]];
    }
    return goalData;
}

- (NSDictionary *) convertGuideDataToJson {
    
    NSMutableDictionary *guideData = [[NSMutableDictionary alloc] init];
    
    //this will tell us where the user is in using the guides (which guides should be unlocked, what guide is currently active and which chapter was last visited).  we can also capture the GuideSummary information so when the JS side guides are unlocked, the summary objects can be appropriately created.
    
    NSDictionary *guideInfo = [[GuideTimer defaultStore] guideTracker];
    
    [guideData setObject:[NSNumber numberWithInt:[[GuideTimer defaultStore] getCurrentActiveGuide]] forKey:@"currentActiveGuide"];
    
    //Not sure if we will get Chapter data or not.  It may not be worth the effort involved
    
    for (NSNumber *key in [guideInfo allKeys]) {
    
        Guide *g = [guideInfo objectForKey:key];
        
        NSMutableArray *keys = [[NSMutableArray alloc] init];
        NSMutableArray *objects = [[NSMutableArray alloc] init];
        
        if (g.startDate) {
            [keys addObject:@"startDate"];
            [objects addObject:[NSString stringWithFormat:@"%@", g.startDate]];
        }
        if (g.endDate) {
            [keys addObject:@"endDate"];
            [objects addObject:[NSString stringWithFormat:@"%@", g.endDate]];
        }
        
        if (g.summaries.count > 0) {
            //capture summaries with key:date

            NSMutableDictionary *summariesHash = [[NSMutableDictionary alloc] init];
            
            NSArray *summaryKeys = [NSArray arrayWithObjects:@"startingWeight", @"currentWeight",  @"dailyCalorieGoal",  @"bodyMassIndex", @"weightGoal", @"dietGoal", @"activityGoal", nil];
            
            for (GuideSummary *summary in g.summaries) {
                
                NSMutableArray *summaryObjects = [[NSMutableArray alloc] init];
                
                [summaryObjects addObject:[NSNumber numberWithFloat:summary.startingWeight]];
                [summaryObjects addObject:[NSNumber numberWithFloat:summary.currentWeight]];
                [summaryObjects addObject:[NSNumber numberWithFloat:summary.dailyCalorieGoal]];
                [summaryObjects addObject:[NSNumber numberWithFloat:summary.bodyMassIndex]];
                
                //weight Goal.
                NSArray *weightGoalKeys = [NSArray arrayWithObjects:@"referenceWeight", @"currentWeight", @"totalWeightLossGoal", @"totalWeightLossGoalPercent", @"targetWeight", @"weightLossRate", @"date", nil];
                NSArray *weightGoalValues = [NSArray arrayWithObjects:
                                   [NSNumber numberWithFloat:summary.weightGoal.referenceWeight],
                                   [NSNumber numberWithFloat:summary.weightGoal.currentWeight],
                                   [NSNumber numberWithFloat:summary.weightGoal.totalWeightLossGoal],
                                   [NSNumber numberWithFloat:summary.weightGoal.totalWeightLossGoalPercent],
                                   [NSNumber numberWithFloat:summary.weightGoal.targetWeight],
                                   [NSNumber numberWithFloat:summary.weightGoal.weightLossRate],
                                   [NSString stringWithFormat:@"%@",summary.weightGoal.date], nil];
                
                [summaryObjects addObject:[NSDictionary dictionaryWithObjects:weightGoalValues forKeys:weightGoalKeys]];
                
                //Diet goal
                NSArray *dietGoalKeys = [NSArray arrayWithObjects:@"goal1", @"goal2", nil];
                NSArray *dietGoalValues = [NSArray arrayWithObjects:summary.dietGoal.goal1, summary.dietGoal.goal2, nil];
                [summaryObjects addObject:[NSDictionary dictionaryWithObjects:dietGoalValues forKeys:dietGoalKeys]];
                
                //Activity goal
                NSArray *activityGoalKeys = [NSArray arrayWithObjects:@"goal1", @"goal2", nil];
                NSArray *activityGoalValues = [NSArray arrayWithObjects:summary.activityGoal.goal1, summary.activityGoal.goal2, nil];
                [summaryObjects addObject:[NSDictionary dictionaryWithObjects:activityGoalValues forKeys:activityGoalKeys]];
                

                
                [summariesHash setObject:[NSDictionary dictionaryWithObjects:summaryObjects forKeys:summaryKeys] forKey:[NSString stringWithFormat:@"%@", summary.dateTime]];
                
            }
            
            //after all summaries have been JSONified, add to the hash for this guide
            [keys addObject:@"summaries"];
            [objects addObject:summariesHash];
        } //end of summaries
        
        if (g.progressCheckIn) {
            
            //progress check in for Guides 7 and 11 (index 6 and 10)
            NSString *meetingActivityGoal = @"No";
            if (g.progressCheckIn.prActivityGoal) meetingActivityGoal = @"Yes";
            NSString *meetingDietGoal = @"No";
            if (g.progressCheckIn.prFoodGoal) meetingDietGoal = @"Yes";
            
            NSArray *progressCheckInKeys = [NSArray arrayWithObjects:@"meetingActivityGoal", @"meetingDietGoal", @"activityChanges", @"activityImprovements", @"dietChanges", @"dietImprovements", nil];
            NSArray *progressCheckInValues = [NSArray arrayWithObjects:meetingActivityGoal, meetingDietGoal, g.progressCheckIn.prActChange, g.progressCheckIn.prActImprove, g.progressCheckIn.prDietChange, g.progressCheckIn.prFoodImprove, nil];
           
            NSDictionary *progressCheckInHash = [NSDictionary dictionaryWithObjects:progressCheckInValues forKeys:progressCheckInKeys];
            
            [keys addObject:@"progressCheckIn"];
            [objects addObject:progressCheckInHash];
        }
        
        [guideData setObject:[NSDictionary dictionaryWithObjects:objects forKeys:keys] forKey:[NSString stringWithFormat:@"%@", key]];
    }
    
    return guideData;
    
}


/*
 This method will take a dictionary object and convert it to JSON and write that JSON object to device disk.
 The purpose of this method is to make this data collected in MCM V1 native code and stored as NSArchives available to MCM V2 as JSON objects.
 
 @param: data - An NSDictionary object containing data in key/value pairs
 @param: name - A meaningful name to describe the key/value pairs in 'data'.  This variable should include the .json file extenstion
 
 @result: a file with 'name' will be written to the device disk in the documents directory and contain the key/value pairs found in 'data' formatted as valid json.
 
 */
- (void) writeDictionary:(NSDictionary *)data toDiskWithName:(NSString *)name {
    
    //Get the path to the documents directory.  Javascript side will have access to this directory
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    
    //Create the meaningful file name at filePath
    NSString *filePath = [documentsDirectory stringByAppendingPathComponent:name];
    
    //Create the Output steam connected to the fileName and path
    NSOutputStream *os = [[NSOutputStream alloc] initToFileAtPath:filePath append:NO];
    
    //Write the dictionary as a JSON object to disk
    [os open];
    [NSJSONSerialization writeJSONObject:data toStream:os options:0 error:nil];
    [os close];
    
    
    //Log the write to disk.  Remove for Production
    NSError *error;
    NSData *jsonData=[NSJSONSerialization dataWithJSONObject:data options:NSJSONWritingPrettyPrinted error:&error];
    NSString *resultAsString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
    NSLog(@"Wrote %@ to disk with contents: %@", name, resultAsString);
    //
    
}










- (void)populateTempData
{
    
    //weights
    for (int i = 250; i > 180; i = i - 6) {
        float f = (float)i;
        [self addWeight:f andDate:nil];
    }
    
    // Height
    [self setHeight:75.5];
    
    // Age
    [self setAge:37.0];

    // Gender
    [self setGender:@"Male"];
    
    // Initial WeightGoal
    [self addWeightGoal:[self getStartingWeight]];
    WeightGoal *initialWeightGoal = [self getStartingWeightGoal];
    
    // Weight Loss Rate
    [initialWeightGoal setWeightLossRate:2.0];
    
    // Total Weight loss goal
    [initialWeightGoal setTotalWeightLossGoal:75.0];
    
    // Diet SMART Goals
    
    [[self getCurrentDietGoal] setGoal1:@"Enter your dietary SMART goals"];
    [[self getCurrentDietGoal] setGoal2:@"Enter your dietary SMART goals"];
    
    // Activity SMART Goals
    [[self getCurrentActivityGoal]setGoal1:@"Enter your physical activity SMART goals"];
    [[self getCurrentActivityGoal]setGoal2:@"Enter your physical activity SMART goals"];
    
    // Motivations
    [self setJoinedMOVE1:@"Enter why you joined MOVE!"];
    [self setJoinedMOVE2:@"Enter why you joined MOVE!"];
    
    [self setChangeWillHelp1:@"Enter what you think change will help you to do"];
    [self setChangeWillHelp2:@"Enter what you think change will help you to do"];
    
    // Calorie inputs
    [self setCaloriesBurned:500.0];
    [self setCaloriesConsumed:3000.0];
    
    // Activity Factor
    // linear calculation of activity factor (to be removed)
    float linearAF = (0.0004608262 * [self caloriesBurned]) + 1.2;
    [self setActivityFactor:linearAF];

}

-(NSString *)description
{
    for (WeightData *w in [self weights]) {
        NSLog(@"%@",[w description]);
    }

    return nil;
}






@end
