//
//  WeightData.h
//  MoveCoachMobile
//
//  Created by Alex Morgan on 8/30/12.
//

#import <Foundation/Foundation.h>

/**
 *  This Class is part of the Data Model and represents a single instance of a user weight.  The UserData Class maintains an array of WeightData objects to provide a history of user weights
 */
@interface WeightData : NSObject <NSCoding>

/**
 *  This property represents the date the weight was entered into the system
 */
@property (nonatomic, strong)NSDate *dateCreated;

/**
 *  This property represents the user's weight in pounds.
 */
@property (nonatomic)float userWeight;

/**
 *  This is the primary method to instantiate a weight object with weight and set dateCreated to the current date.
 *
 *  @param weight - The user's current weight
 */
- (id)initWithWeight:(float)weight;

/**
 *  This method allows a weightData object to be created with a specific date.  This is currently only used in the test harness to auto populate weight data into the past.
 *
 *  @param weight - THe user's current weight
 *
 *  @date - a specific date to set dateCreated to.
 */
- (id)initWithWeight:(float)weight andDate:(NSDate *)date;


//- (void) removeFromWeightsDictionary;
@end
