//
//  WeightGoal.m
//  MoveCoachMobile
//
//  Created by Joel Russell on 9/4/13.
//

#import "WeightGoal.h"

@interface WeightGoal() {
    
    NSString *lastUpdatedGoalType;
}

    -(void)calculateExpectedGoalCompletionDate:(NSDate *)date;

@end

@implementation WeightGoal

@synthesize date, referenceWeight, currentWeight, totalWeightLossGoal, totalWeightLossGoalPercent, targetWeight, weightLossRate, modified;//, expectedGoalCompletionDate;

- (id)init
{
    return [self initWithReferenceWeight:0.0];
}

- (id)initWithReferenceWeight:(float)aRefWeight
{
    return [self initWithReferenceWeight:aRefWeight andCurrentWeight:0.0 andDate:[NSDate date]];
}

- (id)initWithReferenceWeight:(float)aRefWeight andCurrentWeight:(float)aCurrWeight andDate:(NSDate *)aDate
{
    self = [super init];
    
    if (self) {
        self.referenceWeight = aRefWeight;
        self.currentWeight = aCurrWeight;
        self.date = aDate;
        self.totalWeightLossGoalPercent = 5;
        self.weightLossRate = .5;
    }
    
    return self;
}
    
- (id)initWithCurrentWeight:(float)aCurrWeight andDate:(NSDate *)aDate andPercentToLose:(float)aPercentToLose andWeightLossRate:(float)aWeightLossRate {
    
    self = [super initWithGoal1Text:@"" andGoal2Text:@"" andDate:aDate];
    
    if (self) {
        self.referenceWeight = aCurrWeight;
        self.currentWeight = aCurrWeight;
        self.date = aDate;
        super.date = aDate;
        self.totalWeightLossGoalPercent = aPercentToLose;
        self.weightLossRate = aWeightLossRate;
    }
    [self calculateExpectedGoalCompletionDate:aDate];
    return self;
}

- (id)initWithWeightGoal:(WeightGoal *)aWeightGoal
{
    self = [super init];
    
    if (self) {
        date = [NSDate date];
        referenceWeight = aWeightGoal.referenceWeight;
        currentWeight = aWeightGoal.currentWeight;
        totalWeightLossGoal = aWeightGoal.totalWeightLossGoal;
        totalWeightLossGoalPercent = aWeightGoal.totalWeightLossGoalPercent;
        targetWeight = aWeightGoal.targetWeight;
        weightLossRate = aWeightGoal.weightLossRate;
        self.expectedGoalCompletionDate = aWeightGoal.expectedGoalCompletionDate;
        lastUpdatedGoalType = @"";
    }
    
    return self;
}

- (id) initWithCoder:(NSCoder *)aDecoder {
    self = [super init];
    
    if (self) {
        [self setDate:[aDecoder decodeObjectForKey:@"date"]];
        [self setReferenceWeight:[aDecoder decodeFloatForKey:@"referenceWeight"]];
        [self setCurrentWeight:[aDecoder decodeFloatForKey:@"currentWeight"]];
        [self setTotalWeightLossGoal:[aDecoder decodeFloatForKey:@"totalWeightLossGoal"]];
        [self setTotalWeightLossGoalPercent:[aDecoder decodeFloatForKey:@"totalWeightLossGoalPercent"]];
        [self setTargetWeight:[aDecoder decodeFloatForKey:@"targetWeight"]];
        [self setWeightLossRate:[aDecoder decodeFloatForKey:@"weightLossRate"]];
        [self setExpectedGoalCompletionDate:[aDecoder decodeObjectForKey:@"expectedGoalCompletionDate"]];
        lastUpdatedGoalType = [aDecoder decodeObjectForKey:@"lastUpdatedGoalType"];
        
        [self setCalendarEventIdentifier:[aDecoder decodeObjectForKey:@"calendarEventIdentifier"]];
    }
    return self;
}

- (void)setReferenceWeight:(float)refWt
{
    if ((referenceWeight != 0.0) && (referenceWeight != refWt)) {
        self.modified = YES;
    }
    referenceWeight = refWt;
    if (referenceWeight != 0.0) {
        if (currentWeight == 0.0) {
            currentWeight = referenceWeight;
        }
        
        if ([lastUpdatedGoalType isEqualToString:@"targetWeight"]) {
            [self setTargetWeight:targetWeight];
        } else if ([lastUpdatedGoalType isEqualToString:@"ratePounds"]) {
            [self setTotalWeightLossGoal:totalWeightLossGoal];
        } else if ([lastUpdatedGoalType isEqualToString:@"ratePercent"] || [lastUpdatedGoalType isEqualToString:@""]) {
            [self setTotalWeightLossGoalPercent:totalWeightLossGoalPercent];
        }
        [self calculateExpectedGoalCompletionDate:Nil];
    }
}

- (void)setTotalWeightLossGoalPercent:(float)pctLoss
{
    if ((totalWeightLossGoalPercent != 0.0) && (totalWeightLossGoalPercent != pctLoss)) {
        self.modified = YES;
    }
    totalWeightLossGoalPercent = pctLoss;

    lastUpdatedGoalType = @"ratePercent";
    
    float sf = 1.00 - (pctLoss/100);
    targetWeight = sf * referenceWeight;
    totalWeightLossGoal = referenceWeight - targetWeight;
    [self calculateExpectedGoalCompletionDate:Nil];
}

- (void)setTotalWeightLossGoal:(float)lbsToLose
{
    if ((totalWeightLossGoal != 0.0) && (totalWeightLossGoal != lbsToLose)) {
        self.modified = YES;
    }
    totalWeightLossGoal = lbsToLose;

    lastUpdatedGoalType = @"ratePounds";

    
    targetWeight = referenceWeight - lbsToLose;
    totalWeightLossGoalPercent = ((lbsToLose / referenceWeight) * 100);
    if (isnan(totalWeightLossGoalPercent)) {
        totalWeightLossGoalPercent = 0.0;
    }
    [self calculateExpectedGoalCompletionDate:Nil];
}

- (void)setTargetWeight:(float)tw
{
    if ((targetWeight != 0.0) && (targetWeight != tw)) {
        self.modified = YES;
    }
    targetWeight = tw;
    
    lastUpdatedGoalType = @"targetWeight";
    
    totalWeightLossGoal = referenceWeight - tw;
    totalWeightLossGoalPercent = ((totalWeightLossGoal / referenceWeight) * 100);
    if (isnan(totalWeightLossGoalPercent)) {
        totalWeightLossGoalPercent = 0.0;
    }
    [self calculateExpectedGoalCompletionDate:Nil];
}

- (void)setWeightLossRate:(float)wlr
{
    if ((weightLossRate != 0.0) && (weightLossRate != wlr)) {
        self.modified = YES;
    }
    weightLossRate = wlr;
    [self calculateExpectedGoalCompletionDate:Nil];
}

- (void)setCurrentWeight:(float)cwt
{
    currentWeight = cwt;
    if (currentWeight != 0.0) {
        if (referenceWeight == 0.0) {
            referenceWeight = currentWeight;
        }
    }
    [self calculateExpectedGoalCompletionDate:Nil];
}

/**
 * Private method to calculate expectedGoalCompletionDate
 */
    - (void)calculateExpectedGoalCompletionDate:(NSDate *)aDate
{

    NSDate *currentDate;
    if (!aDate) {
        currentDate = [NSDate date];
    } else {
        currentDate = aDate;
    }

    if (weightLossRate == 0.0 || referenceWeight <= targetWeight){
        self.expectedGoalCompletionDate = currentDate;
    } else {
        float poundsToGoalWeight = referenceWeight - targetWeight;
        float weeksToGoal = poundsToGoalWeight / weightLossRate;
        int daysToGoal = [[NSNumber numberWithFloat:(weeksToGoal * 7.0)] integerValue];
        self.expectedGoalCompletionDate = [NSDate dateWithTimeInterval:daysToGoal * SECONDS_PER_DAY sinceDate:currentDate];
    }

    //This code uses the current entered weight to calculate how long until the goal is reached.  This was decided to be unintuiitive to the user.  A goal is set with a baseline weight, and that is the weight that will be used as the starting weight for a particular weight goal.  However, this was a tricky issue to decide, so if we need to revert to using the user entered weight to compute time until goal complete, then replace the code above with the commented out code below.
//    if (weightLossRate == 0.0 || currentWeight <= targetWeight){
//        expectedGoalCompletionDate = currentDate;
//    } else {
//        float poundsToGoalWeight = currentWeight - targetWeight;
//        float weeksToGoal = poundsToGoalWeight / weightLossRate;
//        int daysToGoal = [[NSNumber numberWithFloat:(weeksToGoal * 7.0)] integerValue];
//        expectedGoalCompletionDate = [NSDate dateWithTimeInterval:daysToGoal * SECONDS_PER_DAY sinceDate:currentDate];
//    }
}

- (NSString *)description
{
    return [NSString stringWithFormat:@"\nDate: %@ \nBaseline Weight: %.1f \nCurrent Weight: %.1f \nTotal Weight Loss Goal: %.1f \nTotal Weight Loss Goal Pct: %.1f \nTarget Weight: %.1f \nWeight Loss Rate: %.1f \nCompletion Date: %@\n\n", [self date], [self referenceWeight], [self currentWeight], [self totalWeightLossGoal], [self totalWeightLossGoalPercent], [self targetWeight], [self weightLossRate], [self expectedGoalCompletionDate]];
}

- (void) encodeWithCoder:(NSCoder *)aCoder {
    
    [aCoder encodeObject:date forKey:@"date"];
    [aCoder encodeFloat:referenceWeight forKey:@"referenceWeight"];
    [aCoder encodeFloat:currentWeight forKey:@"currentWeight"];
    [aCoder encodeFloat:totalWeightLossGoal forKey:@"totalWeightLossGoal"];
    [aCoder encodeFloat:totalWeightLossGoalPercent forKey:@"totalWeightLossGoalPercent"];
    [aCoder encodeFloat:targetWeight forKey:@"targetWeight"];
    [aCoder encodeFloat:weightLossRate forKey:@"weightLossRate"];
    [aCoder encodeObject:self.expectedGoalCompletionDate forKey:@"expectedGoalCompletionDate"];
    [aCoder encodeObject:lastUpdatedGoalType forKey:@"lastUpdatedGoalType"];
    [aCoder encodeObject:self.calendarEventIdentifier forKey:@"calendarEventIdentifier"];
}

@end
