define(function(require) {
    var log = require("loglevel"),
        UserData = require("models/UserData"),
        ProgressData = require("models/ProgressReviewModel"),
        Calculator = require("utilities/calculators");

	// Need an array of these for each guide for when user starts guides over
    var GuideSummaryModel = Backbone.Model.extend({

    	id: "GuideSummaryModel", 
    	
    	defaults: {
    		guideId: "",
			dateCompleted: "",
			weight: "",
			bmi: "",
			lossGoal: "",
			lossGoalPct: "",
			lossRate: "",
			calorieGoal: "",
			mca: "",
			dietSmartGoal1: "",
			dietSmartGoal2: "",
			paSmartGoal1: "",
			paSmartGoal2: "",
			// progress review check-in
			activityChangesPR: [],
			dietChangesPR: [],
			activityGoalChangesPR: [],
			dietGoalChangesPR: []
    	},

		keyPoints: [
			// <key>Guide 0</key>
			[
				"Learned tips for successful weight management",
				"Learned strategies for self-management ",
				"Set SMART goals for weight management",
				"Started keeping a food and physical activity diary",
			],
			// <key>Guide 1</key>
			[
				"Made a plan using your daily calorie goal",
				"Practiced reading food labels",
				"Started building a healthy plate",
				"Played a game to learn serving sizes using everyday items",
			],
			// <key>Guide 2</key>
			[
				"Reviewed ways to increase physical activity and keep up with your target calorie goal",
				"Set your new physical activity goal to increase the calories you are burning",
				"Started your physical activity program and started wearing your pedometer",
				"Learned how to warm-up, cool-down, and stretch",
			],
			// <key>Guide 3</key>
			[
				"Used your healthy plate to build healthy, portion-controlled meals",
				"Learned the importance of daily record keeping",
				"Learned how to pay attention to what you are eating and how to manage feelings of hunger and fullness",
			],
			// <key>Guide 4</key>
			[
				"Played a shake game that offered new activities to keep you interested and motivated while adding lifestyle activities ",
				"Learned how to plan around physical activity barriers",
				"Considered upping the ante to increase your physical activity",
			],
			// <key>Guide 5</key>
			[
				"Identified the type of fats that are in your food and drinks and learned how to trim fat from your diet",
				"Learned about fats that are healthier for your heart",
				"In this Guide, we talked about physical activity and safety",
			],
			// <key>Guide 6</key>
			[
				"Learned how to be safe while being physically active",
				"Learned that some activity is better than none",
				"Reviewed frequency, intensity, time, and type of physical activity to determine how hard you should be exercising",
			],
			// <key>Guide 7</key>
			[
				"Learned that planning ahead is an important factor for weight management",
				"Learned tips for eating wisely at home, in a restaurant, and on a budget",
				"Learned healthy alternatives to fast food",
				"Revised your food and physical activity goals and learned ways to be physically active throughout the day",
			],
			// <key>Guide 8</key>
			[
				"Identified cues in the environment that may be influencing your eating, drinking, and physical activity",
				"Learned tips for eating with others and how to take control",
			],
			// <key>Guide 9</key>
			[
				"Identified potential slips and how to handle the slips",
				"Practiced ways to break the chain of events",
				"Learned tips for taking control of your thoughts, feelings, and behavior",
				"Increased your physical activity goals if needed by upping the ante",
			],
			// <key>Guide 10</key>
			[
				"Reviewed your progress and listed changes you’ve made to be more active and to make healthier food choices",
				"Learned tips for coping with a weight plateau, how to stay motivated, and learned ways to maintain your long-term weight loss",
			]
		],

    	initialize: function(guideId, completedDate) {
    		// pass in guide id for keyPoints
    		console.log("in GuideSummaryModel init: " + guideId + ", date: " + completedDate);
    		
    		var formatValue = function(value, decimals) {
    			// may want to deal with NaN
    			return Number(value).toFixed(decimals); 
    		};

    		var initialWeight = Number( UserData.get("initialWeight") );
    		var currentWeight = Number( UserData.get("currentWeight") );
    		// var currentWeight = Number( UserData.getCurrentWeight() );

    		// storing values as strings for consistent formatting in reports and pages
    		this.set("guideId", guideId);
    		this.set("dateCompleted", completedDate.toDateString());

    		this.set("weight", currentWeight.toFixed(1));
    		this.set("weightLost", ( currentWeight - initialWeight ).toFixed(1)); // lost weight is negative value

    		this.set("lossGoal", UserData.get("initialLBSToLose")); // not sure why these are called "initial"
    		this.set("lossGoalPct", UserData.get("initialPercentToLose"));
    		this.set("lossRate", UserData.get("initialWeightLossRange"));
    		this.set("calorieGoal", formatValue( UserData.get("dailyCalorieGoal"), 0 ));
    		this.set("mca", UserData.get("mcaValue"));
    		this.set("dietSmartGoal1", UserData.get("dietSmartGoal1"));
    		this.set("dietSmartGoal2", UserData.get("dietSmartGoal2"));
    		this.set("paSmartGoal1", UserData.get("paSmartGoal1"));
    		this.set("paSmartGoal2", UserData.get("paSmartGoal2"));

    		// only show BMI for odd numbered guides
    		if ( guideId & 1 )
    		{
    			var bmi = Calculator.currentBMI();
	    		this.set("bmi", bmi);
			}
    		
    		this.set("keyPoints", this.keyPoints[guideId -1]); // keyPoints is 0 based
   		
   			// populate progress review check in
   			if (guideId == 7)
   			{
	    		this.set("activityChangesPR", ProgressData.get("activityChanges7"));
	    		this.set("dietChangesPR", ProgressData.get("dietChanges7"));
	    		this.set("activityGoalChangesPR", ProgressData.get("activityGoalChanges7"));
	    		this.set("dietGoalChangesPR", ProgressData.get("dietGoalChanges7"));
	    		this.set("reachingActivityGoalsPR", ProgressData.get("reachingActivityGoals7"));
	    		this.set("reachingDietGoalsPR", ProgressData.get("reachingDietGoals7"));
   			}
   			else if (guideId == 11)
			{
	    		this.set("activityChangesPR", ProgressData.get("activityChanges11"));
	    		this.set("dietChangesPR", ProgressData.get("dietChanges11"));
	    		this.set("activityGoalChangesPR", ProgressData.get("activityGoalChanges11"));
	    		this.set("dietGoalChangesPR", ProgressData.get("dietGoalChanges11"));
	    		this.set("reachingActivityGoalsPR", ProgressData.get("reachingActivityGoals11"));
	    		this.set("reachingDietGoalsPR", ProgressData.get("reachingDietGoals11"));
			}

	   		console.log("guide summary model init: ", this);

    	}


	});
	
	return GuideSummaryModel;
});

