define(function(require) {
    var log = require("loglevel"),
	    Handlebars = require("handlebars"),
        UserData = require("models/UserData"),
        MCM_Keyboard_Plugin = require("plugin/MCM_Keyboard_Plugin"),
        template = require("text!templates/CurrentWeightEntryTemplate.html");

    // Compile the template once.
    var compiled_template = Handlebars.compile(template);

	var textString = "";

    var CurrentWeightEntryView = Backbone.View.extend({
        id: "currentWeightEntryView",
        
        template: "CurrentWeightEntryTemplate.html",
        
        events: {
              "click #callKeyboard": "decimalKeypadRequired"
        },         

        initialize: function() {			
            if (!UserData.get("weights")) {
                UserData.set ("weights", []);
                UserData.save();
            }
        },
		render: function() {

            this.$el.html(compiled_template(this.model.toJSON()));
            textString = "";
            
            if (UserData.get("weights").length > 0){
                var weightsCount = UserData.get("weights").length;
            	textString = UserData.get("weights")[weightsCount-1].weight;            	
            } else {   
        	   textString = "00.0";
            }

            this.$('#lbsDisplay').replaceWith("<span id='lbsDisplay'>" + textString + "</span>");

//ISSUE: Keyboard disappears if user taps anywhere on screen or navigates dates     
//    			this.decimalKeypadRequired();
     
            return this;
		},

        addWeight: function(date) {
                             
            var localWeights = UserData.get("weights");
            var newWeightDate = new Date(date);
            
            if (localWeights.length > 0) {
           
                var firstDate = new Date(localWeights[0].date);
                var lastDate = new Date(localWeights[localWeights.length-1].date);
                
                if (firstDate > newWeightDate) {
                    //add to beginning of array
                    localWeights.unshift({weight:textString, date:newWeightDate});
                } else if (lastDate < newWeightDate) {
                    //add to end of array
                    localWeights.push({weight:textString, date:newWeightDate});                    
                } else {
                    //manaul insert                    
                    for (var i=0; i<localWeights.length; i++) {
                                                
                        var compareDate = new Date(localWeights[i].date);
                        if (newWeightDate < compareDate) {
                            localWeights.splice(i, 0, {weight:textString, date:newWeightDate});
                            break;
                        }   
                    }                    
                }        
            } else {
                localWeights.push({weight:textString, date:newWeightDate});                    
            }
            UserData.save("weights", localWeights);   
            
            var locWs = UserData.get("weights");
            
            console.log ("************Testing Weight Entry***********************************");
            console.log ("*******************************************************************");
            for (var i=0; i<locWs.length; i++) {
                console.log ("weights[" + i + "]: " + JSON.stringify(locWs[i]));
            }
            console.log ("*******************************************************************");
            console.log ("*******************************************************************");        
            
            UserData.set("currentWeight", locWs[locWs.length - 1].weight);
            UserData.save();
            
            console.log("Current Weight: " + UserData.get("currentWeight"));
        },
        
        decimalKeypadRequired: function(){
            return;
            //callback function for keyboard (this sends keyboard entries to the desired HTML tag to be changed)
            function onEdit(results){
                        
                // displays text typed in
                this.$('#lbsDisplay').html(results);
                    
                // sets textString variable to set weight in UserData when done button tapped
                textString = "";
                textString = results;
            }                                
            MCM_Keyboard_Plugin.getKeyboard(
                onEdit,     // callback function
                [0,0],      // native textfield location [x,y]          ** required **
                [0,0],      // native textfield size [width, height]    ** required **
                "",         // current text in native textfield
                "",        // placeholder text in native textfield
                "Decimal"  // keyboard type in this case UIKeyboardTypeDecimalPad
            );              
        }
    });

    return CurrentWeightEntryView;
});
