define(function(require) {
    var log = require("loglevel"),
        UserData = require("models/UserData"),
        //WeightEntry = require('views/CurrentWeightEntryView'),
        WeightEntry = require('views/WeightEntryListView'),
        PAData = require("models/PAData"),
        PortionData = require("models/PortionData"),
        // DietEntry = require('views/DietEntryView'),
        DietEntry = require('views/FoodListView'),
        ActivityEntry = require('views/PAListView'),   //TODO: this throws error.  FIX PAEntryView.js code
        template = require("text!templates/entryHomeContainer.html");

//for these so important screens, we could create a template for phone and another template for tablet and distinguish between them here.

    var customViews = {
        'weightEntry': function () {return new WeightEntry();},
        'DietEntry': function() {return new DietEntry({model: UserData});},
        'ActivityEntry': function() {return new ActivityEntry({model: UserData});}
    };
        

    var EntryHomeContainerView = Backbone.View.extend({
        id: "entryHomeContainerView",
        
        initialize: function(args) {
            
            console.log ("args: " + JSON.stringify(args));
            
           if (args.weightLog) {
               this.weightLog = true;
           }
            
        },
        
        template: template,
        
        events: {
            "click #weightButton": "displayWeightEntry",
            "click #dietButton": "displayDietEntry",
            "click #activityButton": "displayActivityEntry",
            "click #nextDate": function(ev) {this.updateDate("next");},
            "click #previousDate": function(ev) {this.updateDate("previous");},
            
            "click #selectedDate": function(ev) {this.selectDate(ev);},
            
            "click #entryContainerDone": "processDoneButton",
            "click #entryContainerBack": "processDoneButton",
             
            'swipeleft ul#activityList li#selectableCells': function(ev){this.getIndexAndRemove(ev, "removeActivity");},
            'swiperight ul#activityList li#selectableCells': function(ev){this.getIndexAndRemove(ev, "removeActivity");},
            'swipeleft ul#foodList li#selectableCells': function(ev){this.getIndexAndRemove(ev, "removeFood");},
            'swiperight ul#foodList li#selectableCells': function(ev){this.getIndexAndRemove(ev, "removeFood");}
        },

        render: function() {
            this.$el.html(template);

            //we will be able to simplify this when adding support for diet and activity log views.  
            if (this.weightLog) {
                this.$('#wda_navBar').hide();
                this.$('#entryContainerDone').hide();
                this.$('#entryContainerBack').show();
                this.defaultType = "weightEntry";
            } else {
                //entry view
                this.scrollPosition = 142;  //this scrolls past the entry li and the wda_navBar
                this.$('#wda_navBar').show();
                this.$('#entryContainerDone').show();
                this.$('#entryContainerBack').hide();
            }

            //is this needed?
			UserData.set("isEditing", false);

			/*
			 * On rendering this page (which means loading it for the first time, NOT switching between entry types) we set the date to today.
			 */			
			this.currentDate = new Date();			
			this.$('#selectedDate').attr("data-date", this.currentDate);
			this.$('#selectedDate').text("Today");

            UserData.set('entryDate',this.currentDate);
            UserData.save();
                        
            if (this.$('#weightButton').hasClass("ui-btn-active")) {
                this.$('#nextDate').hide();                
            }
            
            this.defaultType = UserData.get('entryType');  // defaults to last used view                                    
            
            if ("weightEntry" === this.defaultType) {            	
                this.displayWeightEntry();
            } else if ("dietEntry" === this.defaultType) {
                this.displayDietEntry();
            } else if ("activityEntry" === this.defaultType) { 
                this.displayActivityEntry();
            } else {
                this.defaultType = "weightEntry";
                UserData.set('entryType',this.defaultType);
                UserData.save();
            	
            	this.displayWeightEntry();
            }           
            return this;
        },
      
        /*
         * support methods
         */
        //This method is invoked from the childview
        processDoneButton: function() {          
          
          if ("weightEntry" === this.type) {             
              this.childView.addWeight();
              
          } else {
              console.log ("processing a different entry");
          }
          
          $.mobile.changePage.defaults.reverse = true;
          
          UserData.set('entryDate',undefined);
            UserData.save();
            
          Backbone.history.history.back();
        },

      
        /*
         * Dom manipulation methods
         */
        deactivateEntryButtons: function() {
            this.$('#weightButton').removeClass("ui-btn-active");
            this.$('#dietButton').removeClass("ui-btn-active");
            this.$('#activityButton').removeClass("ui-btn-active");

        },
        
        displayWeightEntry: function() {
            this.deactivateEntryButtons();
            this.$('#weightButton').addClass("ui-btn-active");
            this.$('#calBarListview').removeClass("calBarListviewShown");
            this.$('#calBarListview').addClass("calBarListviewHidden");
            this.$('#dietEntryDiv').hide();
            this.$('#activityEntryDiv').hide();

            this.$('#weightEntryDiv').html("");

            //only init the child object once
            if ("weightEntry" !== this.type) {
                this.type = "weightEntry";
                this.childView = customViews[this.type]();
                this.childView.parent = this;
            }

            //but rerender it with new data every time
            this.$('#weightEntryDiv').append(this.childView.render().el);
            this.$('#weightEntryDiv').show().css("display","");
            
            var currentDate = new Date(this.$('#selectedDate').attr("data-date"));
            var today = new Date();            
                        
            if (currentDate >= today || convertDateToString(currentDate, "veryVeryShort") == convertDateToString(today, "veryVeryShort")){                                    
            
                	this.$('#selectedDate').text("Today");
                	this.$('#selectedDate').attr("data-date", new Date());
                	
                	this.$('#nextDate').hide();
              
                //TODO: do we need this?  	
                UserData.set('entryDate',this.currentDate);
                UserData.save();
            
             } else {
             	this.$('#nextDate').show();
             }
            
            $('body').enhanceWithin();
            
            //lets find a better place to store this
            UserData.set('entryType',"weightEntry");
            UserData.save();

        },
        
        displayDietEntry: function() {
            this.deactivateEntryButtons();
            this.$('#dietButton').addClass("ui-btn-active");
            this.$('#calBarListview').removeClass("calBarListviewHidden");
            this.$('#calBarListview').addClass("calBarListviewShown");
            this.$('#weightEntryDiv').hide();
            this.$('#activityEntryDiv').hide();
            this.$('#nextDate').show();
            
             // saves weight if coming from weight page and weight is different than previously recorded that day
            
            if(this.type === "weightEntry"){
            	var existingWeight = false;
            	
            	var weights = [];
            	weights = UserData.get("weights");
            	
            	for(var i=0;i < weights.length; i++){

            		var recDate = new Date(weights[i].date);
            		
            		if(this.$('#lbsDisplay').html() == weights[i].weight && convertDateToString(this.currentDate, "veryVeryShort") == convertDateToString(recDate, "veryVeryShort")){
            		
            			existingWeight = true;
            		}
            	}
            	
            	if(existingWeight == false){
            		
            		this.childView.addWeight(this.currentDate);	
            	}
            	existingWeight = false;
            }
            
            
            this.$('#dietEntryDiv').replaceWith('<div id="dietEntryDiv" style="width:100%"></div>');
            
            //TODO: get current diet values from data model/DB and populate the accordian list view, and then attach to the list
            //TODO: will also need event handlers to handle entering a new item.  
            this.type = "DietEntry";    //once dietEntry is set up, change this.  set to weight to avoid crash
            this.childView = customViews[this.type]();
            
            //append the html returned from DietView.render() to the listview.  dietView.render might just return a sting containing a bunch of li elements, or similar
            this.$('#dietEntryDiv').append(this.childView.render().el);
            this.$('#dietEntryDiv').show();

			if(!UserData.get("activityCalories")){
				UserData.set("activityCalories", 0);
			}
			
			if(!UserData.get("foodCalories")){
				UserData.set("foodCalories", 0);
			}
			
			if(!UserData.get("mcaValue")){
				UserData.set("mcaValue", 0);
			}       
			
			if(!UserData.get("initialWeightLossRange")){
				UserData.set("initialWeightLossRange", 0.5);
			}
			
			
			if(!UserData.get("currentWeight")){
				UserData.set("currentWeight", 0.0);
			}
			
			UserData.save();
			
			var goalCal = UserData.calculateCalorieGoal(UserData.get("initialWeightLossRange"),UserData.get("currentWeight"));
			
			var foodCal = UserData.get("foodCalories");
			
			var paCal = UserData.get("activityCalories");
			
			var budgetCalories = Number(goalCal) + Number(UserData.get("mcaValue"));
			
			var remainingCalories = Number(budgetCalories) + Number(paCal) - Number(foodCal);
			
			remainingCalories = remainingCalories.toFixed(0);
			
			this.$("#budgetCalories").html(budgetCalories);
			this.$("#foodCalories").html(UserData.get("foodCalories").toFixed(0));
			this.$("#activityCalories").html(UserData.get("activityCalories"));
			this.$("#remainingCalories").html(remainingCalories);
            
            // refresh page so jquery works
            $('body').enhanceWithin();

            
            UserData.set('entryType',"dietEntry");
            UserData.save();
        },
        
        displayActivityEntry: function() {
            this.deactivateEntryButtons();
            this.$('#activityButton').addClass("ui-btn-active");
            this.$('#calBarListview').removeClass("calBarListviewHidden");
            this.$('#calBarListview').addClass("calBarListviewShown");
            this.$('#weightEntryDiv').hide();
            this.$('#dietEntryDiv').hide();
            this.$('#nextDate').show();
            
             // saves weight if coming from weight page and weight is different than previously recorded that day
            
            if(this.type === "weightEntry"){
            	var existingWeight = false;
            	
            	var weights = [];
            	weights = UserData.get("weights");
            	
            	for(var i=0;i < weights.length; i++){

            		var recDate = new Date(weights[i].date);
            		
            		if(this.$('#lbsDisplay').html() == weights[i].weight && convertDateToString(this.currentDate, "veryVeryShort") == convertDateToString(recDate, "veryVeryShort")){
            		
            			existingWeight = true;
            		}
            	}
            	
            	if(existingWeight == false){
            		
            		this.childView.addWeight(this.currentDate);	
            	}
            	existingWeight = false;
            }
            
            this.$('#activityEntryDiv').replaceWith('<div id="activityEntryDiv" style="width:100%"></div>');

            //TODO: get current diet values from data model/DB and populate the accordian list view, and then attach to the list
            //TODO: will also need event handlers to handle entering a new item.  
            this.type = "ActivityEntry";    //once dietEntry is set up, change this.  set to weight to avoid crash
            this.childView = customViews[this.type]();
            
            this.$('#activityEntryDiv').append(this.childView.render().el);
			this.$('#activityEntryDiv').show();
			
			
			
			if(!UserData.get("activityCalories")){
				UserData.set("activityCalories", 0);
			}
			
			if(!UserData.get("foodCalories")){
				UserData.set("foodCalories", 0);
			}
			
			if(!UserData.get("mcaValue")){
				UserData.set("mcaValue", 0);
			}       
			
			if(!UserData.get("initialWeightLossRange")){
				UserData.set("initialWeightLossRange", 0.5);
			}
			
			
			if(!UserData.get("currentWeight")){
				UserData.set("currentWeight", 0.0);
			}
			
			UserData.save();
			
			var goalCal = UserData.calculateCalorieGoal(UserData.get("initialWeightLossRange"),UserData.get("currentWeight"));
			
			var foodCal = UserData.get("foodCalories");
			
			var paCal = UserData.get("activityCalories");
			
			var budgetCalories = Number(goalCal) + Number(UserData.get("mcaValue"));
			
			var remainingCalories = Number(budgetCalories) + Number(paCal) - Number(foodCal);
			
			remainingCalories = remainingCalories.toFixed(0);
			
			this.$("#budgetCalories").html(budgetCalories);
			this.$("#foodCalories").html(UserData.get("foodCalories").toFixed(0));
			this.$("#activityCalories").html(UserData.get("activityCalories"));
			this.$("#remainingCalories").html(remainingCalories);


            /*
             * Workflow
             *  
             * User selects activity tab: current activities for this date are displayed
             *      need to call the activity model and get the data activityModel.getDataForDate(this.currentDate) returns maybe a JSON object or array or something easy to work with
             *      Display the activity entries appropriately in the list/accordian view
             * 
             * User selects + to add a new activity
             *      Need to launch a view to allow the user to select a new activity and enter any other required info
             *      View will provide a 'done' or 'save' method of some sort that we can call when the user selects done here (or navigates off the page maybe)
             *      View will be responsible for negotiating with the activity model to store the data appropriately
             *      Need to refresh the current view with new data.  View 'save' method could return the new data OR (better) just call the childview.render method and append that appropriatly
             * 
             *      ex:  this.$('#' + divName).append(this.childView.render().el);  THis can append an <li> or a <div> or whatever, to which ever element on the container we need it to.
             */
            
            // refreshes page 
            $('body').enhanceWithin();
            
            UserData.set('entryType',"activityEntry");
            UserData.save();



        },
        
        selectDate: function(ev) {
            
            var self = this;
            
            var currentSelectedDateElement = $('#selectedDate');          
            var currentSelectedDate = currentSelectedDateElement.attr('data-date')
            
            var maxDate = new Date();
            if (!this.$('#weightButton').hasClass("ui-btn-active")) {
                maxDate.setFullYear(maxDate.getFullYear()+10);
            }
            
            var options = {
                date: new Date(currentSelectedDate),
                mode: 'datetime',     //or change to date
                maxDate: maxDate
            };

            function onSuccess(date) {
                
                console.log ("On Success changing date");
                
                var shortDateString = convertDateToString(date, "shortName");
                
                currentSelectedDateElement.text(shortDateString);
                currentSelectedDateElement.attr('data-date', date);
                
                if (self.$('#weightButton').hasClass("ui-btn-active")) {
                    
                    console.log ("Dealing with weights in dateSelect");
                    
                    if ("Today" === shortDateString) {
                        self.$('#nextDate').hide();
                        $.mobile.silentScroll(0);
                    } else {  
                        self.$('#nextDate').show();
                        self.scrollToDate(new Date(date));
                    }                                                         
                }  else {
                    console.log ("Why is weight button not active?");
                }         
            }
            
            function onError() {
                console.log ("There was an error captureing the date");
            }
            
            datePicker.show(options, onSuccess, onError);
                
        },
        
        updateDate: function(direction) {

            /*
             * Get the current date that the user has set the date selector to.  at teh end of this section, newDate will contain a date object representing the 'current selected date prior to any changes'
            */
            var newDate = new Date(this.$('#selectedDate').attr('data-date'));

            /*
             * compute the new date based on user action or updating time every minute.
             * 
             * output: newDate will contain a date object with the new changes applied.
             */
            if (direction === "next") {
                newDate.setDate(newDate.getDate() + 1);                
            } else if (direction === "previous") {                
                 newDate.setDate(newDate.getDate() - 1);                
            } else {
                //i really dont know wht this will do at midnight
                newDate.setMinutes (newDate.getMinutes() + 1);                
            }
            //reset the attr with the full date
            this.$('#selectedDate').attr('data-date', newDate);
            
            /*
             * Reset the title string to display appropriately either the veryVeryShort date or representative word
             */    
             var dateShortName = convertDateToString(newDate, "shortName")
             this.$('#selectedDate').text(dateShortName);
                          
              if (this.$('#weightButton').hasClass("ui-btn-active")) {
                                 
                if ("Today" === dateShortName){                                    
                    this.$('#nextDate').hide();
                    $.mobile.silentScroll(0);
                } else {  
                    this.$('#nextDate').show();
                    
                    this.scrollToDate(newDate);
                }
              }                                                               
            
            //Store the entry date to preserve the user experience
            UserData.set('entryDate',newDate);
            UserData.save();
            
            if(UserData.get('entryType') === "dietEntry"){
            	this.displayDietEntry();
            }else if(UserData.get('entryType') === "activityEntry"){
            	this.displayActivityEntry();
            }else{
                //No Need to redisplay weight page. messes up the input field
                //might need to do something here is usign Alex Keyboard
            	//this.displayWeightEntry();
            }
            
            //avoid using UserData for random info, just set a variable on the appropriate childView
            //have to reset to null for today to get valid values for current times
            // if (newDateString === today) {
                // this.childView.date = null;
            // } else {
                // this.childView.date = this.currentDate;
            // }
        },
        scrollToDate: function(date) {
            //scroll the weights list to the correct date   
            
            if ("Today" === convertDateToString (date, "shortName")) {
                $.mobile.silentScroll(0);
            } else {
                             
                var dateArray = convertDateToString (date, "weightLogFormat");                                
                var newDateString = dateArray.dayOfWeek + ", " + dateArray.month + " " + dateArray.day + ", " + dateArray.year;      
                
                var liDivider = this.$('#weightList li:contains(' + newDateString + ')');
                    
                if (liDivider.position()) {                        
                    //this animate does not work but I wish it did
                    //$("html, body").animate($.mobile.silentScroll(liDivider.position().top - 45), 50000);
                    $.mobile.silentScroll(liDivider.position().top - 45);
                }       
            }     
        },
        
		
		getIndexAndRemove: function (ev, arrayName) {

			selectedIndex = $(ev.currentTarget).index();
			
			
			if(arrayName === "removeFood"){
				var entriesArray = PortionData.get("DietEntrys");
				var titleString = "Delete " + entriesArray[$(ev.currentTarget).attr("data-masterArrayIndex")].foodname + "?";
			}else{
				var entriesArray = PAData.get("PAEntrys");
				var titleString = "Delete " + entriesArray[$(ev.currentTarget).attr("data-masterArrayIndex")].desc + "?";
			}
			
			
			var that=this;
			
			function onConfirm(buttonIndex) {
  				if (buttonIndex === 1) {
  					entriesArray.splice($(ev.currentTarget).attr("data-masterArrayIndex"),1);
  					if(arrayName === "removeFood"){
  						PortionData.set("DietEntrys",entriesArray);
						PortionData.save();

					}else{
		
	  					PAData.set("PAEntrys",entriesArray);
						PAData.save();

					}
  					
  			that.render();
  					// Backbone.history.loadUrl();	
  						

				}
			}
			
			
			navigator.notification.confirm(
    			' ', // message
     			onConfirm,            // callback to invoke with index of button pressed
    			titleString,           // title
    			['Yes','No']     // buttonLabels
			);
			
			
		}
		 
    });

    return EntryHomeContainerView;
});
