define(function(require) {
    var log = require("loglevel"),
        Handlebars = require("handlebars"),
        TriggersAndFixesData = require("models/TriggersAndFixesModel"),
        template = require("text!templates/G9C2ConsiderEnvironment.html");

    // Compile the template once.
    var compiled_template = Handlebars.compile(template);
    
    var isEditing = false;
    var selectedIndex;

    // The view for Guide 1 Quesionnaire
    var G9C2ConsiderEnvironmentView = Backbone.View.extend({
        // Set the id attribute on the div.
        id: "G9C2ConsiderEnvironmentView",

		model: TriggersAndFixesData,

		initialize: function() {},

        events: {

            'click #doneButton': 'fixesDone',
            
            'click ul#triggersListView li#deletableCells': function(ev){this.updateFixesForTrigger(ev);},
            'click ul#fixesListView li#deletableCells': function(ev){this.editFix(ev);},

            'change #triggers': function(ev){this.addTrigger();},                                    
            'change #fixes': function(ev){this.addFix();},
                        
			'swipeleft ul#triggersListView li#deletableCells': function(ev){this.deleteTrigger(ev);},
            'swiperight ul#triggersListView li#deletableCells': function(ev){this.deleteTrigger(ev);},
            
            'swipeleft ul#fixesListView li#deletableCells': function(ev){this.deleteFix(ev);},
            'swiperight ul#fixesListView li#deletableCells': function(ev){this.deleteFix(ev);},          
        },

        render: function() {
            $(this.el).html(compiled_template(TriggersAndFixesData.toJSON()));
            
            //reset this so that the most recently entered trigger will be the one that fixes are added to, unless the user opts to edit an existing fix
            TriggersAndFixesData.set("currentTriggerIndex", -1);
            
            var pulledTriggers = TriggersAndFixesData.get("triggers");
            var codeString = "";
            
            $.each(pulledTriggers, function(index, value) {
  
               codeString = codeString + '<li id="deletableCells" style="white-space:normal"><h3 style="white-space:normal">' + value.trigger + '</h3><p style="white-space:normal">Fixes:<br/>';
  
               $.each (value.fixes, function(ind, val) {
                   
                   codeString = codeString + val + '<br /><br />';
               });
               codeString = codeString + '</p></li>';
            });   
            
            this.$('#triggersListView').append(codeString);                   	
			this.$('#fixEntryPage').hide();

            return this;
        },

        addTrigger: function() {
            setTimeout(function(){
            //this timeout gives the system the time it needs to remove the value if the user tapped the X button in the input field.       
      
                var value = this.$('#triggers').val();
                
                if (value === "") return;
                    
                TriggersAndFixesData.addTrigger(value);

                this.$('#initialPage').hide();
                this.$('#fixEntryPage').show();
            
                this.$('#fixTriggerLabel').replaceWith('<label id="fixTriggerLabel" for="fixes" style="white-space: normal">Fixes for ' + value + '</label>');

            }, 100);

        },

        addFix: function() {

            setTimeout(function(){
            //this timeout gives the system the time it needs to remove the value if the user tapped the X button in the input field.       

                var value = this.$('#fixes').val();
              
                if (value === "") return;
                                            
                //check here to see if this was actaully an edit to an existing fix.  If so, just replace that fix         
                var editedFix;
                var index = -1;
                if (this.$("#fixesListView li").hasClass("isEditing")) {
                    editedFix = this.$("#fixesListView li.isEditing");
               
                    editedFix.css({"font-style":"normal", "text-indent":"0px"});
                    editedFix.removeClass("isEditing");
               
                    index = editedFix.attr("data-index");
                    editedFix.removeAttr("data-index");               
                    editedFix.text(value);
               
                } else {
                    $('<li id="deletableCells" style="white-space:normal">').append(value).appendTo(this.$('#fixesListView'));
                }
           
                //send in index to update here
                TriggersAndFixesData.addFix(value, index);
                      
                this.$('#fixes').val('');
                this.$('#fixesListView').listview("refresh");
            
            }, 100);
        },
        
        /*
         * This method loads the screen that allows a user to add additional fixes or edit/delete exisitng fixes for a particular Trigger
         */
        updateFixesForTrigger: function(ev) {
            
            var self=this;
            
            selectedIndex = $(ev.currentTarget).index();
            
            //this lets the model know that we are editing an existing trigger to add/remove fixes
            TriggersAndFixesData.set("currentTriggerIndex", selectedIndex-1);
            
            var currentTrigger = TriggersAndFixesData.get("triggers")[selectedIndex-1];
                        
            this.$('#initialPage').hide();
            this.$('#fixEntryPage').show();
            
            this.$('#fixTriggerLabel').replaceWith('<label id="fixTriggerLabel" for="fixes" style="white-space: normal">Fixes for ' + currentTrigger.trigger + '</label>');
            
            //populate existing fixes
            $.each(currentTrigger.fixes, function(key, value) {
               
                $('<li id="deletableCells" style="white-space:normal">').append(value).appendTo(self.$('#fixesListView'));                               
            });
            this.$('#fixesListView').listview("refresh");
 
        },

        deleteTrigger: function(ev) {
            
            var self = this;
            selectedIndex = $(ev.currentTarget).index() - 1;
            
            function onConfirm(buttonIndex) {
                if (buttonIndex === 1) {
                    
                    TriggersAndFixesData.deleteTrigger(selectedIndex);
                    self.render(); 
                    $("body").enhanceWithin();                
                }
            }            
            var titleString = "Are you sure you want to delete '" + (TriggersAndFixesData.get("triggers")[selectedIndex]).trigger + "'?";
            
            navigator.notification.confirm(
                ' ', // message
                onConfirm,            // callback to invoke with index of button pressed
                titleString,           // title
                ['Yes','No']     // buttonLabels
            );            
            
        },
        
        deleteFix: function(ev) {
            
            var self = this;
            selectedFixIndex = $(ev.currentTarget).index() - 1;
             
            function onConfirm(buttonIndex) {
                if (buttonIndex === 1) {
                    
                    TriggersAndFixesData.deleteFix(selectedFixIndex);                              
                    $(ev.currentTarget).remove();
                }
            }            
            var titleString = "Are you sure you want to delete the fix: '" + $(ev.currentTarget).text() + "'?"; 
            
            navigator.notification.confirm(
                ' ', // message
                onConfirm,            // callback to invoke with index of button pressed
                titleString,           // title
                ['Yes','No']     // buttonLabels
            );                                    
        },
  
        /*
         * This method is invoked when a user taps an existing fix in the fixes list view for a particular trigger and allows the user to edit the text for that fix
         */
        editFix: function (ev) {
         
           var alreadyEditingIndex = -1;         
           if (this.$("#fixesListView li").hasClass("isEditing")) {
                //remove the indicators from the current selection for editing so the new element can be set to edit mode
               var editedFix = this.$("#fixesListView li.isEditing");
               
               editedFix.css({"font-style":"normal", "text-indent":"0px"});
               editedFix.removeClass("isEditing");
                
               alreadyEditingIndex = editedFix.attr("data-index");                              
               
               editedFix.removeAttr("data-index");
               this.$('#fixes').val(''); 
           }
         
            selectedFixIndex = $(ev.currentTarget).index() - 1;
               
            //if the item the user tapped is already in edit mode, take it out of edit mode and return   
            if (alreadyEditingIndex == selectedFixIndex) return;       
                     
            var currentFix = $(ev.currentTarget).text();
             
            $(ev.currentTarget).css({"font-style":"italic", "text-indent":"10px"});
            $(ev.currentTarget).attr("data-index", selectedFixIndex);
            $(ev.currentTarget).addClass("isEditing");
            
            this.$('#fixes').focus(); 
            this.$('#fixes').val(currentFix);
          
        },   
  
       	fixesDone: function(){
  
       		this.$('ul#fixesListView li#deletableCells').remove();
       		this.$('#fixesListView').listview("refresh");
       		// show initialPage
       		this.$('#fixEntryPage').hide();
       		this.$('#initialPage').show();
			this.render();
			
			$("body").enhanceWithin(); 
			
            isEditing = false;
					    
       	}
    });

    return G9C2ConsiderEnvironmentView;
});


