define(function(require) {
    var log = require("loglevel"),
        Handlebars = require("handlebars"),
        Guides = require("collections/allGuides"),
        template = require("text!templates/guide-chapter.html"),
        ShakeGameView = require ("views/ShakeGameView"),
        ShakeGameModel = require("models/ShakeGameModel"),
        VideoView = require ("views/VideoView"),
        PhysicalGoalsView = require ("views/PhysicalGoalsView"),  // can we have just GoalsView?
        DietaryGoalsView = require ("views/DietaryGoalsView"),
        VideoModel = require("models/VideoModel"),
        PhotosView = require ("views/PhotosView"),
        PhotosModel = require("models/PhotosModel"),
        JoinedMOVEView = require ("views/JoinedMOVEView"),
        JoinedMOVEModel = require("models/JoinedMOVEModel"),
        SummaryChecklistView = require("views/SummaryChecklistView"),
        SummaryStatusView = require("views/SummaryStatusView"),
        SummaryProgressReviewView = require ("views/SummaryProgressReviewView"),
        UserInfoView = require ("views/UserInfoView"),
        WeightLossGoalView = require ("views/WeightLossGoalView"),
        WeightLossRateWithCalView = require("views/WeightLossRateWithCalView"),
        G2Ch1_calRange = require("views/G2Ch1_calRangeView"),
        PDFDisplay = require("views/pdfDisplayView"),
        G2C3PlateSlidersView = require("views/G2C3PlateSlidersView"),
        G4C4HungerSliderView = require("views/G4C4HungerSliderView"),
        FoodPortionsView = require("views/FoodPortionsView"),
        FoodPortionsModel = require("models/FoodPortionsModel"),
        G5C1ActivitySlidersView = require("views/G5C1ActivitySlidersView"),
        ProgressReviewView = require("views/ProgressReviewView"),
        G6C1FattyFoodsView = require("views/G6C1FattyFoodView"),
        EvalView = require("views/EvalView"),
        SuccessView = require("views/SuccessView"),
        G8C1MakePlanView = require("views/G8C1MakePlanView"),
        G8C2FoodSubsView = require("views/G8C2FoodSubsView"),
        G8C3AllActivitiesView = require("views/G8C3AllActivitiesView"),
        G9C2ConsiderEnvironmentView = require("views/G9C2ConsiderEnvironmentView"),
        G9C2UnhealthyFoodsView = require("views/G9C2UnhealthyFoodsView"),
        ResourceListView = require ("views/ResourceListView"),
        G10C2ChainOfEvents1View = require("views/G10C2ChainOfEvents1View"),
        G10C2ChainOfEvents2View = require("views/G10C2ChainOfEvents2View"),
        G10C2ChainOfEvents3View = require("views/G10C2ChainOfEvents3View"),
        UserData = require("models/UserData");


    // Compile the template once.
    var compiled_template = Handlebars.compile(template);

	var self;

	var customViews = {
		'video': function (pageRoot) { return new VideoView({model: new VideoModel(pageRoot)}); },
		'camera': function () { return new PhotosView({model:new PhotosModel()}); },
		'joinedMOVE': function () { return new JoinedMOVEView({model: UserData}); },
		'userInfo': function () { return new UserInfoView({model: UserData}); },
		'physicalGoals': function () { return new PhysicalGoalsView({model: UserData}); },
		'dietaryGoals': function () { return new DietaryGoalsView({model: UserData}); },
		'weightLossGoal': function () { return new WeightLossGoalView({model: UserData}); },
		'weightLossRate': function () { return new WeightLossRateWithCalView({model: UserData}); },
		'summaryChecklist': function () { return new SummaryChecklistView({model: UserData, guide: self.guideNumber}); },
		'summaryStatus': function () { return new SummaryStatusView({model: UserData}); },
		'summaryProgressReview': function() {return new SummaryProgressReviewView({guide:self.guideNumber}); },
		'shakeGame': function (pageRoot) { return new ShakeGameView({model:new ShakeGameModel(pageRoot.source)}); },
		'G2Ch1_calRange': function() { return new G2Ch1_calRangeView({model: UserData});},
		'PDFDisplay': function(pageRoot) {return new PDFDisplay({model: pageRoot});},
		'G2Ch1_calRange': function() { return new G2Ch1_calRangeView({model: UserData, guide: self.guideNumber});},
		'G2C3PlateSliders': function() {return new G2C3PlateSlidersView({model: UserData});},
		'G4C4HungerSlider': function() {return new G4C4HungerSliderView();},
		'G2C3PortionControl': function() {return new FoodPortionsView({model: new FoodPortionsModel()});},
		'G5C1ActivitySlider': function(){return new G5C1ActivitySlidersView();},
		'ProgressReview': function(){return new ProgressReviewView({guide: self.guideNumber});},
		'G6C1FattyFoods': function(){return new G6C1FattyFoodsView();},
		'evaluation': function(){return new EvalView();},
		'successStories': function(){return new SuccessView();},
		'G8C1MakePlan': function(){return new G8C1MakePlanView();},
		'G8C2FoodSubs': function(){return new G8C2FoodSubsView({model: new FoodPortionsModel()});},
		'G8C3Activities': function(){return new G8C3AllActivitiesView();},
		'G9C2ConsiderEnvironment': function(){return new G9C2ConsiderEnvironmentView();},
		'G9C2UnhealthyFoods': function(){return new G9C2UnhealthyFoodsView();},
		'pdfViewer': function(pageRoot){return new ResourceListView(pageRoot);},
		'G10C2EventChain1': function(){return new G10C2ChainOfEvents1View();},
		'G10C2EventChain2': function(){return new G10C2ChainOfEvents2View();},
		'G10C2EventChain3': function(){return new G10C2ChainOfEvents3View();},
	    'default': function () {
			log.debug ("handle other stuff");
			console.log( "unknown custom type" );
		}
	};


    var GuideChapterView = Backbone.View.extend({
        // Set the id attribute on the div.
        id: "guideChapterView",

        events: {
            "click #next-button": "handleNextButton",
            "click #previous-button": "handlePreviousButton"
        },

        initialize: function(args) {
        	self = this;
        		this.guideNumber = args.guide;

            	var chapters = Guides.get(args.guide).get("chapters");
            	var chapterNbr = Number(args.chapter);
				this.chapterNumber = chapterNbr;

				this.model = chapters[chapterNbr];

				var pages = this.model.pages;
				var pageNbr = Number (args.page);
				this.pageNumber = pageNbr;

            // Used to toggle display of previous/next buttons.
            this.model.isPrevious = false;
			this.model.isDone = true;

			if (pages.length === 1) {
                this.model.label = "Done";

                // Since there's only 1 chapter in this guide, set next to go back to the main guides page.
                this.model.nextHref = "#guide/" + args.guide;
                //this should transition in reverse

				this.model.isChapterComplete = "check";
				Guides.get(this.guideNumber).save();

            } else {

            	if (this.pageNumber + 1 === pages.length) {
                	this.model.label = "Done";
					
					this.model.isChapterComplete = "check";
                	Guides.get(this.guideNumber).save();
                	
                	// This is the last page in this chapter so set next to go back to the current guide page. SJB changed comment that said go to main guides page
                	this.model.nextHref = "#guide/" + args.guide;
            	} else {
            		this.model.label = "Next";
            		this.model.isDone = false;
                	// There are more chapters so set next to go to the next chapter.
                	this.model.nextHref = "#guide/" + args.guide + "/chapter/" + (this.chapterNumber) + "/page/" + (this.pageNumber + 1);
            	}

                // Don't display the previous button on the 1st chapter.
                if (this.pageNumber !== 0) {
                    this.model.isPrevious = true;
                    this.model.previousHref = "#guide/" + args.guide + "/chapter/" + this.chapterNumber + "/page/" + (this.pageNumber - 1);
                }
            }
            this.model.footerHeading = (pages.length === 1) ? "1 of 1" : (this.pageNumber + 1) + " of " + pages.length;

        },

        render: function() {
            var deferred = $.Deferred();
            var self = this;

			var pageRoot = this.model.pages[this.pageNumber];

			//set temp value for testing until custom is completed
			var contentTemplate = "404.html";

			if (pageRoot.custom) {
		
				//pull 'name' of custom page from JSON and use to generate a childView and contentTemplate.
				contentTemplate = this.setUpCustomTemplate(pageRoot);
				
				self.model.leftMargin = "0px";
				
			} else if (pageRoot.standard) {

				self.model.leftMargin = "10px";
				contentTemplate = pageRoot.standard;
			}

			if (!this.childView) {
				//This is used when we need to load a content template from the bundle (for html pages or videos), so we know we are getting the template from file
				require(["text!templates/" + contentTemplate], function(template) {

                	self.model.content = template;
                	deferred.resolve();
            	});

			} else {
				//this is used when the template is pulled from a childView
				self.model.content = contentTemplate;
				deferred.resolve();
			}

            $.when(deferred).done(function() {
                $(self.el).html(compiled_template(self.model));

                if (self.childView) {

					self.$('#content').append(self.childView.render().el);

				}


                // Tell JQM to enhance the page since we missed the intial event.
                $(self.el).trigger("create");

            });


            return this;
        },

		handlePreviousButton: function() {
			$.mobile.changePage.defaults.transition = "flip";
			$.mobile.changePage.defaults.reverse = true;
			mainRouter.navigate(this.model.previousHref, {
                trigger: true, replace:true
            });
		},

        handleNextButton: function(e) {
        	$.mobile.changePage.defaults.transition = "flip";
        	$.mobile.changePage.defaults.reverse = false;
          	mainRouter.navigate(this.model.nextHref, {
            	    trigger: true, replace:true
            });
        },

        setUpCustomTemplate: function(pageRoot) {
			var custTemplate;
        	this.childView = customViews[pageRoot.custom](pageRoot);
        	custTemplate = this.childView.template;

console.log ("Custom template in GCV: " + custTemplate);

        	return custTemplate;
        }
    });

    return GuideChapterView;
});
