/**
 * Represents a logical piece of an application's user interface, and provides a lightweight,
 * overridable API for rendering content and handling delegated DOM events on a container element.
 *
 * @module Views
 */
define(function(require) {
    var log = require("loglevel"),
        Handlebars = require("handlebars"),
        Guides = require("collections/allGuides"),
        template = require("text!templates/page.html");

    // Compile the template once.
    var compiled_template = Handlebars.compile(template);

	var guideNumber, title;
    /**
     * Class responsible for rendering a standard page view. This class can be sub-classed to
     * render different page templates or render partial templates that are to complex to be
     * used as inline templates. This view is defined in page_template.html
     *
     * @class PageView
     * @extends Backbone.View
     * @constructor
     */
    var PageView = Backbone.View.extend({
        id: "pageView",

        initialize: function(args) {
            this.model = Guides.get(args.guide);
            guideNumber = args.guide;
            title = args.title;
            
            
            console.log ("title in pageView: " + title);
            
            if (title === "Congratulations") {
            	this.congratulations = true;
            	Guides.get(guideNumber).save("congratulationsComplete", "check");
            } else if (title === "Homework") {
            	Guides.get(guideNumber).save("homeworkComplete", "check");
            } else if (title === "Objectives") {
            	Guides.get(guideNumber).save("objectivesComplete", "check");
            } else if (title === "Get Ready") {
            	Guides.get(guideNumber).save("getReadyComplete", "check");
            }
            
        },

        /**
         * Method to render this view.
         * <p>
         * <b>Note: </b>This method can be overridden by subclasses of PageView
         * to provide page specific functionality.
         * </p>
         *
         * @method render
         * @chainable
         * @return {PageView} An instance of this object.
         * @since 1.0
         */
        render: function() {
            var deferred = $.Deferred();

console.log ("Using page view");

            var self = this;
            require(["text!../../content/guides/guide" + guideNumber + "/" + this.model.get(this.id)], function(template) {
            	            	
                self.template = template;
                deferred.resolve();
            });

            $.when(deferred).done(function() {
            	
            	var contentJSON = {"content":self.template, "title": title};
            	if (self.congratulations) {            		            		
            		contentJSON.congratulations = true;            		
            	} 
            	           	
            	var html = compiled_template(contentJSON);
                                  
                $(self.el).html(html);
                $(self.el).trigger("create");
                return this;
            });
        }
    });

    return PageView;
});
