define(function(require) {
    var log = require("loglevel"),
        Handlebars = require("handlebars"),
        template = require("text!templates/video.html");

    var compiled_template = Handlebars.compile(template);
    
	var VideoView = Backbone.View.extend({
        id: "videoView",

        templateName: "video.html",

         events: {
             "click #transcriptButton": "showTranscript",
             "click #overlayElement": "clickedOverlayElement"
         },

		initialize: function() {
			this.errorCondition = false;
		},
		
		showOverlay: function() {
				
			self.$('#overlay').css("visibility", "visible");
			var styles = {
    			position : "absolute",
    			top: "-2000px"
    		};
			self.$('#videoContainer').css(styles);
			this.overlayShown = true;
		},
		
		hideOverlay: function() {
			self.$('#overlay').css("visibility", "hidden");
			
			var styles = {
				position : "relative",
    			top: "0px"
			};
			self.$('#videoContainer').css(styles);	
			this.overlayShown = false;
		},
		
		render: function() {
        	
        	this.errorCondition = false;
            this.$el.html(compiled_template(this.model));
            
            var onCanPlay = function () {
            	ActivityIndicator.hide();
            	
            	console.log ("width: " + $("#overlay").width());
            	
            	
            };
            var onError = function () {
            
            	ActivityIndicator.hide();
            	self.errorCondition = true;       
            };
			
            var vid = this.$("#videoId");
            vid[0].addEventListener("canplay", onCanPlay, false);
            vid[0].addEventListener("error", onError, true);

			self = this;                        
            this.overlayShown = false;                    
			this.showOverlay();                    

            return this;
		},
		
		clickedOverlayElement: function(event) {
			
			if (self.errorCondition) {				
				function onConfirm(buttonIndex) {
  					self.render();
				};
				navigator.notification.confirm(
    				'The Video could not be loaded.  Please confirm your device is connected to the internet.', // message
     				onConfirm,            // callback to invoke with index of button pressed
    				'Connection Error',           // title
    				['OK']     // buttonLabels
				);

			} else {
						
				ActivityIndicator.show("Loading Video");
			
				window.setTimeout(function() {
					if (self.overlayShown) {
						self.hideOverlay();
				 		var videoElement = document.getElementById("videoId");
						videoElement.play();
						videoElement.webkitEnterFullscreen();
					} 
				}, 750);
			}
		},
		
		showTranscript: function() {
			
			mainRouter.navigate("#transcript/" + this.model.transcriptLink, {
                trigger: true, replace:false
            });
		},
		afterRender: function() {
		}
	});
	return VideoView;
});