define(function(require) {
    var log = require("loglevel"),
        Handlebars = require("handlebars"),
        WeightModel = require("models/WeightModel"),
       
       //required with current implementation get rid of this if we can
       UserData = require("models/UserData"),
       
       WeightsCollection = require('collections/weightsCollection'),
       
       template = require("text!templates/WeightEntryList.html");

    // Compile the template once.
    var compiled_template = Handlebars.compile(template);
    
    var WeightEntryListView = Backbone.View.extend({
        // Set the id attribute on the div.
        id: "WeightEntryListView",

        initialize: function() {
                        
            //temporary so the container works
              if (!UserData.get("weights")) {
                UserData.set ("weights", []);
                UserData.save();
              }
            //
            
            this.collection = new WeightsCollection();            
                        
            var self = this;
            Handlebars.registerHelper('getCurrentWeightString', function() {
                
                var returnString = '';
                
                if (self.collection.length == 0) {
                    returnString = "Please enter your weight.  The date can be changed above."    
                } else {                    
                    var currentWeight = self.collection.getCurrentWeight();                
                    returnString = "Most recent weight: " + currentWeight.get("weight") + " lbs on " + convertDateToString(new Date(currentWeight.get("date")), "veryVeryShort");
                }                
                return returnString;
            });            
        },

        events: {

            'change #weightEntry': function(ev){this.addWeight(ev);},
            'click ul#weightList li#deletableCells': function(ev){this.editWeight(ev);},            

            'swipeleft ul#weightList li#deletableCells': function(ev){this.deleteWeight(ev);},
            'swiperight ul#weightList li#deletableCells': function(ev){this.deleteWeight(ev);}            
        },

        render: function() {

            $(this.el).html(compiled_template());

            var self = this;
            
            this.$('#editWeightText').hide();
            
            var currentDate = '';
            this.collection.each(function (weight, index, all) {
                
                //Format the weight cell properly to display all necessary data 
                var dateArray = convertDateToString (new Date(weight.get("date")), "weightLogFormat");
                                
                newDateString = dateArray.dayOfWeek + ", " + dateArray.month + " " + dateArray.day + ", " + dateArray.year;      
                                            
                if (currentDate != newDateString) {
                    currentDate = newDateString;
                                        
                    liDivider = $('<li data-role="list-divider">').append(currentDate).appendTo(self.$('#weightList'));                                  
                }
 
                var liString = '<p class="weightString">' + weight.get("weight") + ' <span class="fontLarge">lbs</span></p><p class="ui-li-aside"><strong>' + dateArray.time +'</strong> ' + dateArray.ampm + '</p>';                       
                var li = $('<li id="deletableCells" class="weightCell">');  

                //inject data into DOM element for easy retrieval
               $(li).attr("data-weight", weight.get("weight"));
               $(li).attr("data-date", new Date(weight.get("date")));
               $(li).attr("data-id", weight.get("id"));
                
                li.append(liString).appendTo(self.$('#weightList'));
            });
            
               $('body').enhanceWithin();
            

            return this;
        },

        addWeight: function() {
                      
            var weightDate = new Date($('#selectedDate').attr('data-date'));
            var self = this;
            
            setTimeout(function(){
            //this timeout gives the system the time it needs to remove the value if the user tapped the X button in the input field.       
      
                var value = this.$('#weightEntry').val();
                                
                if (value === "") return;                

                var editedWeightIndex = -1;
                //if a weight item has been edited, pass in the index of the edited weight and reset the object    
                if (this.$("#weightList li").hasClass("isEditing")) {                   
                                                          
                    var editedWeight = this.$("#weightList li.isEditing");
               
                    editedWeight.css({"font-style":"normal", "text-indent":"0px"});
                    editedWeight.removeClass("isEditing");
                             
                    editedWeightIndex = editedWeight.attr("data-index");          
                    editedWeight.removeAttr("data-index");
                    editedWeight.text(value);

                    //remove from data model
                    self.collection.deleteWeight(editedWeight.attr("data-id"));
                    
                    //if the old data and the new date are the same, use the old date to preserve the time                        
                    var oldDate = new Date(editedWeight.attr("data-date"));        
                    if (convertDateToString(new Date(oldDate), "veryVeryShort") === convertDateToString(weightDate, "veryVeryShort")) {
                        weightDate = oldDate;  
                    }    
                    
                    self.$('#editWeightText').hide();          
                    self.$('#editWeightText').text("");                    
                 }     
                                
                //add to data model
                self.collection.create ({weight:value, date:weightDate});
                
                self.render();
                    
                self.$('#weightEntry').val('');                
                self.$('#weightList').listview("refresh");

                self.parent.scrollToDate(weightDate);      

            }, 100);

        }, 
        
        editWeight: function(ev) {
          
           var alreadyEditingIndex = -1;          
           if (this.$("#weightList li").hasClass("isEditing")) {
                //remove the indicators from the current selection for editing so the new element can be set to edit mode
               var editedWeight = this.$("#weightList li.isEditing");
               
               editedWeight.css({"font-style":"normal", "text-indent":"0px"});
               editedWeight.removeClass("isEditing");
               
               alreadyEditingIndex = editedWeight.attr("data-index");
                             
               self.$('#editWeightText').hide();          
               self.$('#editWeightText').text("");                               
                             
               editedWeight.removeAttr("data-index");
               this.$('#weightEntry').val(''); 
           }
         
            selectedWeightIndex = $(ev.currentTarget).index() - 1;
                              
            if (alreadyEditingIndex == selectedWeightIndex) return;  //tapped the weight that was already in edit mode   
                        
            var currentTarget = $(ev.currentTarget);
            
            var currentWeight = currentTarget.attr("data-weight");
            var currentDate   = currentTarget.attr("data-date"); 
          
            $('#selectedDate').attr('data-date', currentDate);
            var updatedDateLabel = convertDateToString(new Date(currentDate), "shortName");
            $('#selectedDate').text(updatedDateLabel);               

            if (updatedDateLabel == "Today") {
                $('#nextDate').hide();
            } else {
                $('#nextDate').show();
            }
                        
            //add editing string  
            var editingString = "Currently editing " + currentWeight + " lbs on " + convertDateToString(new Date(currentDate), "veryVeryShort");
            this.$('#editWeightText').show();          
            this.$('#editWeightText').text(editingString);
                                  
            currentTarget.css({"font-style":"italic", "text-indent":"10px"});
            currentTarget.attr("data-index", selectedWeightIndex);
            currentTarget.addClass("isEditing");
            
            this.$('#weightEntry').focus(); 
            this.$('#weightEntry').val(currentWeight);                        
            
        },
        
        deleteWeight: function(ev) {               
                     
            var self = this;
            selectedWeightIndex = $(ev.currentTarget).index() - 1;    //DO WE NEED THIS ANYMORE?
                             
            var weight = $(ev.currentTarget).attr("data-weight");
            var date = $(ev.currentTarget).attr("data-date"); 
            var id = $(ev.currentTarget).attr("data-id");
           
            function onConfirm(buttonIndex) {
                if (buttonIndex === 1) {
        
                    self.collection.deleteWeight(id);                              
                   //rerender to recreate the table list dividers properly
                    self.render();
                }
            }            
            var titleString = "Are you sure you want to delete '" + weight + " lbs' on " + convertDateToString(new Date(date),"veryVeryShort") + "?"; 
            
            if ( typeof navigator.notification == 'undefined') {
                if (confirm(titleString)) onConfirm(1);
            } else {
                navigator.notification.confirm(
                    ' ', // message
                     onConfirm,            // callback to invoke with index of button pressed
                    titleString,           // title
                    ['Yes','No']     // buttonLabels
                );
            }                                       
        }
    });

    return WeightEntryListView;
});


