define(function (require) {
    var log = require("loglevel"),
        mockjax = require("mockjax");
    // var SingleGuide = require("collections/singleGuideModel");

    var GuidesCollection = Backbone.Collection.extend({
        localStorage: new Backbone.LocalStorage("Guides"),
        // url: 'http://localhost:9000/content/guides',

        // model: SingleGuide,
        model: Backbone.Model,

        // Used to allow others to wait for the collection to be populated.
        deferred: "",

        /**
         * Method to initialize this collection.
         *
         * @method initialize
         */
        initialize: function () {
            console.log("guides init");
            // Get guide information from local storage.
            // 
            // NOTE: 'this.fetch' will call localStorage.fetch which returns a deferred. We assigned
            // that deferred to our object deferred attribute so that anyone (main.js) waiting on this object can
            // be notified. This is the same logic in UserData with the exception that passing the 'reset' option
            // is used incorrectly. The reset option is for bulk collection updates. The localStorage fetch does
            // not trigger this event.
            this.deferred = this.fetch();

            if (this.length === 0) {
                // Reset our deferred since localStorage would have set it to resolved.
                this.deferred = new $.Deferred();

                // TODO: Figure out how to update content from server, or check for updated content on disk.
                // load all necessary JSON data into guides.
                var self = this;
                $.getJSON("content/guides.json")
                    .done(function (json) {
                        $.each(json.guides, function (index, value) {
                            // Create the model instance and add to the collection.
                            // 
                            // NOTE: Using Backbone convenience method 'create' which will create a new instance
                            // of the model, save the model (to local storage in this case), and add the model
                            // to the collection. See http://www.backbonejs.org/#Collection-create.
                            self.create(new Backbone.Model(value));
                        });

                        // We're done, so resolve our deferred.
                        self.deferred.resolve();
                    })
                    .fail(function (jqxhr, textStatus, error) {
                        var err = textStatus + ", " + error;
                        console.log("Request Failed: " + err);
                    });
            }
        },

        /**
         * Method (callback) in response to a 'fetch' or 'save' request. Parse is called
         * whenever a model's data is returned by the server in fetch and save method calls.
         *
         * @method parse
         * @param response JSON response from a server fetch or save request.
         */
        parse: function (response) {
            log.debug(JSON.stringify(response));
            return response;

            // log.debug(JSON.stringify(response));
            // return response.guides;
        },
        
        clear: function() {
            // reset guide progress
            // destroy guide objects so they can get reloaded with new data
        	console.log("allGuides.clear()");
            for (i = this.models.length -1; i >= 0; i--) { 
                this.models[i].destroy();
            }

            // reload guides
        	this.initialize();
        }

    });

    //     //TODO: First look on disk to see if guides object exists.  if so, load from there.
    //     //load all necessary JSON data into guides.
    //     $.getJSON("content/guides.json")
    //         .done(function(json) {
    //             $.each(json.guides, function(index, value) {
    //                 //create the model instance and add to the collection
    //                 app.guideGroup.add(new app.singleGuide(value));
    //             });
    //         })
    //         .fail(function(jqxhr, textStatus, error) {
    //             var err = textStatus + ", " + error;
    //             console.log("Request Failed: " + err);
    //         });
    // };

    // TESTING ONLY!
    // Does the same thing as the code above. Comment out this code to actually retrieve the data from a webservice.
    // $.mockjax({
    //     logging: false,
    //     url: 'http://localhost:9000/content/guides',
    //     contentType: 'text/json',
    //     proxy: 'content/guides.json'
    // });

    // This is a Singleton since we fetch the data in the initialize method.
    guidesCollection = new GuidesCollection();
    return guidesCollection;
    // return new GuidesCollection();
});