define(function (require) {
    var log = require("loglevel");
   

    var WeightsCollection = Backbone.Collection.extend({
        localStorage: new Backbone.LocalStorage("Weights"),

        model:Backbone.Model,

        // Used to allow others to wait for the collection to be populated.
        deferred: "",

        /*
         * insert Weight Objects in order based on date with most recent at the front of the collection
         */
        comparator: function(model) {
          
            return -(new Date(model.get('date')).getTime());
        },
        

        /**
         * Method to initialize this collection.
         *
         * @method initialize
         */
        initialize: function () {

            // Get guide information from local storage.
            // 
            // NOTE: 'this.fetch' will call localStorage.fetch which returns a deferred. We assigned
            // that deferred to our object deferred attribute so that anyone (main.js) waiting on this object can
            // be notified. This is the same logic in UserData with the exception that passing the 'reset' option
            // is used incorrectly. The reset option is for bulk collection updates. The localStorage fetch does
            // not trigger this event.  THIS DEFERRED IS NOT CURRENTLY USED
            this.deferred = this.fetch();

            if (this.length === 0) {
                console.log("no weights in Collection");             
               
            }
            else {
                console.log("this weights collection: ", this);
                
                            
                console.log ("Test getCurrentWeight: " + this.getCurrentWeight());
                console.log ("Test getInitialWeight: " + this.getInitialWeight());
                
            }
        },

        /**
         * Method (callback) in response to a 'fetch' or 'save' request. Parse is called
         * whenever a model's data is returned by the server in fetch and save method calls.
         *
         * @method parse
         * @param response JSON response from a server fetch or save request.
         */
        parse: function (response) {
                        
            //log.debug(JSON.stringify(response));
            return response;

        },
        
        clear: function() {
            // reset weights progress
            // destroy weight objects so they can get reloaded with new data
        	console.log("weightsCollection.clear()");
            for (i = this.models.length -1; i >= 0; i--) { 
                this.models[i].destroy();
            }

            // reload weights - (from server eventually?)
        	this.initialize();
        },

        getCurrentWeight: function() {

            return this.at(0);
        },

        getInitialWeight: function() {

            return this.at(this.length-1).get("weight");

        },
        
      //TODO: stop using weight and date as ID use ID      
        deleteWeight: function(id) {
                
           // console.log ("deleteing with : " + weight + "     " + date);
                     
            // var weightToDelete = this.findWhere({weight:weight, date:date.toLocaleString()});
         
            var weightToDelete = this.get (id);
         
            if (weightToDelete) {
                weightToDelete.destroy();
                console.log ("Destroying the found weight");
            } else {
                console.log ("COuld not find the object to delete.  Weight: " + weight + " -- Date: " + date);
            }          
        }
        
    });

    return WeightsCollection;
});