/* *************
 * This class deals with interfacing with the calendar plugin.
 * find is called on focusin, create is called on focusout
 * 
 * Usability:
 * var cal = new CalendarModel(calEventTitle, calEventNotes);
 * cal.findEvent(eventDateString);
 * cal.createEvent(newEventDateString);
 */

define(function(require) {
    var log = require("loglevel");

	var self = null;

	// takes into account local timezone offset
	Date.prototype.toJSONLocal = function() {
    	var local = new Date(this);
    	local.setHours( this.getHours()+(this.getTimezoneOffset()/-60) );
    	return local.toJSON();
	};

    var CalendarEventModel = Backbone.Model.extend({
    	
    	initialize: function(title, notes) {
			console.log("intialize-1");
    		
			this.title = title;
			this.notes = notes;
			
    		this.calEventExists = false;
    		this.origStartDate = null;
    		this.origEndDate = null;
    	},
    	
    	// this takes a string from a datetime-local input type
    	// and keeps track of the state of the original event
    	findEvent: function(eventDateString) {
			console.log("findEvent-1");

			var textValue = eventDateString;
    		this.calEventExists = false;
		
			if ( this.noCalendarPlugin(false) ) return;

		    
			console.log("findEvent-2: " + textValue);
		    var dttm = textValue.split("T");
			console.log("findEvent-2.5: " + dttm);
		    if ( dttm.length != 2 )
		    {
		    	console.log("invalid datetime format for finding existing event");
		    	return;
		    }
		    
			console.log("findEvent-3");

		    var dt = dttm[0];
		    var tm = dttm[1];
			console.log("findEvent-4");

		    var dtvals = dt.split("-");
		    var yr = dtvals[0];
		    var mo = dtvals[1] -1;  // month starts at 0
		    var dy = dtvals[2];
			console.log("findEvent-5");

		    var tmvals = tm.split(":");
		    var hr = tmvals[0];
		    var mn = tmvals[1];
			console.log("findEvent-6");

			var startDate = new Date(yr,mo,dy,hr, mn    ,0,0,0); // beware: month 0 = january, 11 = december
			var endDate   = new Date(yr,mo,dy,hr,+mn +15,0,0,0); // minutes roll over - tested on ios
			console.log("findEvent-7");
			
			this.findEventWithDates(startDate, endDate);
			console.log("findCalendarEvent finished:  exists: " + this.calEventExists);
		},
		
    	findEventWithDates: function(startDate, endDate, aDeferred) {
			console.log("findEventWithDates-1");

			this.origStartDate = startDate;
			this.origEndDate = endDate;

			if ( this.noCalendarPlugin(false) ) return;
			
  			self = this; // for callback functions
			var findSuccess = function(message) 
			{ 
				console.log("----- in findEventWithDates callback success - message: " + message);
				console.log("aDeferred:" + aDeferred);
				
				if ( message.length > 0 )
				{
					console.log("----- " + message.length + " events found");
					self.calEventExists = true;
					console.log("----- message: " + message);
					console.log("----- message 0: " + message[0]);
					console.log("----- message: " + JSON.stringify(message));
					console.log("----- message 0: " + JSON.stringify(message[0]));
					
					// if ( 'undefined' == typeof message.constructor )
					// {
						// console.log("constructor undefined");
					// }
					// else if ( 'undefined' == typeof message.constructor.name )
					// {
						// console.log("constructor name undefined");
					// }
					// else 
					// {
						// console.log("name: " + message.constructor.name);
						// console.log("name[0]: " + message[0].constructor.name);
					// }
				} 
				else console.log("----- no event found");
				
				if ( 'undefined' == typeof aDeferred )
				{
					console.log("aDeferred undefined");
				}
				else
				{
					console.log("resolving deferred");
					aDeferred.resolve();
				}
				
			};
			var findError = function(message) { console.log("** FAIL ** trying to find event: " + message); };
		
			console.log("finding event: " + this.origStartDate + ", " + this.origEndDate);
		  	// find events - ignore location and notes
		  	window.plugins.calendar.findEvent(this.title,null,null,this.origStartDate,this.origEndDate,findSuccess,findError);
			console.log("findCalendarEventWithDates finished:  exists: " + this.calEventExists);
    	},
    	
    	// this deletes existing event then creates a new one
    	// reminders are at time of event and halfway between event and now
    	createEvent: function(newEventDateString) {
			console.log("*** createEvent: " + this.title);
			var textValue = newEventDateString;
			console.log("createEvent for date: " + textValue);
			
            console.log("*** createEvent - event exists: " + this.calEventExists);

			if ( this.noCalendarPlugin(true) ) return;


            var dttm = textValue.split("T");
            // if blank (or invalid format [ie, no time]), check for existing event and delete
            if ( dttm.length != 2 )
            {
            	console.log("invalid datetime format for new event");
	            if ( this.calEventExists )
				{
		  			self = this; // save context for callback function
					var delSuccess = function(message) 
					{ 
						console.log("+++++ in delete existing callback success, no new event");
						self.calEventExists = false;
						console.log("+++++ SUCCESS: Calendar event deleted: " + JSON.stringify(message)); 
					};
					var delError = function(message) { console.log("** FAIL ** delete Calendar event failed: " + message); };
					
					
					console.log("event exists, attempting to delete old event without creating new");
					window.plugins.calendar.deleteEvent(this.title, null, null,this.origStartDate,this.origEndDate,delSuccess,delError);
					console.log(this.title + " event deleted -1");
				} else console.log("no existing event to delete -1");
            	return;
            }
            
            var dt = dttm[0];
            var tm = dttm[1];
            
            var dtvals = dt.split("-");
            var yr = dtvals[0];
            var mo = dtvals[1] -1;  // months start at 0
            var dy = dtvals[2];
            
            var tmvals = tm.split(":");
            var hr = tmvals[0];
            var mn = tmvals[1];
            
			var startDate = new Date(yr,mo,dy,hr, mn    ,0,0,0); // beware: month 0 = january, 11 = december
  			var endDate   = new Date(yr,mo,dy,hr,+mn +15,0,0,0); // minutes roll over - tested on ios
  			
  			console.log("calling createEventWithDates");
			this.createEventWithDates( startDate, endDate );
			console.log("createEvent finished");
	    },
    
      	// this deletes existing event then creates a new one
    	// reminders are at time of event and halfway between event and now
    	createEventWithDates: function(startDate, endDate) {
			console.log("*** createEventWithDates: " + this.title);
			console.log("" + startDate + ", " + endDate);
			console.log("*** createEventWithDates - event exists: " + this.calEventExists);

			// don't delete if same date/time as orig (no change to checkin date)
  			if ( this.calEventExists && (startDate - this.origStartDate == 0) )
  			{
  				// same checkin date, and event already exists, so don't do anything
  				console.log("+++ same checkin date, event exists, doing nothing");
  				return;
  			}
  			
  			self = this; // save context for callback function
  			var createSuccess = function(message) 
  			{ 
				console.log("------- in create with dates callback success");
  				console.log("*** SUCCESS: Calendar event created: " + JSON.stringify(message)); 
  			};
  			var createError = function(message) { console.log("** FAIL ** Calendar event failed: " + message); };
  			var deleteSuccess = function(message) 
  			{ 
				console.log("------- in delete with dates callback success");
  				if ( self.calEventExists ) // then this was a delete call
  				{
  					self.calEventExists = false;
  					console.log("///////// setting event exists to false");
  				}
  				console.log("*** SUCCESS: Calendar event deleted: " + JSON.stringify(message)); 
  			};
  			var deleteError = function(message) { console.log("** FAIL ** Calendar event failed: " + message); };

			// rather than modifying event, delete and add new so reminders get set correctly.			
			if ( this.calEventExists )
			{
				console.log("event exists, attempting to delete event before creating new");
				window.plugins.calendar.deleteEvent(this.title, null, null,this.origStartDate,this.origEndDate,deleteSuccess,deleteError);
				console.log(this.title + " event deleted -2");
			} else console.log("calEventExists: " + this.calEventExists);
			
			console.log("creating event: " + startDate + ", " + endDate);
  			var calOptions = window.plugins.calendar.getCalendarOptions(); // grab the defaults
  			calOptions.firstReminderMinutes = 0;  // at time of event
  			// second reminder is halfway.  find the number of days in between and convert to minutes.
  			// don't find minutes between, because the reminder should be same time of day as event.
  			var today = new Date();

  			var numDays = Math.ceil( (startDate - today) / 86400000 );
  			console.log("days: " + numDays );
  			
  			// if there's at least 2 days before, set a second alarm halfway
  			if ( numDays >= 2 )
  			{
  				console.log("adding second reminder: " + Math.floor(numDays / 2));
				calOptions.secondReminderMinutes = Math.floor(numDays / 2) * 1440;
  			}
			else
			{
				console.log("not creating second reminder");
				calOptions.secondReminderMinutes = null;
			}

  			window.plugins.calendar.createEventWithOptions(this.title,null,this.notes,startDate,endDate,calOptions,createSuccess,createError);
			console.log(this.title + " event created");			
		},
		
		moveEvent: function(oldEventDate, newEventDate) {
			console.log(this.title + " move event");			

			if ( this.noCalendarPlugin(true) ) return;

			var deferredCalEvent = $.Deferred();
			var self = this;
			$.when(deferredCalEvent).done(function() {
				console.log("*-*-*-* find event done, calling createEventWithDates");
				var endDate = new Date(newEventDate);
				endDate.setMinutes( endDate.getMinutes() + 15);
				console.log("start: " + newEventDate + ", end:" + endDate);
				self.createEventWithDates(newEventDate, endDate);
				console.log("create called");
			});

			var endDate = new Date(oldEventDate);
			endDate.setMinutes( endDate.getMinutes() + 15);
			this.findEventWithDates(oldEventDate, endDate, deferredCalEvent);  // TODO find old date
			console.log("moveEvent find called with deferred");
			console.log(this.title + " move event finished");			
		},
		
		// returns true if no calendar plugin found
		noCalendarPlugin: function(showAlert)
		{
		    if ( 'undefined' == typeof window.plugins || 'undefined' == typeof window.plugins.calendar )
		    {
		    	console.log( "No calendar found" );
		    	if (showAlert) alert( "No calendar found." );
		    	return true;
		    }
			return false; // calendar plugin found
		},
		
		// if existing date is in future, return it
		// if existing date is in past, or blank, suggest date # days ahead of today
		suggestFutureDate: function(existingDateString)
		{
			var checkinDate = new Date(existingDateString); // could be Invalid Date
			var today = new Date();

			// if existing checkin date is empty or in the past, suggest 2 weeks from today
			if ( checkinDate > today ) // invalid date always false for all equality checks
			{
				console.log("doing nothing, checkin date is after today");
				return existingDateString;
			}
			else
			{
				console.log("suggesting checkin date in 2 weeks");
				checkinDate = new Date();
				checkinDate.setDate(checkinDate.getDate() + 14); // refactor for diff timeframe if wanted
				return checkinDate.toJSONLocal().slice(0,19);
			}	
		}		
		
	});
	return CalendarEventModel;
});