define(function(require) {
    var log = require("loglevel"),
	    Handlebars = require("handlebars");

	var app = new Object();
	var dbUSDA_Foods; //session USDA_Foods db
	var dbUSDA_Portions; //session USDA_Portions db
	var dbUSDA_PA; //session USDA_PA db

    var database = {
		
		// start here
		openAppDatabases: function() {
			this.openUSDAFoods();
			this.openUSDA_Portions ();
			this.openUSDA_PA();
		},
		
		// Check if this browser supports Web SQL
		getOpenDatabase: function() {
			//alert("getOpenDatabase");
		    try {
		        if( !! window.openDatabase ) return window.openDatabase;
		        else return undefined;
		    } catch(e) {
		        return undefined;
		    }
		},
		
		openUSDAFoods: function() {
			
				//USDA_Foods
		    var createUSDA_FoodsSQL = 'CREATE TABLE IF NOT EXISTS USDA_Foods (' +
			  'Action text,' +
			  'Food_ID text,' +
			  'foodcode text,' +
			  'modcode text,' +
			  'foodname text,' +
			  'Protein text,' +
			  'Total_Fat text,' +
			  'Carbohydrate text,' +
			  'Energy text,' +
			  'Alcohol text,' +
			  'Water text,' +
			  'Caffeine text,' +
			  'Theobromine text,' +
			  'Sugars text,' +
			  'Fiber text,' +
			  'Calcium text,' +
			  'Iron text,' +
			  'Magnesium text,' +
			  'Phosphorus text,' +
			  'Potassium text,' +
			  'Sodium text,' +
			  'Zinc text,' +
			  'Copper text,' +
			  'Selenium text,' +
			  'Retinol text,' +
			  'Vitamin_A text,' +
			  'Carotene_beta text,' +
			  'Carotene_alpha text,' +
			  'Vitamin_E text,' +
			  'Vitamin_D text,' +
			  'Cryptoxanthin_beta text,' +
			  'Lycopene text,' +
			  'Lutin_zeaxanthin text,' +
			  'Vitamin_C text,' +
			  'Thiamin text,' +
			  'Riboflavin text,' +
			  'Niacin text,' +
			  'Vitamin_B6 text,' +
			  'Folate_total text,' +
			  'Vitamin_B12 text,' +
			  'Choline_total text,' +
			  'Vitamin_K text,' +
			  'Folic_acid text,' +
			  'Folate_food text,' +
			  'Folate_DFE text,' +
			  'Vitamin_E_added text,' +
			  'Vitamin_B12_added text,' +
			  'Cholesterol text,' +
			  'Fatty_acids_total_saturated text,' +
			  '_4_0 text,' +
			  '_6_0 text,' +
			  '_8_0 text,' +
			  '_10_0 text,' +
			  '_12_0 text,' +
			  '_14_0 text,' +
			  '_16_0 text,' +
			  '_18_0 text,' +
			  '_18_1 text,' +
			  '_18_2 text,' +
			  '_18_3 text,' +
			  '_20_4 text,' +
			  '_22_6_n3 text,' +
			  '_16_1 text,' +
			  '_18_4 text,' +
			  '_20_1 text,' +
			  '_20_5_n3 text,' +
			  '_22_1 text,' +
			  '_22_5_n3 text,' +
			  'Fatty_acids_total_mono text,' + 
			  'Fatty_acids_total_poly text' + 
			')';
		    
			dbUSDA_Foods = this.prepareDatabase(createUSDA_FoodsSQL, "db/USDA_Foods.sqlite3",  "USDA_Foods", "1.0", 5);
		},

		openUSDA_Portions: function() {		
			//USDA_Portions
			var createUSDA_PortionsSQL = 'CREATE TABLE IF NOT EXISTS USDA_Portions (' +
		  		'Action text,' +
		  		'Food_Item_ID text,' +
		  		'foodcode text,' +
		  		'modcode text,' +
		  		'foodname text,' +
		  		'portioncode text,' +
		  		'portioncode_addendum text,' +
		  		'portion_code text,' +
		  		'portionwgt text,' +
		  		'Portion_Factor text,' +
		  		'portiondesc text,' +
				'increment text,' +
		  		'default_portion text,' +
		  		'Use_as_default text,' +
		  		'Active text,' +
		  		'blank text' +
			')';
			
			dbUSDA_Portions = this.prepareDatabase(createUSDA_PortionsSQL, "USDA_Portions",  "USDA_Portions", "1.0", 5);
		},
		
		openUSDA_PA: function() {	
			//USDA_PA
			var createUSDA_PASQL = 'CREATE TABLE IF NOT EXISTS USDA_PA (' +
		  		'DB_ver text,' +
		  		'Compcode text,' +
		  		'METS text,' +
		  		'Cat_ID text,' +
		  		'Search_Cat text,' +
		  		'Desc text,' +
		  		'Sort_Order text,' +
		  		'Muscle_Str text,' +
		  		'Bone_Str text,' +
		  		'Vigorous text,' +
		  		'Cardio text,' +
		  		'Strength text,' +
		  		'Flexibility' +
			')';
		
			dbUSDA_PA = this.prepareDatabase(createUSDA_PASQL, "USDA_PA.sqlite3",  "USDA_PA", "1.0", 5);
		},
		
		// Open the Web SQL database
		prepareDatabase: function(createSQL, dbName, dbReadableName, dbVersion, dbSizeInMeg) {
		    var odb = this.getOpenDatabase();
		    if(!odb) {
		    	alert('Web SQL Not Supported');
		        dispError('Web SQL Not Supported');
		        return undefined;
		    } else {
		    	//alert('Web SQL Supported!');
		        var db = odb( dbName, dbVersion, dbReadableName, dbSizeInMeg * 1024 * 1024 );
		        db.transaction(function (t) {
		            t.executeSql( createSQL, [], function(t, r) {
		        	if (r.rowsAffected>0){
		        		alert('Create Table Successful: ' +dbReadableName);
		        	} else {
		        		alert('Table Already Exists: ' +dbReadableName);
		        	}
		        	
		        	}, function(t, e) {
		            	alert('Failed to create table: ' + dbReadableName + " : " + t.message);
		            }
		            );
		        });
		        return db;
		    }
		},
		
		findFood: function(food) {
			var sql = "select * from USDA_Foods where foodname = " + food;
			
		     dbUSDA_Foods.transaction(function(tx) {
		         tx.executeSql('SELECT * FROM USDA_Foods',
		                 [],
		                 function(tx, results)
		                 {
		                   for (var i=0; i<results.rows.length; i++) 
		                   { 
		                       var row=results.rows.item(i);
// 		                       console.log("row: " + row);
		                       alert("Food_ID: " + row['Food_ID']);
		                       var stringout = "Protein: " + row['Protein'] + "\n"; 
		                       alert(stringout); 
		                       stringout = "foodname: " + row['foodname'] + "\n"; 
		                       alert(stringout); 
		                   } 
		                 },                
		                 function(tx, error)
							{
								alert("Error: " + error.message);
							}
		                );
		     });
			
		}

		
	};
	return database;
});

/*
function dbSeed(db, insertSql, sqlFileName) {
    if(!db) {

    	openAppDatabases();
    	//alert ("db object empty");
    	//return;
    }
    var jqxhr = $.get(sqlFileName, function(sql) {
        var lines = sql.split("\n");
        for (var i = 0, len = lines.length-1; i < len; i++) {
            //alert("sql: "+lines[i].trim());
            persistDataToDB(db, insertSql, "(" + lines[i].trim() + ")"); 
        }
    });
}
*/
//Separated out persist to remove duplicate persist issue     
function persistDataToDB(db, insertSql, sqlTest){
	
    db.transaction( function(t) {
    	//t.executeSql(insertSql, lines[i]); //use to insert favorites
    	t.executeSql(insertSql+sqlTest); //works
    	//t.executeSql(insertSql+insertMultiple); //batch updates 
		}, 
		function(t, e){ 
			alert('Error Insert row: ' + t.message); 
		}, //error
		function() {
			//alert('Success Insert row: ' + i);
		}
	);
    
    //Stopped transaction after the first persist
	/*
	 .done(function() {
		alert( "insert success" );
	})
	.fail(function() {
		alert( "error" +jqxhr );
	})
	.always(function() {
		alert( "finished inserts" );
	}); 
	*/
}    
       

/*
// How many rows do we have?
function countRows(db, dbTableName) {
    if(!db) return;
    db.readTransaction(function (t) {
        t.executeSql('SELECT COUNT(*) AS c FROM ' + dbTableName, [], function (t, r) {
            var c = r.rows.item(0).c;
            //element('rowCount').innerHTML = c ? c : 0;
            alert(dbTableName + ' countRows : ' + c ? c : 0);
        }, function(t, e) {
            alert(dbTableName + ' countRows Error: ' + t.message);
        });
    });
}

function persistData(dateValue, type, value) {

	var dataRecord;
	var data;
	//clear out time
	dateValue.setHours(0);dateValue.setMinutes(0);dateValue.setSeconds(0);dateValue.setMilliseconds(0);
	//clear out timezone offset
	var strDateValue = dateValue.toString();
	var timeZoneIndex = strDateValue.indexOf("00:00:00")+8;
	var dateKey = strDateValue.substr(0,timeZoneIndex);

	if (value) {
		//localStorage.setItem(dateKey, value);
		data = localStorage[dateKey];
		if (typeof data === "undefined") {
			dataRecord = new Object();
		} else {
			dataRecord = JSON.parse(localStorage[dateKey]);
		}

		switch (type)
		{
			case "weight":
				dataRecord.weight = value;
				break;
			case "breakfast":
				dataRecord.breakfast = value;
				break;
			case "lunch":
				dataRecord.lunch = value;
				break;
			case "dinner":
				dataRecord.dinner = value;
				break;
			case "snack":
				dataRecord.snack = value;
				break;
			case "flexibility":
				dataRecord.flexibility = value;
				break;
			case "strength":
				dataRecord.strength = value;
				break;
			case "aerobic":
				dataRecord.aerobic = value;
				break;
			default:	
				break;
		}
		localStorage[dateKey] = JSON.stringify(dataRecord);
	}	
} // function persistData


function populateWeight() {

	var currentDate = new Date();
	//currentDate.setHours(0);currentDate.setMinutes(0);currentDate.setSeconds(0);currentDate.setMilliseconds(0);
	
	persistData(currentDate,"weight",190);
	persistData(currentDate,"breakfast",210);
	persistData(currentDate,"lunch",190);
	persistData(currentDate,"dinner",400);
	persistData(currentDate,"snack",1000);
	persistData(currentDate,"flexibility",40);
	persistData(currentDate,"strength",20);
	persistData(currentDate,"aerobic",90);
	currentDate.setDate(currentDate.getDate() - 1);
	persistData(currentDate,"weight",190);
	persistData(currentDate,"breakfast",220);
	persistData(currentDate,"lunch",180);
	persistData(currentDate,"dinner",390);
	persistData(currentDate,"snack",1200);
	persistData(currentDate,"flexibility",30);
	persistData(currentDate,"strength",30);
	persistData(currentDate,"aerobic",60);
	currentDate.setDate(currentDate.getDate() - 1);
	persistData(currentDate,"weight",192);
	persistData(currentDate,"breakfast",0); //test zero
	persistData(currentDate,"lunch",180);
	persistData(currentDate,"dinner",160);
	persistData(currentDate,"snack",900);
	persistData(currentDate,"flexibility",10);
	persistData(currentDate,"strength",0);
	persistData(currentDate,"aerobic",60);
	currentDate.setDate(currentDate.getDate() - 1);
	persistData(currentDate,"weight",194);
	//persistData(currentDate,"breakfast",100);  -test no breakfast
	persistData(currentDate,"lunch",180);
	persistData(currentDate,"dinner",360);
	persistData(currentDate,"snack",100);
	persistData(currentDate,"flexibility",0);
	persistData(currentDate,"strength",0);
	persistData(currentDate,"aerobic",65);
	currentDate.setDate(currentDate.getDate() - 1);
	persistData(currentDate,"weight",195);
	persistData(currentDate,"breakfast",100);
	persistData(currentDate,"lunch",180);
	persistData(currentDate,"dinner",360);
	persistData(currentDate,"snack",100);
	persistData(currentDate,"flexibility",10);
	persistData(currentDate,"strength",20);
	persistData(currentDate,"aerobic",35);
	currentDate.setDate(currentDate.getDate() - 1);
	persistData(currentDate,"weight",199);
	persistData(currentDate,"breakfast",100);  
	persistData(currentDate,"lunch",180);
	persistData(currentDate,"dinner",360);
	persistData(currentDate,"snack",100);
	persistData(currentDate,"flexibility",5);
	persistData(currentDate,"strength",10);
	persistData(currentDate,"aerobic",45);
	currentDate.setDate(currentDate.getDate() - 1);
	persistData(currentDate,"weight",204);
	persistData(currentDate,"breakfast",100);  
	persistData(currentDate,"lunch",180);
	persistData(currentDate,"dinner",360);
	persistData(currentDate,"snack",100);
	persistData(currentDate,"flexibility",5);
	persistData(currentDate,"strength",10);
	persistData(currentDate,"aerobic",45);
	currentDate.setDate(currentDate.getDate() - 7);
	persistData(currentDate,"weight",220);
	persistData(currentDate,"breakfast",200);
	persistData(currentDate,"lunch",180);
	persistData(currentDate,"dinner",360);
	persistData(currentDate,"snack",1500);
	persistData(currentDate,"flexibility",0);
	persistData(currentDate,"strength",0);
	persistData(currentDate,"aerobic",0);
	currentDate.setDate(currentDate.getDate() - 7);
	persistData(currentDate,"weight",229);
	persistData(currentDate,"breakfast",200);
	persistData(currentDate,"lunch",310);
	persistData(currentDate,"dinner",390);
	persistData(currentDate,"snack",200);
	persistData(currentDate,"flexibility",10);
	persistData(currentDate,"strength",0);
	persistData(currentDate,"aerobic",60);
	currentDate.setDate(currentDate.getDate() - 7);
	persistData(currentDate,"weight",240);
	persistData(currentDate,"breakfast",350);
	persistData(currentDate,"lunch",200);
	persistData(currentDate,"dinner",200);
	persistData(currentDate,"snack",200);
	persistData(currentDate,"flexibility",20);
	persistData(currentDate,"strength",0);
	persistData(currentDate,"aerobic",60);
	currentDate.setDate(currentDate.getDate() - 7);
	persistData(currentDate,"weight",250);
	persistData(currentDate,"breakfast",490);
	persistData(currentDate,"lunch",450);
	persistData(currentDate,"dinner",200);
	persistData(currentDate,"snack",200);
	persistData(currentDate,"flexibility",20);
	persistData(currentDate,"strength",120);
	persistData(currentDate,"aerobic",0);
	currentDate.setDate(currentDate.getDate() - 7);
	persistData(currentDate,"weight",250);
	persistData(currentDate,"breakfast",200);
	persistData(currentDate,"lunch",800);
	persistData(currentDate,"dinner",600);
	persistData(currentDate,"snack",1200);
	//persistData(currentDate,"flexibility",20);
	persistData(currentDate,"strength",0);
	persistData(currentDate,"aerobic",120);	
	currentDate.setMonth(currentDate.getMonth() - 6);
	persistData(currentDate,"weight",295);
	persistData(currentDate,"breakfast",400);
	persistData(currentDate,"lunch",1000);
	persistData(currentDate,"dinner",800);
	persistData(currentDate,"snack",1400);
	persistData(currentDate,"flexibility",20);
	persistData(currentDate,"strength",0);
	persistData(currentDate,"aerobic",12);	
	currentDate.setFullYear(currentDate.getFullYear() - 1);
	persistData(currentDate,"weight",320);
	persistData(currentDate,"breakfast",600);
	persistData(currentDate,"lunch",1000);
	persistData(currentDate,"dinner",800);
	persistData(currentDate,"snack",1400);
	persistData(currentDate,"flexibility",10);
	persistData(currentDate,"strength",0);
	persistData(currentDate,"aerobic",10);	
	
	alert("App test data has been populated");	
	
} //function populateWeight




var _activity;

$("#home").live('pageinit', function (evt)
{
	$("#1w").bind("tap", function(){identifyButton("1w");generate_report();});
	$("#1m").bind("tap", function(){identifyButton("1m");generate_report();});
	$("#3m").bind("tap", function(){identifyButton("3m");generate_report();});
	$("#6m").bind("tap", function(){identifyButton("6m");generate_report();});
	$("#1y").bind("tap", function(){identifyButton("1y");generate_report();});
	$("#2y").bind("tap", function(){identifyButton("2y");generate_report();});
	$("#weight").bind("tap", function(){identifyReport("weight");generate_report();});
	$("#diet").bind("tap", function(){identifyReport(app.report="diet");generate_report();});
	$("#activity").bind("tap", function(){identifyReport(app.report="activity");generate_report();});
	$("#populatetestdata").bind("tap", function(){populateWeight();});
	$("#clearalldata").bind("tap", function(){localStorage.clear();alert("All App data has been reset.");});
    $("#addweightdone").bind("tap", function(){
        var weightval = $("#weightinput").val();
        var date = new Date();      
        //var dateFormat = date.getFullYear() + "-" + ("0" + (date.getMonth() + 1)).slice(-2) + "-" + ("0" + date.getDate()).slice(-2) + " 12:00PM";
        //persistData(dateFormat, weightval);
        //date.setHours(0);date.setMinutes(0);date.setSeconds(0);date.setMilliseconds(0);
        persistData(date,"weight",weightval);
        identifyReport("weight");
    });
    // SJB
    $("#addheightdone").bind("tap", function(){
        var heightval = $("#heightinput").val();
        localStorage["height"] = heightval;
    });

    $("#addinitialdone").bind("tap", function(){
        var heightval = $("#heightinput").val();
        localStorage["height"] = heightval;
    });

    //$('div[data-role="dialog"]').live('pagebeforeshow', function(e, ui) {
	//ui.prevPage.addClass("ui-dialog-background ");
	//});
	
    //$('div[data-role="popup"]').live('pagebeforeshow', function(e, ui) {ui.prevPage.addClass("ui-dialog-background ");});
	//$('div[data-role="popup"]').live('pagehide', function(e, ui) {$(".ui-dialog-background ").removeClass("ui-dialog-background ");});
	//$("#displaydiet").live('pagebeforeshow', function(e, ui) {ui.prevPage.addClass("ui-dialog-background ");$('#weight').data('disabled',true);alert("disabled");});
	//$("#displaydiet").live('pagehide', function(e, ui) {$(".ui-dialog-background ").removeClass("ui-dialog-background ");$('#weight').data('disabled',false);alert("enabled");});
	//$("#home").live('pagehide', function(e, ui) {$("#weight").attr("disabled", true);alert("home disabled");});


	$("#previous").bind("tap", function(){if (app.dietChartPointIndex>0){app.dietChartPointIndex-=1;};displayDietDialogChart();});
	$("#next").bind("tap", function(){if (app.dietChartPointIndex<3){app.dietChartPointIndex+=1;};displayDietDialogChart();});
	$("#seeddb").bind("tap", function(){
		//USDA_Foods
		var insertSQL = 'INSERT INTO USDA_Foods( Action, Food_ID, foodcode, modcode, foodname, Protein, Total_Fat, Carbohydrate, Energy, Alcohol, Water, Caffeine, Theobromine, Sugars, Fiber, Calcium, Iron, Magnesium, Phosphorus, Potassium, Sodium, Zinc, Copper, Selenium ,Retinol, Vitamin_A, Carotene_beta, Carotene_alpha, Vitamin_E, Vitamin_D, ' + 
		 'Cryptoxanthin_beta, Lycopene, Lutin_zeaxanthin, Vitamin_C, Thiamin, Riboflavin, Niacin, Vitamin_B6, Folate_total, Vitamin_B12, Choline_total, Vitamin_K, Folic_acid, Folate_food, Folate_DFE, Vitamin_E_added, Vitamin_B12_added, Cholesterol, Fatty_acids_total_saturated, _4_0, _6_0, _8_0, _10_0, _12_0, _14_0, _16_0, _18_0, _18_1, _18_2, _18_3, ' + 
		 '_20_4, _22_6_n3, _16_1, _18_4, _20_1, _20_5_n3, _22_1, _22_5_n3, Fatty_acids_total_mono, Fatty_acids_total_poly) VALUES ';
		dbSeed(dbUSDA_Foods, insertSQL, "data/USDA_FoodsConverted.csv");
		alert("DB USDA_Foods Loads Initiated"); //test load took between 15-90 seconds
		
		insertSQL = 'INSERT INTO USDA_Portions( Action, Food_Item_ID, foodcode, modcode, foodname, portioncode, portioncode_addendum, portion_code, portionwgt, Portion_Factor, portiondesc, increment, default_portion, Use_as_default, Active, blank) VALUES ';
		dbSeed(dbUSDA_Portions, insertSQL, "data/USDA_PortionsConverted.csv");
		alert("DB USDA_Portions Loads Initiated"); //test load 60-90 seconds
		
		insertSQL = 'INSERT INTO USDA_PA( DB_ver, Compcode, METS, Cat_ID, Search_Cat, Desc, Sort_Order, Muscle_Str, Bone_Str, Vigorous, Cardio, Strength, Flexibility ) VALUES ';
		dbSeed(dbUSDA_PA, insertSQL, "data/USDA_PAConverted.csv");
		alert("DB USDA_PA Loads Initiated"); //test load 60-90 seconds
		
	});
});
*/


