define(function(require) {
    var log = require("loglevel"),
        Handlebars = require("handlebars"),
        TriggersAndFixesData = require("models/TriggersAndFixesModel"),
        template = require("text!templates/G9C2UnhealthyFoods.html");

    // Compile the template once.
    var compiled_template = Handlebars.compile(template);
    
    var isEditing = false;
    var selectedIndex;

    var G9C2UnhealthyFoods = Backbone.View.extend({
        // Set the id attribute on the div.
        id: "G9C2UnhealthyFoods",

        model: TriggersAndFixesData,

        initialize: function() {},

        events: {

            'change #unhealthyFoods': function(ev){this.addFood(ev);},
            'click ul#unhealthyFoodsListView li#deletableCells': function(ev){this.editFood(ev);},            

            'swipeleft ul#unhealthyFoodsListView li#deletableCells': function(ev){this.deleteFood(ev);},
            'swiperight ul#unhealthyFoodsListView li#deletableCells': function(ev){this.deleteFood(ev);}            
        },

        render: function() {
            $(this.el).html(compiled_template(TriggersAndFixesData.toJSON()));

            var self = this;
            var unhealthyFoods = TriggersAndFixesData.get ("unhealthyFoods");
            $.each( unhealthyFoods, function( key, value ) {
                $('<li id="deletableCells" style="white-space:normal">').append(value).appendTo(self.$('#unhealthyFoodsListView'));
            });

            return this;
        },

        addFood: function(ev) {
            setTimeout(function(){
            //this timeout gives the system the time it needs to remove the value if the user tapped the X button in the input field.       
      
                var value = this.$('#unhealthyFoods').val();
                
                if (value === "") return;
                
                var editedFoodIndex = -1;
                //if a food item has been edited, pass in the index of the edited food item and reset the object    
                if (this.$("#unhealthyFoodsListView li").hasClass("isEditing")) {                   
                    
                    var editedFood = this.$("#unhealthyFoodsListView li.isEditing");
               
                    editedFood.css({"font-style":"normal", "text-indent":"0px"});
                    editedFood.removeClass("isEditing");
                             
                    editedFoodIndex = editedFood.attr("data-index");          
                    editedFood.removeAttr("data-index");
                    editedFood.text(value);
                        
                 } else {
                    $('<li id="deletableCells" style="white-space:normal">').append(value).appendTo(this.$('#unhealthyFoodsListView'));     
                 }    
                    
                TriggersAndFixesData.addFood(value, editedFoodIndex);
                
                this.$('#unhealthyFoods').val('');                
                this.$('#unhealthyFoodsListView').listview("refresh");

            }, 100);

        }, 
        
        editFood: function(ev) {
          
           var alreadyEditingIndex = -1;          
           if (this.$("#unhealthyFoodsListView li").hasClass("isEditing")) {
                //remove the indicators from the current selection for editing so the new element can be set to edit mode
               var editedFood = this.$("#unhealthyFoodsListView li.isEditing");
               
               editedFood.css({"font-style":"normal", "text-indent":"0px"});
               editedFood.removeClass("isEditing");
               
               alreadyEditingIndex = editedFood.attr("data-index");
                             
               editedFood.removeAttr("data-index");
               this.$('#unhealthyFoods').val(''); 
           }
         
            selectedFoodIndex = $(ev.currentTarget).index() - 1;
                              
            if (alreadyEditingIndex == selectedFoodIndex) return;   
                  
            var currentFood = $(ev.currentTarget).text();
                          
            $(ev.currentTarget).css({"font-style":"italic", "text-indent":"10px"});
            $(ev.currentTarget).attr("data-index", selectedFoodIndex);
            $(ev.currentTarget).addClass("isEditing");
            
            this.$('#unhealthyFoods').focus(); 
            this.$('#unhealthyFoods').val(currentFood);                        
            
        },
        
        deleteFood: function(ev) {
                     
           var self = this;
            selectedFoodIndex = $(ev.currentTarget).index() - 1;
             
            function onConfirm(buttonIndex) {
                if (buttonIndex === 1) {
                    
                    TriggersAndFixesData.deleteFood(selectedFoodIndex);                              
                    $(ev.currentTarget).remove();
                }
            }            
            var titleString = "Are you sure you want to delete '" + $(ev.currentTarget).text() + "'?"; 
            
            navigator.notification.confirm(
                ' ', // message
                onConfirm,            // callback to invoke with index of button pressed
                titleString,           // title
                ['Yes','No']     // buttonLabels
            );                                       
        }
    });

    return G9C2UnhealthyFoods;
});


