define(function(require) {
    var log = require("loglevel"),
        Handlebars = require("handlebars"),
        Guides = require("collections/allGuides"),
        template = require("text!templates/guide-chapter.html");

    // Compile the template once.
    var compiled_template = Handlebars.compile(template);

    var GenericPageDisplayView = Backbone.View.extend({
        // Set the id attribute on the div.
        id: "genericPageDisplayView",

        events: {
            "click #next-button": "handleNextButton",
            "click #previous-button": "handlePreviousButton"
        },

        initialize: function(args) {

        	var pageNbr = Number (args.pageNumber);
			this.pageNumber = pageNbr;
        	this.type = args.type;

        	this.model = new Backbone.Model.extend;
			this.model.leftMargin = "10px";

        	if (args.type === "aboutGuides") {
        		this.model.pagesArray = ["../../content/guides/programWarning.html", "../../content/guides/guideInfo.html", "../../content/guides/guideSuccess.html", "../../content/guides/guidesGetStarted.html"];
        		this.model.backLink = "#guides";
        		this.model.chapter = "";   //set the title
              
        		if (Guides.get(1).get("status") == "lock") {
                    Guides.get(1).save({status: "carat-r"});
                    
                    var startDate = new Date();
                    var endDate = new Date();
                    if (isTesting) {
                    	endDate.setSeconds (startDate.getSeconds() + 30);
                    } else {                    
                    	endDate.setDate(startDate.getDate()+7);
                    }
                    Guides.get(1).save({ startDate:startDate});
                    Guides.get(1).save({ endDate:endDate});
 
        		}

        	} else if (args.type === "transcript") {

        		//get transcript to display from args
//        		this.model.transcriptURL = args.transcriptURL;
 				var transUrl = "../../content/videoTranscripts/" + args.transcriptURL;

        		this.model.pagesArray = [];
        		this.model.pagesArray.push (transUrl);
        		//not sure about backlink
        		this.model.chapter = "Transcript";
 				this.model.backlink = "#";

        	} else if (args.type === "G2C2NutritionLabelDetail") {

        		this.model.pagesArray = ["../../content/guides/guide2/G2C2NutritionLabel1_retina.html", "../../content/guides/guide2/G2C2NutritionLabel2_retina.html", "../../content/guides/guide2/G2C2NutritionLabel3_retina.html", "../../content/guides/guide2/G2C2NutritionLabel4_retina.html", "../../content/guides/guide2/G2C2NutritionLabel5_retina.html"];
        		titlesArray = ["Serving Size", "Amount of Calories", "Percent Daily Values", "Limit these Nutrients", "Get Enough of These Nutrients"];

        		this.model.backLink = "#G2C2NutritionLabelDetail/0";

        		this.model.chapter = titlesArray[this.pageNumber];

        	} else {
        		//load a different array for other content
        		console.log ("NO VALID TYPE WAS SENT TO GENERIC PAGE LOADER");
        	}
        	var pages = this.model.pagesArray;

            // Used to toggle display of previous/next buttons.
            this.model.isPrevious = false;
			this.model.isDone = true;

			if (pages.length === 1) {
                this.model.label = "Done";

                // Since there's only 1 chapter in this guide, set next to go back to the main guides page.
                this.model.nextHref = this.model.backLink;
                //this should transition in reverse

            } else {

            	if (this.pageNumber + 1 === pages.length) {
                	this.model.label = "Done";

                	// This is the last page in this chapter so set next to go back to the current guide page. SJB changed comment that said go to main guides page
                	this.model.nextHref = this.model.backLink;
            	} else {
            		this.model.label = "Next";
            		this.model.isDone = false;
                	// There are more chapters so set next to go to the next chapter.
                	this.model.nextHref = "#" + this.type + "/" + (this.pageNumber + 1);
            	}

                // Don't display the previous button on the 1st chapter.
                if (this.pageNumber !== 0) {
                    this.model.isPrevious = true;
                    this.model.previousHref = "#" + this.type + "/" + (this.pageNumber - 1);
                }
            }
            this.model.footerHeading = (pages.length === 1) ? "1 of 1" : (this.pageNumber + 1) + " of " + pages.length;
        },

        render: function() {
            var deferred = $.Deferred();
            var self = this;

//Currently this assumes all content is simple HTML pages
			contentTemplate = this.model.pagesArray[this.pageNumber];
            require(["text!" + contentTemplate], function(template) {
                // Jus set the content attribute to the template. We could compile the template
                // if needed. This is the same as a partial template.
                self.model.content = template;
                deferred.resolve();
            });

            $.when(deferred).done(function() {
                $(self.el).html(compiled_template(self.model));

                // Tell JQM to enhance the page since we missed the intial event.
                $(self.el).trigger("create");
            });

            return this;
        },

		handlePreviousButton: function() {
			$.mobile.changePage.defaults.transition = "flip";
			$.mobile.changePage.defaults.reverse = true;
			mainRouter.navigate(this.model.previousHref, {
                trigger: true, replace:true
            });
		},

        handleNextButton: function(e) {
        	$.mobile.changePage.defaults.transition = "flip";
        	$.mobile.changePage.defaults.reverse = false;
          	mainRouter.navigate(this.model.nextHref, {
            	    trigger: true, replace:true
            });
        }
    });

    return GenericPageDisplayView;
});
