define(function(require) {
    var log = require("loglevel"),
        Handlebars = require("handlebars"),
        GuidesCollection = require("collections/allGuides"),
        Summary = require("models/GuideSummaryModel"),
        template = require("text!templates/guides.html");

    // Compile the template once.
    var compiled_template = Handlebars.compile(template);

//TODO:
/*
 * Catch the onClick event for each guide tap, with the guideId.  
 * 
 * Get the Guide and check the status
 * 
 * either call the router to load, or call the method below to show the error
 * 
 * OR
 * 
 * todo: DO THIS WAY
 * Move error handling to SingelGuideView.js
 * 
 * as soon as guide renders, if locked, show the error
 * 
 * when user clicks ok, go back to guides.  Remember to set transitions
 * 
 */



    // The view for all the self-management guides
    var AllGuidesView = Backbone.View.extend({
        // Set the id attribute on the div.
        id: "allGuidesView",
		
		initialize: function() {
			
			var self = this;
			GuidesCollection.each (function(guide) {
					
				var status = guide.get("status");
				if (status === "carat-r") {
					self.isGuideComplete(guide.id);
				}
			}); 
		},
		
        render: function() {

            this.$el.html(compiled_template(GuidesCollection.toJSON()));
            return this;
        },
        
        /*
         * This method detemines if an active guide can be made complete.  If the active guide can be made compelete, the status is changed, the end date is updated and a summary object is created.  Additionaly, the next guide (if it exists) is made active.
         * 
         * @param: id - The ID of the currently active guide.  Because this guide is active, it is assumed that it has a valid end date.
         * 
         * This method is called when this view initializes.  Whenever a guide is loaded in the router class, an allGuidesView object is initialized so the router will have the latest status data for each guide.
         * 
         */
        isGuideComplete: function (id) {
        	           	        	                    
            var endDate = new Date(GuidesCollection.get(id).get("endDate"));
            var now = new Date();
                    
            if (endDate < now) {  //This guide can be set to complete.  Create a snapshot (GuideSummaryObject) and save to the guide.guideSummariesArray
                    	
            	GuidesCollection.get(id).save("status", "check");
                //For accuracy, save the actual date (now) that this guide is ended, not the startDate+duration.  
                GuidesCollection.get(id).save("endDate", now);
                    	
                // save summary here
               	var summaries = GuidesCollection.get(id).get("summaries");
	            var summary = new Summary(id, now);
	            summaries.unshift(summary.toJSON());
                GuidesCollection.get(id).save("summaries", summaries);
                    	
                var nextId = Number(id) + 1;
                if (nextId < 12) {
                	var duration = this.getGuideDuration(nextId);
                    		                    		
                    var nextEndDate = new Date();
                    if (isTesting) {
                    	nextEndDate.setSeconds (now.getSeconds() + (30 * duration));
                    } else {                    
                   		nextEndDate.setDate(now.getDate() + (7 * duration));
                	}
                    		                    		
                    GuidesCollection.get(nextId).save("status", "carat-r");
                  	GuidesCollection.get(nextId).save("startDate", now);
                    GuidesCollection.get(nextId).save("endDate", nextEndDate);
            	}                    	
          	}
        },
        
        getGuideDuration: function (id) {
        	
        	var duration = 1;
        	if (id > 3) {
        		duration = 2;
        	}
        	return duration;
        }
 
    });
    
    return AllGuidesView;
});
