define(function(require) {
    var log = require("loglevel"),
        Handlebars = require("handlebars"),
        UserData = require("models/UserData"),
        SurveyData = require("models/SurveyData"),
        Summary = require("models/GuideSummaryModel"),
        Guides = require("collections/allGuides"),
        WeightsCollection = require('collections/weightsCollection'),
        template = require("text!templates/main-settings.html");

    // Compile the template once.
    var compiled_template = Handlebars.compile(template);

    // The view for all the flowers
    var SettingsView = Backbone.View.extend({
        // Set the id attribute on the div.
        id: "allSettingsView",

        events: {
        	'click #unlockGuides': 'unlockGuides',
            'click #deleteData': 'clearAllData',
            'click #test-data-button': 'populateTestData',
            'change #test-switch': 'saveSettingsData' // add more events with this function if necessary
        },

        render: function() {
            
            $(this.el).html(compiled_template(UserData.toJSON()));
            return this;
        },

        clearAllData: function() {
            // DON'T DO THIS!!!!
            // LocalStorage is shared between all apps.
            //localStorage.clear();

            // Clear the local UserData object...
            UserData.clear();
			SurveyData.clear();
			Guides.clear(); // Guides saves themselves
            // Save changes to local storage.
            UserData.save();
            SurveyData.save();
            
            var weightsCollection = new WeightsCollection();
            weightsCollection.clear();
            
            alert("All App data has been reset.");
		},

		saveSettingsData: function() {
			var ts = $("#test-switch").val();
			console.log("test-switch val: " + ts );

			var sel = (ts === "on") ? "selected" : "";
			UserData.set("testingConfig", sel);
			UserData.save();

        },

        populateTestData: function() {
            this.$('#test-data-popup').popup({
                afterclose: function(event) {
                                        
                    populateTestWeights();                     
                    
                }
            });

            this.$("#test-data-popup #ok-button").one('click', function() {
                // Save test data to local storage by creating a UserData object with default data.
                UserData.reset();
                $('#test-data-popup').popup('close');
            });

            // Handle 'cancel' button click.
            this.$("#test-data-popup #cancel-button").one('click', function() {
                $('#test-data-popup').popup('close');
            });

            this.$('#test-data-popup').popup('open', {
                positionTo: 'window'
            });
 
            function getRandomInt(min, max) {
                return Math.floor(Math.random() * (max - min + 1)) + min;
            }
            
            function populateTestWeights() {
                console.log ("Populating Test Weights");
                
                var weightsCollection = new WeightsCollection();                                
                weightsCollection.clear();
                /*
                 * Edit this value to change how far back in time weights are populated 1460 = 4 years
                 */ 
                var daysBack = 100;
                                
                var millisecondsInDay = 86400000;
                var today = new Date();
                 
                var currentDateUTC = Date.UTC(today.getFullYear(0), today.getMonth(), today.getDate());
                
                var startingWeight = getRandomInt(250, 450);
                var startingDate  = new Date(currentDateUTC - (millisecondsInDay * daysBack));
                
                var randomSkipDay = Math.floor(getRandomInt(startingWeight/10, startingWeight/6));
                var randomMultipleEntryDay = Math.floor(getRandomInt (startingWeight/30, startingWeight/20));                                
                
                //seed                
                var weight = startingWeight;
                
                for (var i=daysBack; i>=-1; i--) {
                    var randomWeightDirection = getRandomInt(0, 1);
                    var randomWeightLossGain = getRandomInt (0, 5);
                                                            
                    if (randomWeightDirection == 1) {
                        randomWeightLossGain = randomWeightLossGain * -1;        
                    }
                     
                    if (i % randomSkipDay == 0 && i != 0) {
                       //nop console.log ("******************Skipping a day at day: " + i);
                    } else {
                        weight += randomWeightLossGain;
                        var date = new Date(currentDateUTC - (millisecondsInDay * i));
                                                
                        weightsCollection.create ({weight:weight, date:date});
                        
                        //compute multiple entries for this date if
                        if (i % randomMultipleEntryDay == 0 && i != 0) {
                                                    
                            for (var j=0; j<=getRandomInt(1,4); j++) {
                                 
                                var extraDate = date.setMinutes(date.getMinutes() + getRandomInt(1, 100));
                                var extraWeight = getRandomInt (0,1);
                                if (getRandomInt (0,2) > 0) {
                                    weight += extraWeight * -1;
                                }
                            
                                weightsCollection.create ({weight:weight, date:extraDate});       
                            }                            
                        }                        
                    }
                }
                
                console.log ("\n\n******************");
                console.log ("Starting weight: " + startingWeight);
                console.log ("Ending Weight  : " + weight);
                console.log ("populated dates from " + startingDate + " to " + today);
                console.log ("***********************\n\n");
                
                
            }
            
        },
        
        unlockGuides: function() {
    		console.log ("UNLOCK THE GUIDES");
    	
    		//we can flesh this out to go thru and set the start and end dates to actual real dates
    		var now = new Date();
    		var summary, summaries;

    		Guides.each (function(guide) {
    			guide.save("status", "check");
				// create a summary for each guide
             	summaries = guide.get("summaries");
	            summary = new Summary(guide.id, now);
	            summaries.unshift(summary.toJSON());
                guide.save("summaries", summaries);
    		});
    	
    	}
    });



    return SettingsView;
});
